@@ -1,14 +1,14 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice-E is licensed to you under the terms described in the
 # ICEE_LICENSE file included in this distribution.
 #
 # **********************************************************************
 
-import fileinput, re
+import fileinput, re, string
 
 previous = ""
 
@@ -18,6 +18,8 @@ for line in fileinput.input():
     if re.compile("^#").search(line, 0):
         continue;
 
+    line = string.replace(line, ".o:", "$(OBJEXT):")
+
     if(previous):
         line = previous + " " + line
 
