@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,9 +11,9 @@
 
 using namespace std;
 
-SessionPingThread::SessionPingThread(const Glacier2::SessionPrx& session) :
+SessionPingThread::SessionPingThread(const Glacier2::SessionPrx& session, long timeout) :
     _session(session),
-    _timeout(IceUtil::Time::seconds(20)),
+    _timeout(IceUtil::Time::seconds(timeout)),
     _destroy(false)
 {
 }
