@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -10,15 +10,14 @@
 #ifndef CHAT_PING_THREAD_H
 #define CHAT_PING_THREAD_H
 
-#include <IceE/Thread.h>
-#include <IceE/Monitor.h>
+#include <IceE/IceE.h>
 #include <Router.h>
 
 class SessionPingThread : public IceUtil::Thread, public IceUtil::Monitor<IceUtil::Mutex>
 {
 public:
 
-    SessionPingThread(const Glacier2::SessionPrx& session);
+    SessionPingThread(const Glacier2::SessionPrx& session, long timeout);
 
     virtual void run();
     void destroy();
