@@ -1,20 +1,24 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
-#include <IceE/IceE.h>
 #include <HelloI.h>
+#include <IceE/IceE.h>
 
 using namespace std;
 
 void
-HelloI::sayHello(const Ice::Current& c) const
+HelloI::sayHello(int delay, const Ice::Current&) const
 {
+    if(delay != 0)
+    {
+        IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(delay));
+    }
     printf("Hello World!\n");
 }
 
