@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -12,11 +12,11 @@
 
 #include <Hello.h>
 
-class HelloI : public ::Demo::Hello
+class HelloI : public Demo::Hello
 {
 public:
 
-    virtual void sayHello(const Ice::Current&) const;
+    virtual void sayHello(int delay, const Ice::Current&) const;
     virtual void shutdown(const Ice::Current&);
 };
 
