@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,19 +11,27 @@
 #define ICEE_COMMUNICATOR_F_H
 
 #include <IceE/Handle.h>
+#include <IceE/Shared.h>
 
 namespace Ice
 {
 
 class Communicator;
+inline bool operator==(const Communicator& l, const Communicator& r)
+{
+    return &l == &r;
+}
+inline bool operator<(const Communicator& l, const Communicator& r)
+{
+    return &l < &r;
+}
 
 }
 
 namespace IceInternal
 {
 
-ICE_API void incRef(::Ice::Communicator*);
-ICE_API void decRef(::Ice::Communicator*);
+ICE_API IceUtil::Shared* upCast(::Ice::Communicator*);
 
 }
 
