@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,22 +11,27 @@
 #define ICEE_CONNECTION_F_H
 
 #include <IceE/Handle.h>
+#include <IceE/Shared.h>
 
 namespace Ice
 {
 
 class Connection;
-ICE_API bool operator==(const Connection&, const Connection&);
-ICE_API bool operator!=(const Connection&, const Connection&);
-ICE_API bool operator<(const Connection&, const Connection&);
+inline bool operator==(const Connection& l, const Connection& r)
+{
+    return &l == &r;
+}
+inline bool operator<(const Connection& l, const Connection& r)
+{
+    return &l < &r;
+}
 
 }
 
 namespace IceInternal
 {
 
-ICE_API void incRef(::Ice::Connection*);
-ICE_API void decRef(::Ice::Connection*);
+ICE_API IceUtil::Shared* upCast(::Ice::Connection*);
 
 }
 
