@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -19,10 +19,18 @@
 #ifndef ICEE_PURE_CLIENT
 #   include <IceE/ObjectAdapter.h>
 #   include <IceE/Object.h>
+#   include <IceE/UUID.h>
 #endif
 #include <IceE/Proxy.h>
-#include <IceE/IdentityUtil.h>
 #include <IceE/Connection.h>
 #include <IceE/Functional.h>
+#ifdef ICEE_HAS_WSTRING
+#   include <IceE/Unicode.h>
+#endif
+
+#include <IceE/Thread.h>
+#include <IceE/Monitor.h>
+#include <IceE/Mutex.h>
+#include <IceE/SafeStdio.h>
 
 #endif
