@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -28,7 +28,7 @@ public:
     void setAdapter(const Ice::ObjectAdapterPtr&);
     const Ice::ObjectAdapterPtr& getAdapter() const { return _adapter; }
 
-    void invoke(bool);
+    void invoke(bool, Ice::Int);
 
     // Inlined for speed optimization.
     BasicStream* os() { return &_os; }
