@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -23,8 +23,9 @@ public:
 
     AlreadyRegisteredException(const char*, int);
     AlreadyRegisteredException(const char*, int, const ::std::string&, const ::std::string&);
+    virtual ~AlreadyRegisteredException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -39,8 +40,9 @@ public:
 
     NotRegisteredException(const char*, int);
     NotRegisteredException(const char*, int, const ::std::string&, const ::std::string&);
+    virtual ~NotRegisteredException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -55,8 +57,9 @@ public:
 
     TwowayOnlyException(const char*, int);
     TwowayOnlyException(const char*, int, const ::std::string&);
+    virtual ~TwowayOnlyException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -69,8 +72,9 @@ class ICE_API CloneNotImplementedException : public ::Ice::LocalException
 public:
 
     CloneNotImplementedException(const char*, int);
+    virtual ~CloneNotImplementedException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -82,8 +86,9 @@ public:
 
     UnknownException(const char*, int);
     UnknownException(const char*, int, const ::std::string&);
+    virtual ~UnknownException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -97,8 +102,9 @@ public:
 
     UnknownLocalException(const char*, int);
     UnknownLocalException(const char*, int, const ::std::string&);
+    virtual ~UnknownLocalException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -110,8 +116,9 @@ public:
 
     UnknownUserException(const char*, int);
     UnknownUserException(const char*, int, const ::std::string&);
+    virtual ~UnknownUserException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -122,8 +129,9 @@ class ICE_API VersionMismatchException : public ::Ice::LocalException
 public:
 
     VersionMismatchException(const char*, int);
+    virtual ~VersionMismatchException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -134,8 +142,9 @@ class ICE_API CommunicatorDestroyedException : public ::Ice::LocalException
 public:
 
     CommunicatorDestroyedException(const char*, int);
+    virtual ~CommunicatorDestroyedException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -149,8 +158,9 @@ public:
 
     ObjectAdapterDeactivatedException(const char*, int);
     ObjectAdapterDeactivatedException(const char*, int, const ::std::string&);
+    virtual ~ObjectAdapterDeactivatedException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -164,8 +174,9 @@ public:
 
     ObjectAdapterIdInUseException(const char*, int);
     ObjectAdapterIdInUseException(const char*, int, const ::std::string&);
+    virtual ~ObjectAdapterIdInUseException()throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -180,8 +191,9 @@ public:
 
     NoEndpointException(const char*, int);
     NoEndpointException(const char*, int, const ::std::string&);
+    virtual ~NoEndpointException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -195,8 +207,9 @@ public:
 
     EndpointParseException(const char*, int);
     EndpointParseException(const char*, int, const ::std::string&);
+    virtual ~EndpointParseException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -210,8 +223,9 @@ public:
 
     IdentityParseException(const char*, int);
     IdentityParseException(const char*, int, const ::std::string&);
+    virtual ~IdentityParseException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -225,8 +239,9 @@ public:
 
     ProxyParseException(const char*, int);
     ProxyParseException(const char*, int, const ::std::string&);
+    virtual ~ProxyParseException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -240,8 +255,9 @@ public:
 
     IllegalIdentityException(const char*, int);
     IllegalIdentityException(const char*, int, const ::Ice::Identity&);
+    virtual ~IllegalIdentityException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -255,8 +271,9 @@ public:
 
     RequestFailedException(const char*, int);
     RequestFailedException(const char*, int, const ::Ice::Identity&, const ::std::string&, const ::std::string&);
+    virtual ~RequestFailedException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -272,8 +289,9 @@ public:
 
     ObjectNotExistException(const char*, int);
     ObjectNotExistException(const char*, int, const ::Ice::Identity&, const ::std::string&, const ::std::string&);
+    virtual ~ObjectNotExistException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -285,8 +303,9 @@ public:
 
     FacetNotExistException(const char*, int);
     FacetNotExistException(const char*, int, const ::Ice::Identity&, const ::std::string&, const ::std::string&);
+    virtual ~FacetNotExistException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -298,8 +317,9 @@ public:
 
     OperationNotExistException(const char*, int);
     OperationNotExistException(const char*, int, const ::Ice::Identity&, const ::std::string&, const ::std::string&);
+    virtual ~OperationNotExistException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -311,8 +331,9 @@ public:
 
     SyscallException(const char*, int);
     SyscallException(const char*, int, ::Ice::Int);
+    virtual ~SyscallException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -326,8 +347,9 @@ public:
 
     SocketException(const char*, int);
     SocketException(const char*, int, ::Ice::Int);
+    virtual ~SocketException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -339,8 +361,9 @@ public:
 
     FileException(const char*, int);
     FileException(const char*, int, ::Ice::Int, const ::std::string&);
+    virtual ~FileException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -354,8 +377,9 @@ public:
 
     ConnectFailedException(const char*, int);
     ConnectFailedException(const char*, int, ::Ice::Int);
+    virtual ~ConnectFailedException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -367,8 +391,9 @@ public:
 
     ConnectionRefusedException(const char*, int);
     ConnectionRefusedException(const char*, int, ::Ice::Int);
+    virtual ~ConnectionRefusedException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -380,8 +405,9 @@ public:
 
     ConnectionLostException(const char*, int);
     ConnectionLostException(const char*, int, ::Ice::Int);
+    virtual ~ConnectionLostException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -393,8 +419,9 @@ public:
 
     DNSException(const char*, int);
     DNSException(const char*, int, ::Ice::Int, const ::std::string&);
+    virtual ~DNSException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -408,8 +435,9 @@ class ICE_API TimeoutException : public ::Ice::LocalException
 public:
 
     TimeoutException(const char*, int);
+    virtual ~TimeoutException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -420,8 +448,9 @@ class ICE_API ConnectTimeoutException : public ::Ice::TimeoutException
 public:
 
     ConnectTimeoutException(const char*, int);
+    virtual ~ConnectTimeoutException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -432,8 +461,9 @@ class ICE_API CloseTimeoutException : public ::Ice::TimeoutException
 public:
 
     CloseTimeoutException(const char*, int);
+    virtual ~CloseTimeoutException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -444,110 +474,15 @@ class ICE_API ProtocolException : public ::Ice::LocalException
 public:
 
     ProtocolException(const char*, int);
+    ProtocolException(const char*, int, const ::std::string&);
+    virtual ~ProtocolException() throw();
 
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
-};
-
-class ICE_API BadMagicException : public ::Ice::ProtocolException
-{
-public:
-
-    BadMagicException(const char*, int);
-    BadMagicException(const char*, int, const ::Ice::ByteSeq&);
-
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
 
-    ::Ice::ByteSeq badMagic;
-};
-
-class ICE_API UnsupportedProtocolException : public ::Ice::ProtocolException
-{
-public:
-
-    UnsupportedProtocolException(const char*, int);
-    UnsupportedProtocolException(const char*, int, ::Ice::Int, ::Ice::Int, ::Ice::Int, ::Ice::Int);
-
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
-
-    ::Ice::Int badMajor;
-    ::Ice::Int badMinor;
-    ::Ice::Int major;
-    ::Ice::Int minor;
-};
-
-class ICE_API UnsupportedEncodingException : public ::Ice::ProtocolException
-{
-public:
-
-    UnsupportedEncodingException(const char*, int);
-    UnsupportedEncodingException(const char*, int, ::Ice::Int, ::Ice::Int, ::Ice::Int, ::Ice::Int);
-
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
-
-    ::Ice::Int badMajor;
-    ::Ice::Int badMinor;
-    ::Ice::Int major;
-    ::Ice::Int minor;
-};
-
-class ICE_API UnknownMessageException : public ::Ice::ProtocolException
-{
-public:
-
-    UnknownMessageException(const char*, int);
-
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
-};
-
-class ICE_API ConnectionNotValidatedException : public ::Ice::ProtocolException
-{
-public:
-
-    ConnectionNotValidatedException(const char*, int);
-
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
-};
-
-class ICE_API UnknownRequestIdException : public ::Ice::ProtocolException
-{
-public:
-
-    UnknownRequestIdException(const char*, int);
-
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
-};
-
-class ICE_API UnknownReplyStatusException : public ::Ice::ProtocolException
-{
-public:
-
-    UnknownReplyStatusException(const char*, int);
-
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
+    ::std::string reason;
 };
 
 class ICE_API CloseConnectionException : public ::Ice::ProtocolException
@@ -555,8 +490,9 @@ class ICE_API CloseConnectionException : public ::Ice::ProtocolException
 public:
 
     CloseConnectionException(const char*, int);
+    virtual ~CloseConnectionException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -567,38 +503,42 @@ class ICE_API ForcedCloseConnectionException : public ::Ice::ProtocolException
 public:
 
     ForcedCloseConnectionException(const char*, int);
+    virtual ~ForcedCloseConnectionException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
 };
 
-class ICE_API IllegalMessageSizeException : public ::Ice::ProtocolException
+ICE_API void throwBadMagicException(const char*, int, const ::Ice::ByteSeq&);
+ICE_API void throwUnsupportedProtocolException(const char*, int, ::Ice::Int, ::Ice::Int, ::Ice::Int, ::Ice::Int);
+ICE_API void throwUnsupportedEncodingException(const char*, int, ::Ice::Int, ::Ice::Int, ::Ice::Int, ::Ice::Int);
+
+inline void throwUnknownMessageException(const char* file, int line)
 {
-public:
+    throw ProtocolException(file, line, "unknown message type");
+}
 
-    IllegalMessageSizeException(const char*, int);
-
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
-};
-
-class ICE_API FeatureNotSupportedException : public ::Ice::LocalException
+inline void throwConnectionNotValidatedException(const char* file, int line)
 {
-public:
+    throw ProtocolException(file, line, "received message over unvalidated connection");
+}
 
-    FeatureNotSupportedException(const char*, int);
+inline void throwUnknownRequestIdException(const char* file, int line)
+{
+    throw ProtocolException(file, line, "unknown request id");
+}
 
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
+inline void throwUnknownReplyStatusException(const char* file, int line)
+{
+    throw ProtocolException(file, line, "unknown reply status");
+}
 
-    ::std::string unsupportedFeature;
-};
+inline void throwIllegalMessageSizeException(const char* file, int line)
+{
+    throw ProtocolException(file, line, "illegal message size");
+}
 
 class ICE_API MarshalException : public ::Ice::ProtocolException
 {
@@ -606,36 +546,9 @@ public:
 
     MarshalException(const char*, int);
     MarshalException(const char*, int, const ::std::string&);
+    virtual ~MarshalException() throw();
 
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
-
-    ::std::string reason;
-};
-
-class ICE_API ProxyUnmarshalException : public ::Ice::MarshalException
-{
-public:
-
-    ProxyUnmarshalException(const char*, int);
-    ProxyUnmarshalException(const char*, int, const ::std::string&);
-
-    virtual const ::std::string ice_name() const;
-    virtual ::std::string  toString() const;
-    virtual ::Ice::Exception* ice_clone() const;
-    virtual void ice_throw() const;
-};
-
-class ICE_API UnmarshalOutOfBoundsException : public ::Ice::MarshalException
-{
-public:
-
-    UnmarshalOutOfBoundsException(const char*, int);
-    UnmarshalOutOfBoundsException(const char*, int, const ::std::string&);
-
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
@@ -647,38 +560,67 @@ public:
 
     MemoryLimitException(const char*, int);
     MemoryLimitException(const char*, int, const ::std::string&);
+    virtual ~MemoryLimitException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
 };
 
-class ICE_API EncapsulationException : public ::Ice::MarshalException
+ICE_API void throwMemoryLimitException(const char*, int);
+ICE_API void throwUnmarshalOutOfBoundsException(const char*, int);
+ICE_API void throwNegativeSizeException(const char*, int);
+
+inline void throwProxyUnmarshalException(const char* file, int line)
+{     
+    throw MarshalException(file, line, "inconsistent proxy data during unmarshaling");
+}
+
+class ICE_API FeatureNotSupportedException : public ::Ice::LocalException
 {
 public:
 
-    EncapsulationException(const char*, int);
-    EncapsulationException(const char*, int, const ::std::string&);
+    FeatureNotSupportedException(const char*, int);
+    FeatureNotSupportedException(const char*, int, const std::string&);
+    virtual ~FeatureNotSupportedException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
+
+    ::std::string unsupportedFeature;
 };
 
-class ICE_API NegativeSizeException : public ::Ice::MarshalException
+class ICE_API FixedProxyException : public ::Ice::LocalException
 {
 public:
 
-    NegativeSizeException(const char*, int);
-    NegativeSizeException(const char*, int, const ::std::string&);
+    FixedProxyException(const char*, int);
+    virtual ~FixedProxyException() throw();
 
-    virtual const ::std::string ice_name() const;
+    virtual ::std::string ice_name() const;
+    virtual ::std::string toString() const;
+    virtual ::Ice::Exception* ice_clone() const;
+    virtual void ice_throw() const;
+};
+
+#ifdef ICEE_HAS_WSTRING
+class ICE_API StringConversionException : public ::Ice::MarshalException
+{
+public:
+
+    StringConversionException(const char*, int);
+    StringConversionException(const char*, int, const ::std::string&); 
+    virtual ~StringConversionException() throw();
+
+    virtual ::std::string ice_name() const;
     virtual ::std::string  toString() const;
     virtual ::Ice::Exception* ice_clone() const;
     virtual void ice_throw() const;
 };
+#endif
 
 }
 
