@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -15,23 +15,21 @@
 #ifdef ICEE_HAS_LOCATOR
 
 #include <IceE/Handle.h>
+#include <IceE/Shared.h>
 
 namespace IceInternal
 {
 
 class LocatorManager;
-void incRef(LocatorManager*);
-void decRef(LocatorManager*);
+IceUtil::Shared* upCast(LocatorManager*);
 typedef Handle<LocatorManager> LocatorManagerPtr;
 
 class LocatorInfo;
-void incRef(LocatorInfo*);
-void decRef(LocatorInfo*);
+IceUtil::Shared* upCast(LocatorInfo*);
 typedef Handle<LocatorInfo> LocatorInfoPtr;
 
 class LocatorTable;
-void incRef(LocatorTable*);
-void decRef(LocatorTable*);
+IceUtil::Shared* upCast(LocatorTable*);
 typedef Handle<LocatorTable> LocatorTablePtr;
 
 }
