@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,22 +11,27 @@
 #define ICEE_OBJECT_ADAPTER_F_H
 
 #include <IceE/Handle.h>
+#include <IceE/Shared.h>
 
 namespace Ice
 {
 
 class ObjectAdapter;
-ICE_API bool operator==(const ObjectAdapter&, const ObjectAdapter&);
-ICE_API bool operator!=(const ObjectAdapter&, const ObjectAdapter&);
-ICE_API bool operator<(const ObjectAdapter&, const ObjectAdapter&);
+inline bool operator==(const ObjectAdapter& l, const ObjectAdapter& r)
+{
+    return &l == &r;
+}
+inline bool operator<(const ObjectAdapter& l, const ObjectAdapter& r)
+{
+    return &l < &r;
+}
 
 }
 
 namespace IceInternal
 {
 
-ICE_API void incRef(::Ice::ObjectAdapter*);
-ICE_API void decRef(::Ice::ObjectAdapter*);
+ICE_API IceUtil::Shared* upCast(::Ice::ObjectAdapter*);
 
 }
 
