@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,6 +11,7 @@
 #define ICEE_OBJECT_F_H
 
 #include <IceE/Handle.h>
+#include <IceE/Shared.h>
 
 namespace Ice
 {
@@ -22,8 +23,7 @@ class Object;
 namespace IceInternal
 {
 
-ICE_API void incRef(::Ice::Object*);
-ICE_API void decRef(::Ice::Object*);
+ICE_API IceUtil::Shared* upCast(::Ice::Object*);
 
 }
 
