@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -15,18 +15,17 @@
 #ifdef ICEE_HAS_ROUTER
 
 #include <IceE/Handle.h>
+#include <IceE/Shared.h>
 
 namespace IceInternal
 {
 
 class RouterManager;
-void incRef(RouterManager*);
-void decRef(RouterManager*);
+IceUtil::Shared* upCast(RouterManager*);
 typedef Handle<RouterManager> RouterManagerPtr;
 
 class RouterInfo;
-void incRef(RouterInfo*);
-void decRef(RouterInfo*);
+IceUtil::Shared* upCast(RouterInfo*);
 typedef Handle<RouterInfo> RouterInfoPtr;
 
 }
