@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,13 +11,13 @@
 #define ICEE_SERVANT_MANAGER_F_H
 
 #include <IceE/Handle.h>
+#include <IceE/Shared.h>
 
 namespace IceInternal
 {
 
 class ServantManager;
-void incRef(ServantManager*);
-void decRef(ServantManager*);
+IceUtil::Shared* upCast(ServantManager*);
 typedef Handle<ServantManager> ServantManagerPtr;
 
 }
