@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -20,7 +20,7 @@ class ICE_API UUIDGenerationException : public Exception
 public:
     
     UUIDGenerationException(const char*, int);
-    virtual const std::string ice_name() const;
+    virtual std::string ice_name() const;
     virtual Exception* ice_clone() const;
     virtual void ice_throw() const;
 
