@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -20,12 +20,9 @@
 // Here, we undefine symbols that cause such problems.
 //
 
-#ifdef major
 #undef major
-#endif
-
-#ifdef minor
 #undef minor
-#endif
+#undef min
+#undef max
 
 #endif
