@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -31,7 +31,7 @@ namespace IceInternal
 
 class Endpoint;
 
-class ICE_API Acceptor : public ::IceUtil::Shared
+class Acceptor : public ::IceUtil::Shared
 {
 public:
 
