@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,13 +11,13 @@
 #define ICEE_ACCEPTOR_F_H
 
 #include <IceE/Handle.h>
+#include <IceE/Shared.h>
 
 namespace IceInternal
 {
 
 class Acceptor;
-ICE_API void incRef(Acceptor*);
-ICE_API void decRef(Acceptor*);
+IceUtil::Shared* upCast(Acceptor*);
 typedef Handle<Acceptor> AcceptorPtr;
 
 }
