@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -8,6 +8,7 @@
 // **********************************************************************
 
 #include <IceE/DisableWarnings.h>
+#include <IceE/Properties.h>
 #include <IceE/BasicStream.h>
 #include <IceE/Instance.h>
 #include <IceE/ProxyFactory.h>
@@ -95,6 +96,7 @@ IceInternal::BasicStream::swap(BasicStream& other)
     }
 
     std::swap(_seqDataStack, other._seqDataStack);
+    std::swap(_unlimited, other._unlimited);
 }
 
 //
@@ -168,7 +170,7 @@ IceInternal::BasicStream::startSeq(int numElements, int minSize)
 	//
 	if(numElements * minSize > bytesLeft) 
 	{
-	    throw UnmarshalOutOfBoundsException(__FILE__, __LINE__);
+	    throwUnmarshalOutOfBoundsException(__FILE__, __LINE__);
 	}
     }
     else // Nested sequence
@@ -188,7 +190,7 @@ IceInternal::BasicStream::checkFixedSeq(int numElements, int elemSize)
 	//
 	if(numElements * elemSize > bytesLeft) 
 	{
-	    throw UnmarshalOutOfBoundsException(__FILE__, __LINE__);
+	    throwUnmarshalOutOfBoundsException(__FILE__, __LINE__);
 	}
     }
     else // Nested sequence
@@ -249,11 +251,11 @@ IceInternal::BasicStream::skipEncaps()
     read(sz);
     if(sz < 0)
     {
-	throw NegativeSizeException(__FILE__, __LINE__);
+	throwNegativeSizeException(__FILE__, __LINE__);
     }
     if(i - sizeof(Int) + sz > b.end())
     {
-	throw UnmarshalOutOfBoundsException(__FILE__, __LINE__);
+	throwUnmarshalOutOfBoundsException(__FILE__, __LINE__);
     }
     i += sz - sizeof(Int);
 }
@@ -292,7 +294,7 @@ IceInternal::BasicStream::startReadSlice()
     read(sz);
     if(sz < 0)
     {
-	throw NegativeSizeException(__FILE__, __LINE__);
+	throwNegativeSizeException(__FILE__, __LINE__);
     }
     _readSlice = i - b.begin();
 }
@@ -309,12 +311,12 @@ IceInternal::BasicStream::skipSlice()
     read(sz);
     if(sz < 0)
     {
-	throw NegativeSizeException(__FILE__, __LINE__);
+	throwNegativeSizeException(__FILE__, __LINE__);
     }
     i += sz - sizeof(Int);
     if(i > b.end())
     {
-	throw UnmarshalOutOfBoundsException(__FILE__, __LINE__);
+	throwUnmarshalOutOfBoundsException(__FILE__, __LINE__);
     }
 }
 
@@ -336,7 +338,7 @@ IceInternal::BasicStream::readBlob(vector<Byte>& v, Int sz)
     {
 	if(b.end() - i < sz)
 	{
-	    throw UnmarshalOutOfBoundsException(__FILE__, __LINE__);
+	    throwUnmarshalOutOfBoundsException(__FILE__, __LINE__);
 	}
 	vector<Byte>(i, i + sz).swap(v);
 	i += sz;
@@ -454,8 +456,7 @@ namespace
 template<size_t boolSize>
 struct BasicStreamReadBoolHelper
 {
-    static void read(pair<const bool*, const bool*>& v, IceUtil::auto_array<bool>& b,
-		     Int sz, BasicStream::Container::iterator& i)
+    static bool* read(pair<const bool*, const bool*>& v, Int sz, BasicStream::Container::iterator& i)
     {
         bool* array = new bool[sz];
         for(int idx = 0; idx < sz; ++idx)
@@ -464,38 +465,40 @@ struct BasicStreamReadBoolHelper
         }
         v.first = array;
         v.second = array + sz;
-        b.reset(array);
+	return array;
     }
 };
 
 template<>
 struct BasicStreamReadBoolHelper<1>
 {
-    static void read(pair<const bool*, const bool*>& v, IceUtil::auto_array<bool>& b,
-		     Int sz, BasicStream::Container::iterator& i)
+    static bool* read(pair<const bool*, const bool*>& v, Int sz, BasicStream::Container::iterator& i)
     {
         v.first = reinterpret_cast<bool*>(i);
         v.second = reinterpret_cast<bool*>(i) + sz;
+	return 0;
     }
 };
 
 }
 
-void
-IceInternal::BasicStream::read(pair<const bool*, const bool*>& v, IceUtil::auto_array<bool>& b)
+bool*
+IceInternal::BasicStream::read(pair<const bool*, const bool*>& v)
 {
+    bool* result = 0;
     Int sz;
     readSize(sz);
     if(sz > 0)
     {
         checkFixedSeq(sz, 1);
-	BasicStreamReadBoolHelper<sizeof(bool)>::read(v, b, sz, i);
+	result = BasicStreamReadBoolHelper<sizeof(bool)>::read(v, sz, i);
         i += sz;
     }
     else
     {
         v.first = v.second = reinterpret_cast<bool*>(i);
     }
+    return result;
 }
 
 void
@@ -521,7 +524,7 @@ IceInternal::BasicStream::read(Short& v)
 {
     if(b.end() - i < static_cast<int>(sizeof(Short)))
     {
-	throw UnmarshalOutOfBoundsException(__FILE__, __LINE__);
+	throwUnmarshalOutOfBoundsException(__FILE__, __LINE__);
     }
     const Byte* src = &(*i);
     i += sizeof(Short);
@@ -590,9 +593,10 @@ IceInternal::BasicStream::read(vector<Short>& v)
     }
 }
 
-void
-IceInternal::BasicStream::read(pair<const Short*, const Short*>& v, IceUtil::auto_array<Short>& b)
+Short*
+IceInternal::BasicStream::read(pair<const Short*, const Short*>& v)
 {
+    Short* result = 0;
     Int sz;
     readSize(sz);
     if(sz > 0)
@@ -603,16 +607,15 @@ IceInternal::BasicStream::read(pair<const Short*, const Short*>& v, IceUtil::aut
         i += sz * static_cast<int>(sizeof(Short));
         v.second = reinterpret_cast<Short*>(i);
 #else
-        Short* array = new Short[sz];
-        v.first = array;
-        v.second = array + sz;
-        b.reset(array);
+        result = new Short[sz];
+        v.first = result;
+        v.second = result + sz;
 
         Container::iterator begin = i;
         i += sz * static_cast<int>(sizeof(Short));
 #  ifdef ICE_BIG_ENDIAN
         const Byte* src = &(*begin);
-        Byte* dest = reinterpret_cast<Byte*>(&array[0]) + sizeof(Short) - 1;
+        Byte* dest = reinterpret_cast<Byte*>(&result[0]) + sizeof(Short) - 1;
         for(int j = 0 ; j < sz ; ++j)
         {
             *dest-- = *src++;
@@ -620,7 +623,7 @@ IceInternal::BasicStream::read(pair<const Short*, const Short*>& v, IceUtil::aut
             dest += 2 * sizeof(Short);
         }
 #  else
-        copy(begin, i, reinterpret_cast<Byte*>(&array[0]));
+        copy(begin, i, reinterpret_cast<Byte*>(&result[0]));
 #  endif
 #endif
     }
@@ -628,6 +631,7 @@ IceInternal::BasicStream::read(pair<const Short*, const Short*>& v, IceUtil::aut
     {
         v.first = v.second = 0;
     }
+    return result;
 }
 
 void
@@ -688,9 +692,10 @@ IceInternal::BasicStream::read(vector<Int>& v)
     }
 }
 
-void
-IceInternal::BasicStream::read(pair<const Int*, const Int*>& v, IceUtil::auto_array<Int>& b)
+Int*
+IceInternal::BasicStream::read(pair<const Int*, const Int*>& v)
 {
+    Int* result = 0;
     Int sz;
     readSize(sz);
     if(sz > 0)
@@ -701,16 +706,15 @@ IceInternal::BasicStream::read(pair<const Int*, const Int*>& v, IceUtil::auto_ar
         i += sz * static_cast<int>(sizeof(Int));
         v.second = reinterpret_cast<Int*>(i);
 #else
-        Int* array = new Int[sz];
-        v.first = array;
-        v.second = array + sz;
-        b.reset(array);
+        result = new Int[sz];
+        v.first = result;
+        v.second = result + sz;
 
         Container::iterator begin = i;
         i += sz * static_cast<int>(sizeof(Int));
 #  ifdef ICE_BIG_ENDIAN
         const Byte* src = &(*begin);
-        Byte* dest = reinterpret_cast<Byte*>(&array[0]) + sizeof(Int) - 1;
+        Byte* dest = reinterpret_cast<Byte*>(&result[0]) + sizeof(Int) - 1;
         for(int j = 0 ; j < sz ; ++j)
         {
             *dest-- = *src++;
@@ -720,7 +724,7 @@ IceInternal::BasicStream::read(pair<const Int*, const Int*>& v, IceUtil::auto_ar
             dest += 2 * sizeof(Int);
         }
 #  else
-        copy(begin, i, reinterpret_cast<Byte*>(&array[0]));
+        copy(begin, i, reinterpret_cast<Byte*>(&result[0]));
 #  endif
 #endif
     }
@@ -728,6 +732,7 @@ IceInternal::BasicStream::read(pair<const Int*, const Int*>& v, IceUtil::auto_ar
     {
         v.first = v.second = 0;
     }
+    return result;
 }
 
 void
@@ -764,7 +769,7 @@ IceInternal::BasicStream::read(Long& v)
 {
     if(b.end() - i < static_cast<int>(sizeof(Long)))
     {
-	throw UnmarshalOutOfBoundsException(__FILE__, __LINE__);
+	throwUnmarshalOutOfBoundsException(__FILE__, __LINE__);
     }
     const Byte* src = &(*i);
     i += sizeof(Long);
@@ -857,9 +862,10 @@ IceInternal::BasicStream::read(vector<Long>& v)
     }
 }
 
-void
-IceInternal::BasicStream::read(pair<const Long*, const Long*>& v, IceUtil::auto_array<Long>& b)
+Long*
+IceInternal::BasicStream::read(pair<const Long*, const Long*>& v)
 {
+    Long* result = 0;
     Int sz;
     readSize(sz);
     if(sz > 0)
@@ -870,16 +876,15 @@ IceInternal::BasicStream::read(pair<const Long*, const Long*>& v, IceUtil::auto_
         i += sz * static_cast<int>(sizeof(Long));
         v.second = reinterpret_cast<Long*>(i);
 #else
-        Long* array = new Long[sz];
-        v.first = array;
-        v.second = array + sz;
-        b.reset(array);
+        result = new Long[sz];
+        v.first = result;
+        v.second = result + sz;
 
         Container::iterator begin = i;
         i += sz * static_cast<int>(sizeof(Long));
 #  ifdef ICE_BIG_ENDIAN
         const Byte* src = &(*begin);
-        Byte* dest = reinterpret_cast<Byte*>(&array[0]) + sizeof(Long) - 1;
+        Byte* dest = reinterpret_cast<Byte*>(&result[0]) + sizeof(Long) - 1;
         for(int j = 0 ; j < sz ; ++j)
         {
             *dest-- = *src++;
@@ -893,7 +898,7 @@ IceInternal::BasicStream::read(pair<const Long*, const Long*>& v, IceUtil::auto_
             dest += 2 * sizeof(Long);
         }
 #  else
-        copy(begin, i, reinterpret_cast<Byte*>(&array[0]));
+        copy(begin, i, reinterpret_cast<Byte*>(&result[0]));
 #  endif
 #endif
     }
@@ -901,6 +906,7 @@ IceInternal::BasicStream::read(pair<const Long*, const Long*>& v, IceUtil::auto_
     {
         v.first = v.second = 0;
     }
+    return result;
 }
 
 void
@@ -929,7 +935,7 @@ IceInternal::BasicStream::read(Float& v)
 {
     if(b.end() - i < static_cast<int>(sizeof(Float)))
     {
-	throw UnmarshalOutOfBoundsException(__FILE__, __LINE__);
+	throwUnmarshalOutOfBoundsException(__FILE__, __LINE__);
     }
     const Byte* src = &(*i);
     i += sizeof(Float);
@@ -1006,9 +1012,10 @@ IceInternal::BasicStream::read(vector<Float>& v)
     }
 }
 
-void
-IceInternal::BasicStream::read(pair<const Float*, const Float*>& v, IceUtil::auto_array<Float>& b)
+Float*
+IceInternal::BasicStream::read(pair<const Float*, const Float*>& v)
 {
+    Float* result = 0;
     Int sz;
     readSize(sz);
     if(sz > 0)
@@ -1019,16 +1026,15 @@ IceInternal::BasicStream::read(pair<const Float*, const Float*>& v, IceUtil::aut
         i += sz * static_cast<int>(sizeof(Float));
         v.second = reinterpret_cast<Float*>(i);
 #else
-        Float* array = new Float[sz];
-        v.first = array;
-        v.second = array + sz;
-        b.reset(array);
+        result = new Float[sz];
+        v.first = result;
+        v.second = result + sz;
 
         Container::iterator begin = i;
         i += sz * static_cast<int>(sizeof(Float));
 #  ifdef ICE_BIG_ENDIAN
         const Byte* src = &(*begin);
-        Byte* dest = reinterpret_cast<Byte*>(&array[0]) + sizeof(Float) - 1;
+        Byte* dest = reinterpret_cast<Byte*>(&result[0]) + sizeof(Float) - 1;
         for(int j = 0 ; j < sz ; ++j)
         {
             *dest-- = *src++;
@@ -1038,7 +1044,7 @@ IceInternal::BasicStream::read(pair<const Float*, const Float*>& v, IceUtil::aut
             dest += 2 * sizeof(Float);
         }
 #  else
-        copy(begin, i, reinterpret_cast<Byte*>(&array[0]));
+        copy(begin, i, reinterpret_cast<Byte*>(&result[0]));
 #  endif
 #endif
     }
@@ -1046,6 +1052,7 @@ IceInternal::BasicStream::read(pair<const Float*, const Float*>& v, IceUtil::aut
     {
         v.first = v.second = 0;
     }
+    return result;
 }
 
 void
@@ -1066,7 +1073,7 @@ IceInternal::BasicStream::write(Double v)
     *dest = *src;
 #else
     const Byte* src = reinterpret_cast<const Byte*>(&v);
-#  if defined(_ARM_) && defined(__linux)
+#  if defined(__arm__) && defined(__linux)
     dest[4] = *src++;
     dest[5] = *src++;
     dest[6] = *src++;
@@ -1093,7 +1100,7 @@ IceInternal::BasicStream::read(Double& v)
 {
     if(b.end() - i < static_cast<int>(sizeof(Double)))
     {
-	throw UnmarshalOutOfBoundsException(__FILE__, __LINE__);
+	throwUnmarshalOutOfBoundsException(__FILE__, __LINE__);
     }
     const Byte* src = &(*i);
     i += sizeof(Double);
@@ -1109,7 +1116,7 @@ IceInternal::BasicStream::read(Double& v)
     *dest = *src;
 #else
     Byte* dest = reinterpret_cast<Byte*>(&v);
-#  if defined(_ARM_) && defined(__linux)
+#  if defined(__arm__) && defined(__linux)
     dest[4] = *src++;
     dest[5] = *src++;
     dest[6] = *src++;
@@ -1155,7 +1162,7 @@ IceInternal::BasicStream::write(const Double* begin, const Double* end)
 	    *dest++ = *src--;
 	    src += 2 * sizeof(Double);
 	}
-#elif defined(_ARM_) && defined(__linux)
+#elif defined(__arm__) && defined(__linux)
 	const Byte* src = reinterpret_cast<const Byte*>(begin);
 	Byte* dest = &(*(b.begin() + pos));
 	for(int j = 0 ; j < sz ; ++j)
@@ -1202,7 +1209,7 @@ IceInternal::BasicStream::read(vector<Double>& v)
 	    *dest-- = *src++;
 	    dest += 2 * sizeof(Double);
 	}
-#elif defined(_ARM_) && defined(__linux)
+#elif defined(__arm__) && defined(__linux)
 	const Byte* src = &(*begin);
 	Byte* dest = reinterpret_cast<Byte*>(&v[0]);
 	for(int j = 0 ; j < sz ; ++j)
@@ -1227,9 +1234,10 @@ IceInternal::BasicStream::read(vector<Double>& v)
     }
 }
 
-void
-IceInternal::BasicStream::read(pair<const Double*, const Double*>& v, IceUtil::auto_array<Double>& b)
+Double*
+IceInternal::BasicStream::read(pair<const Double*, const Double*>& v)
 {
+    Double* result = 0;
     Int sz;
     readSize(sz);
     if(sz > 0)
@@ -1240,16 +1248,15 @@ IceInternal::BasicStream::read(pair<const Double*, const Double*>& v, IceUtil::a
         i += sz * static_cast<int>(sizeof(Double));
         v.second = reinterpret_cast<Double*>(i);
 #else
-        Double* array = new Double[sz];
-        v.first = array;
-        v.second = array + sz;
-        b.reset(array);
+        result = new Double[sz];
+        v.first = result;
+        v.second = result + sz;
 
         Container::iterator begin = i;
         i += sz * static_cast<int>(sizeof(Double));
 #  ifdef ICE_BIG_ENDIAN
         const Byte* src = &(*begin);
-        Byte* dest = reinterpret_cast<Byte*>(&array[0]) + sizeof(Double) - 1;
+        Byte* dest = reinterpret_cast<Byte*>(&result[0]) + sizeof(Double) - 1;
         for(int j = 0 ; j < sz ; ++j)
         {
             *dest-- = *src++;
@@ -1263,7 +1270,7 @@ IceInternal::BasicStream::read(pair<const Double*, const Double*>& v, IceUtil::a
             dest += 2 * sizeof(Double);
         }
 #  else
-        copy(begin, i, reinterpret_cast<Byte*>(&array[0]));
+        copy(begin, i, reinterpret_cast<Byte*>(&result[0]));
 #  endif
 #endif
     }
@@ -1271,6 +1278,7 @@ IceInternal::BasicStream::read(pair<const Double*, const Double*>& v, IceUtil::a
     {
         v.first = v.second = 0;
     }
+    return result;
 }
 
 
@@ -1289,8 +1297,70 @@ IceInternal::BasicStream::write(const char*)
 }
 */
 
+#ifdef ICEE_HAS_WSTRING
+
 void
-IceInternal::BasicStream::write(const string* begin, const string* end)
+IceInternal::BasicStream::writeConverted(const string& v)
+{
+    //
+    // What is the size of the resulting UTF-8 encoded string?
+    // Impossible to tell, so we guess. If we don't guess correctly,
+    // we'll have to fix the mistake afterwards
+    //
+
+    Int guessedSize = static_cast<Int>(v.size());
+    writeSize(guessedSize); // writeSize() only writes the size; it does not reserve any buffer space.
+
+    size_t firstIndex = b.size();
+    StreamUTF8BufferI buffer(*this);
+
+    Byte* lastByte = _stringConverter->toUTF8(v.data(), v.data() + v.size(), buffer);
+    if(lastByte != b.end())
+    {
+        b.resize(lastByte - b.begin());
+    }
+    size_t lastIndex = b.size();
+
+    Int actualSize = static_cast<Int>(lastIndex - firstIndex);
+
+    //
+    // Check against the guess
+    //
+    if(guessedSize != actualSize)
+    {
+        if(guessedSize <= 254 && actualSize > 254)
+        {
+            //
+            // Move the UTF-8 sequence 4 bytes further
+            // Use memmove instead of memcpy since the source and destination typically overlap.
+            //
+            resize(b.size() + 4);
+            memmove(b.begin() + firstIndex + 4, b.begin() + firstIndex, actualSize);
+        }
+        else if(guessedSize > 254 && actualSize <= 254)
+        {
+            //
+            // Move the UTF-8 sequence 4 bytes back
+            //
+            memmove(b.begin() + firstIndex - 4, b.begin() + firstIndex, actualSize);
+            resize(b.size() - 4);
+        }
+
+        if(guessedSize <= 254)
+        {
+            rewriteSize(actualSize, b.begin() + firstIndex - 1);
+        }
+        else
+        {
+            rewriteSize(actualSize, b.begin() + firstIndex - 1 - 4);
+        }
+    }
+}
+
+#endif
+
+void
+IceInternal::BasicStream::write(const string* begin, const string* end, bool convert)
 {
     Int sz = static_cast<Int>(end - begin);
     writeSize(sz);
@@ -1298,13 +1368,13 @@ IceInternal::BasicStream::write(const string* begin, const string* end)
     {
 	for(int i = 0; i < sz; ++i)
 	{
-	    write(begin[i]);
+	    write(begin[i], convert);
 	}
     }
 }
 
 void
-IceInternal::BasicStream::read(vector<string>& v)
+IceInternal::BasicStream::read(vector<string>& v, bool convert)
 {
     Int sz;
     readSize(sz);
@@ -1314,7 +1384,7 @@ IceInternal::BasicStream::read(vector<string>& v)
 	v.resize(sz);
 	for(int i = 0; i < sz; ++i)
 	{
-	    read(v[i]);
+	    read(v[i], convert);
 	    checkSeq();
 	    endElement();
 	}
@@ -1326,6 +1396,111 @@ IceInternal::BasicStream::read(vector<string>& v)
     }
 }
 
+#ifdef ICEE_HAS_WSTRING
+
+void
+IceInternal::BasicStream::write(const wstring& v)
+{
+    if(v.size() == 0)
+    {
+        writeSize(0);
+        return;
+    }
+
+    //
+    // What is the size of the resulting UTF-8 encoded string?
+    // Impossible to tell, so we guess. If we don't guess correctly,
+    // we'll have to fix the mistake afterwards
+    //
+
+    Int guessedSize = static_cast<Int>(v.size());
+    writeSize(guessedSize); // writeSize() only writes the size; it does not reserve any buffer space.
+
+    size_t firstIndex = b.size();
+    StreamUTF8BufferI buffer(*this);
+
+    Byte* lastByte = _wstringConverter->toUTF8(v.data(), v.data() + v.size(), buffer);
+    if(lastByte != b.end())
+    {
+        b.resize(lastByte - b.begin());
+    }
+    size_t lastIndex = b.size();
+
+    Int actualSize = static_cast<Int>(lastIndex - firstIndex);
+
+    //
+    // Check against the guess
+    //
+    if(guessedSize != actualSize)
+    {
+        if(guessedSize <= 254 && actualSize > 254)
+        {
+            //
+            // Move the UTF-8 sequence 4 bytes further
+            // Use memmove instead of memcpy since the source and destination typically overlap.
+            //
+            resize(b.size() + 4);
+            memmove(b.begin() + firstIndex + 4, b.begin() + firstIndex, actualSize);
+        }
+        else if(guessedSize > 254 && actualSize <= 254)
+        {
+            //
+            // Move the UTF-8 sequence 4 bytes back
+            //
+            memmove(b.begin() + firstIndex - 4, b.begin() + firstIndex, actualSize);
+            resize(b.size() - 4);
+        }
+
+        if(guessedSize <= 254)
+        {
+            rewriteSize(actualSize, b.begin() + firstIndex - 1);
+        }
+        else
+        {
+            rewriteSize(actualSize, b.begin() + firstIndex - 1 - 4);
+        }
+    }
+}
+
+void
+IceInternal::BasicStream::write(const wstring* begin, const wstring* end)
+{
+    Int sz = static_cast<Int>(end - begin);
+    writeSize(sz);
+    if(sz > 0)
+    {
+        for(int i = 0; i < sz; ++i)
+        {
+            write(begin[i]);
+        }
+    }
+}   
+    
+void
+IceInternal::BasicStream::read(vector<wstring>& v)
+{   
+    Int sz;
+    readSize(sz);
+    if(sz > 0)
+    {
+        startSeq(sz, 1);
+        v.resize(sz);
+        for(int j = 0; j < sz; ++j)
+        {
+            read(v[j]);
+            checkSeq();
+            endElement();
+        }
+        endSeq(sz);
+    }
+    else
+    {
+       v.clear();
+    }
+}
+
+#endif
+
 void
 IceInternal::BasicStream::write(const ObjectPrx& v)
 {
@@ -1352,7 +1527,7 @@ IceInternal::BasicStream::throwException()
     read(usesClasses);
 
     string id;
-    read(id);
+    read(id, false);
     for(;;)
     {
 	//
@@ -1380,7 +1555,7 @@ IceInternal::BasicStream::throwException()
 	else
 	{
 	    skipSlice(); // Slice off what we don't understand.
-	    read(id); // Read type id for next slice.
+	    read(id, false); // Read type id for next slice.
 	}
     }
 
@@ -1393,35 +1568,6 @@ IceInternal::BasicStream::throwException()
     //
 }
 
-void
-IceInternal::BasicStream::throwUnmarshalOutOfBoundsException(const char* file, int line)
-{
-    throw UnmarshalOutOfBoundsException(file, line);
-}
-
-void
-IceInternal::BasicStream::throwMemoryLimitException(const char* file, int line)
-{
-    throw MemoryLimitException(file, line);
-}
-
-void
-IceInternal::BasicStream::throwNegativeSizeException(const char* file, int line)
-{
-    throw NegativeSizeException(file, line);
-}
-
-void
-IceInternal::BasicStream::throwUnsupportedEncodingException(const char* file, int line, Byte eMajor, Byte eMinor)
-{
-    UnsupportedEncodingException ex(file, line);
-    ex.badMajor = static_cast<unsigned char>(eMajor);
-    ex.badMinor = static_cast<unsigned char>(eMinor);
-    ex.major = static_cast<unsigned char>(encodingMajor);
-    ex.minor = static_cast<unsigned char>(encodingMinor);
-    throw ex;
-}
-
 IceInternal::BasicStream::SeqData::SeqData(int num, int sz) : numElements(num), minSize(sz)
 {
 }
