@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -32,26 +32,7 @@ using namespace std;
 using namespace Ice;
 using namespace IceInternal;
 
-void IceInternal::incRef(Connection* p) { p->__incRef(); }
-void IceInternal::decRef(Connection* p) { p->__decRef(); }
-
-bool
-Ice::operator==(const Connection& l, const Connection& r)
-{
-    return &l == &r;
-}
-
-bool
-Ice::operator!=(const Connection& l, const Connection& r)
-{
-    return &l != &r;
-}
-
-bool
-Ice::operator<(const Connection& l, const Connection& r)
-{
-    return &l < &r;
-}
+IceUtil::Shared* IceInternal::upCast(Connection* p) { return p; }
 
 void
 Ice::Connection::waitForValidation()
@@ -412,17 +393,17 @@ Ice::Connection::sendRequest(BasicStream* os, Outgoing* out)
 	    readStreamAndParseMessage(*os, receivedRequestId, invokeNum);
 	    if(invokeNum > 0)
 	    {
-		throw UnknownMessageException(__FILE__, __LINE__);
+		throwUnknownMessageException(__FILE__, __LINE__);
 	    }
 	    else if(requestId != receivedRequestId)
 	    {
-		throw UnknownRequestIdException(__FILE__, __LINE__);
+		throwUnknownRequestIdException(__FILE__, __LINE__);
 	    }
 #else
 	    readStreamAndParseMessage(*os, receivedRequestId);
 	    if(requestId != receivedRequestId)
 	    {
-		throw UnknownRequestIdException(__FILE__, __LINE__);
+		throwUnknownRequestIdException(__FILE__, __LINE__);
 	    }
 #endif
 	    out->finished(*os);
@@ -530,7 +511,7 @@ Ice::Connection::prepareBatchRequest(BasicStream* os)
     {
 	try
 	{
-	    _batchStream.writeBlob(&_requestBatchHdr[0], headerSize + sizeof(Int));
+	    _batchStream.writeBlob(requestBatchHdr, sizeof(requestBatchHdr));
 	}
 	catch(const LocalException& ex)
 	{
@@ -540,6 +521,7 @@ Ice::Connection::prepareBatchRequest(BasicStream* os)
     }
 
     _batchStreamInUse = true;
+    _batchMarker = _batchStream.b.size();
     _batchStream.swap(*os);
 
     //
@@ -551,21 +533,103 @@ Ice::Connection::prepareBatchRequest(BasicStream* os)
 void
 Ice::Connection::finishBatchRequest(BasicStream* os)
 {
-    IceUtil::Monitor<IceUtil::Mutex>::Lock sync(*this);
+    bool autoflush = false;
+    vector<Ice::Byte> lastRequest;
 
-    //
-    // Get the batch stream back and increment the number of requests
-    // in the batch.
-    //
-    _batchStream.swap(*os);
-    ++_batchRequestNum;
+    {
+        IceUtil::Monitor<IceUtil::Mutex>::Lock sync(*this);
 
-    //
-    // Notify about the batch stream not being in use anymore.
-    //
-    assert(_batchStreamInUse);
-    _batchStreamInUse = false;
-    notifyAll();
+        //
+        // Get the batch stream back.
+        //
+        _batchStream.swap(*os);
+
+        if(_batchAutoFlush)
+        {
+            Lock sendSync(_sendMonitor);
+
+            if(!_transceiver)
+            {
+                assert(_exception.get());
+                _exception->ice_throw(); // The exception is immutable at this point.
+            }
+
+            //
+            // Throw memory limit exception if the first message added causes us to 
+            // go over limit. Otherwise put aside the marshalled message that caused
+            // limit to be exceeded and rollback stream to the marker.
+            //
+            if(_batchStream.b.size() > _instance->messageSizeMax())
+            {
+                if(_batchRequestNum == 0)
+                {
+                    resetBatch(true);
+                    throwMemoryLimitException(__FILE__, __LINE__);
+                }
+                vector<Ice::Byte>(_batchStream.b.begin() + _batchMarker, _batchStream.b.end()).swap(lastRequest);
+                _batchStream.b.resize(_batchMarker);
+                autoflush = true;
+            }
+        }
+
+        if(!autoflush)
+        {
+            //
+            // Increment the number of requests in the batch.
+            //
+            ++_batchRequestNum;
+
+            //
+            // Notify about the batch stream not being in use anymore.
+            //
+            assert(_batchStreamInUse);
+            _batchStreamInUse = false;
+            notifyAll();
+        }
+    }
+
+    if(autoflush)
+    {
+        //
+        // We have to keep _batchStreamInUse set until after we insert the
+        // saved marshalled data into a new stream.
+        //
+        flushBatchRequestsInternal(true);
+
+        IceUtil::Monitor<IceUtil::Mutex>::Lock sync(*this);
+
+        //
+        // Throw memory limit exception if the message that caused us to go over
+        // limit causes us to exceed the limit by itself.
+        //
+        if(sizeof(requestBatchHdr) + lastRequest.size() >  _instance->messageSizeMax())
+        {
+            resetBatch(true);
+            throwMemoryLimitException(__FILE__, __LINE__);
+        }
+
+        //
+        // Start a new batch with the last message that caused us to 
+        // go over the limit.
+        //
+        try
+        {
+            _batchStream.writeBlob(requestBatchHdr, sizeof(requestBatchHdr));
+            _batchStream.writeBlob(&lastRequest[0], lastRequest.size());
+        }
+        catch(const LocalException& ex)
+        {
+            setState(StateClosed, ex);
+            ex.ice_throw();
+        }
+
+        //
+        // Notify that the batch stream not in use anymore.
+        //
+        ++_batchRequestNum;
+        _batchStreamInUse = false;
+        notifyAll();
+    }
 }
 
 void
@@ -574,33 +638,32 @@ Ice::Connection::abortBatchRequest()
     IceUtil::Monitor<IceUtil::Mutex>::Lock sync(*this);
     
     //
-    // Destroy and reset the batch stream and batch count. We cannot
-    // safe old requests in the batch stream, as they might be
-    // corrupted due to incomplete marshaling.
+    // Reset the batch stream. We cannot save old requests
+    // in the batch stream, as they might be corrupted due to
+    // incomplete marshaling.
     //
-    BasicStream dummy(_instance.get(), _instance->messageSizeMax());
-    _batchStream.swap(dummy);
-    _batchRequestNum = 0;
-
-    //
-    // Notify about the batch stream not being in use
-    // anymore.
-    //
-    assert(_batchStreamInUse);
-    _batchStreamInUse = false;
-    notifyAll();
+    resetBatch(true);
 }
 
 void
 Ice::Connection::flushBatchRequests()
+{
+    flushBatchRequestsInternal(false);
+}
+
+void
+Ice::Connection::flushBatchRequestsInternal(bool ignoreInUse)
 {
     {
 	IceUtil::Monitor<IceUtil::Mutex>::Lock sync(*this);
 
-	while(_batchStreamInUse && !_exception.get())
-	{
-	    wait();
-	}
+        if(!ignoreInUse)
+        {
+	    while(_batchStreamInUse && !_exception.get())
+	    {
+	        wait();
+	    }
+        }
 	
 	if(_exception.get())
 	{
@@ -697,11 +760,31 @@ Ice::Connection::flushBatchRequests()
 	//
 	// Reset the batch stream, and notify that flushing is over.
 	//
-	BasicStream dummy(_instance.get(), _instance->messageSizeMax());
-	_batchStream.swap(dummy);
-	_batchRequestNum = 0;
-	_batchStreamInUse = false;
-	notifyAll();
+        resetBatch(!ignoreInUse);
+    }
+}
+
+void
+Ice::Connection::resetBatch(bool resetInUse)
+{
+    BasicStream dummy(_instance.get(), _instance->messageSizeMax(),
+#ifdef ICEE_HAS_WSTRING
+        _instance->initializationData().stringConverter, _instance->initializationData().wstringConverter,
+#endif
+        _batchAutoFlush);
+    _batchStream.swap(dummy);
+    _batchRequestNum = 0;
+    _batchMarker = 0;
+
+    //
+    // Notify about the batch stream not being in use
+    // anymore.
+    //
+    if(resetInUse)
+    {
+        assert(_batchStreamInUse);
+        _batchStreamInUse = false;
+        notifyAll();
     }
 }
 
@@ -823,9 +906,7 @@ Ice::Connection::setAdapter(const ObjectAdapterPtr& adapter)
 
     if(_blocking)
     {
-        FeatureNotSupportedException ex(__FILE__, __LINE__);
-	ex.unsupportedFeature = "setAdapter with blocking connection";
-        throw ex;
+        throw FeatureNotSupportedException(__FILE__, __LINE__, "setAdapter with blocking connection");
     }
 
     //
@@ -864,8 +945,8 @@ Ice::Connection::createProxy(const Identity& ident) const
     //
     vector<ConnectionPtr> connections;
     connections.push_back(const_cast<Connection*>(this));
-    ReferencePtr ref = _instance->referenceFactory()->create(ident, _instance->getDefaultContext(), "",
-    							     Reference::ModeTwoway, connections);
+    ReferencePtr ref = _instance->referenceFactory()->create(ident, Ice::Context(), "", ReferenceModeTwoway, 
+							     connections);
     return _instance->proxyFactory()->referenceToProxy(ref);
 }
 
@@ -905,22 +986,30 @@ Ice::Connection::Connection(const InstancePtr& instance,
 	_desc(transceiver->toString()),
 	_type(transceiver->type()),
 	_endpoint(endpoint),
-	_logger(_instance->logger()), // Cached for better performance.
+	_logger(_instance->initializationData().logger), // Cached for better performance.
 	_traceLevels(_instance->traceLevels()), // Cached for better performance.
-	_warn(_instance->properties()->getPropertyAsInt("Ice.Warn.Connections") > 0),
-	_requestHdr(headerSize + sizeof(Int), 0),
+	_warn(_instance->initializationData().properties->getPropertyAsInt("Ice.Warn.Connections") > 0),
 #ifndef ICEE_PURE_CLIENT
-	_replyHdr(headerSize, 0),
 	_in(_instance.get(), this, _stream, adapter),
 #endif
 #ifndef ICEE_PURE_BLOCKING_CLIENT
-	_stream(_instance.get(), _instance->messageSizeMax()),
+	_stream(_instance.get(), _instance->messageSizeMax()
+#ifdef ICEE_HAS_WSTRING
+                , _instance->initializationData().stringConverter, _instance->initializationData().wstringConverter
+#endif
+                ),
 #endif
 #ifdef ICEE_HAS_BATCH
-	_requestBatchHdr(headerSize + sizeof(Int), 0),
-	_batchStream(_instance.get(), _instance->messageSizeMax()),
+        _batchAutoFlush(
+            _instance->initializationData().properties->getPropertyAsIntWithDefault("Ice.BatchAutoFlush", 1) > 0),
+	_batchStream(_instance.get(), _instance->messageSizeMax(),
+#ifdef ICEE_HAS_WSTRING
+                     _instance->initializationData().stringConverter, _instance->initializationData().wstringConverter,
+#endif
+                     _batchAutoFlush),
 	_batchStreamInUse(false),
 	_batchRequestNum(0),
+        _batchMarker(0),
 #endif
 	_dispatchCount(0),
 	_state(StateNotValidated),
@@ -932,9 +1021,9 @@ Ice::Connection::Connection(const InstancePtr& instance,
 {
 #ifndef ICEE_PURE_BLOCKING_CLIENT
 #  ifdef ICEE_PURE_CLIENT
-    _blocking = _instance->properties()->getPropertyAsInt("Ice.Blocking") > 0;
+    _blocking = _instance->initializationData().properties->getPropertyAsInt("Ice.Blocking") > 0;
 #  else
-    _blocking = _instance->properties()->getPropertyAsInt("Ice.Blocking") > 0 && !adapter;
+    _blocking = _instance->initializationData().properties->getPropertyAsInt("Ice.Blocking") > 0 && !adapter;
 #  endif
     if(_blocking)
     {
@@ -956,47 +1045,6 @@ Ice::Connection::Connection(const InstancePtr& instance,
     _transceiver->setTimeouts(_endpoint->timeout(), _endpoint->timeout());
 #endif
 
-    vector<Byte>& requestHdr = const_cast<vector<Byte>&>(_requestHdr);
-    requestHdr[0] = magic[0];
-    requestHdr[1] = magic[1];
-    requestHdr[2] = magic[2];
-    requestHdr[3] = magic[3];
-    requestHdr[4] = protocolMajor;
-    requestHdr[5] = protocolMinor;
-    requestHdr[6] = encodingMajor;
-    requestHdr[7] = encodingMinor;
-    requestHdr[8] = requestMsg;
-    requestHdr[9] = 0;
-
-#ifdef ICEE_HAS_BATCH
-    vector<Byte>& requestBatchHdr = const_cast<vector<Byte>&>(_requestBatchHdr);
-    requestBatchHdr[0] = magic[0];
-    requestBatchHdr[1] = magic[1];
-    requestBatchHdr[2] = magic[2];
-    requestBatchHdr[3] = magic[3];
-    requestBatchHdr[4] = protocolMajor;
-    requestBatchHdr[5] = protocolMinor;
-    requestBatchHdr[6] = encodingMajor;
-    requestBatchHdr[7] = encodingMinor;
-    requestBatchHdr[8] = requestBatchMsg;
-    requestBatchHdr[9] = 0;
-#endif
-
-#ifndef ICEE_PURE_CLIENT
-    vector<Byte>& replyHdr = const_cast<vector<Byte>&>(_replyHdr);
-    replyHdr[0] = magic[0];
-    replyHdr[1] = magic[1];
-    replyHdr[2] = magic[2];
-    replyHdr[3] = magic[3];
-    replyHdr[4] = protocolMajor;
-    replyHdr[5] = protocolMinor;
-    replyHdr[6] = encodingMajor;
-    replyHdr[7] = encodingMinor;
-    replyHdr[8] = replyMsg;
-    replyHdr[9] = 0;
-
-#endif
-
 #ifdef ICEE_PURE_BLOCKING_CLIENT
     validate();
 #else
@@ -1098,7 +1146,12 @@ Ice::Connection::validate()
 #ifndef ICEE_PURE_CLIENT
         if(active)
         {
-    	    BasicStream os(_instance.get(), _instance->messageSizeMax());
+    	    BasicStream os(_instance.get(), _instance->messageSizeMax()
+#ifdef ICEE_HAS_WSTRING
+                           , _instance->initializationData().stringConverter,
+                           _instance->initializationData().wstringConverter
+#endif
+                          );
 	    os.write(magic[0]);
 	    os.write(magic[1]);
 	    os.write(magic[2]);
@@ -1127,7 +1180,12 @@ Ice::Connection::validate()
         else
 #endif
         {
-    	    BasicStream is(_instance.get(), _instance->messageSizeMax());
+    	    BasicStream is(_instance.get(), _instance->messageSizeMax()
+#ifdef ICEE_HAS_WSTRING
+                           , _instance->initializationData().stringConverter,
+                           _instance->initializationData().wstringConverter
+#endif
+                          );
     	    is.b.resize(headerSize);
     	    is.i = is.b.begin();
     	    try
@@ -1147,9 +1205,7 @@ Ice::Connection::validate()
 	    is.read(m[3]);
 	    if(m[0] != magic[0] || m[1] != magic[1] || m[2] != magic[2] || m[3] != magic[3])
 	    {
-		BadMagicException ex(__FILE__, __LINE__);
-		ex.badMagic = Ice::ByteSeq(&m[0], &m[0] + sizeof(m));
-		throw ex;
+		throwBadMagicException(__FILE__, __LINE__, Ice::ByteSeq(&m[0], &m[0] + sizeof(m)));
 	    }
     	    Byte pMajor;
     	    Byte pMinor;
@@ -1157,12 +1213,7 @@ Ice::Connection::validate()
     	    is.read(pMinor);
     	    if(pMajor != protocolMajor)
     	    {
-    	        UnsupportedProtocolException ex(__FILE__, __LINE__);
-    	        ex.badMajor = static_cast<unsigned char>(pMajor);
-    	        ex.badMinor = static_cast<unsigned char>(pMinor);
-    	        ex.major = static_cast<unsigned char>(protocolMajor);
-    	        ex.minor = static_cast<unsigned char>(protocolMinor);
-    	        throw ex;
+    	        throwUnsupportedProtocolException(__FILE__, __LINE__, pMajor, pMinor, protocolMajor, protocolMinor);
     	    }
     	    Byte eMajor;
     	    Byte eMinor;
@@ -1170,18 +1221,13 @@ Ice::Connection::validate()
     	    is.read(eMinor);
     	    if(eMajor != encodingMajor)
     	    {
-    	        UnsupportedEncodingException ex(__FILE__, __LINE__);
-    	        ex.badMajor = static_cast<unsigned char>(eMajor);
-    	        ex.badMinor = static_cast<unsigned char>(eMinor);
-    	        ex.major = static_cast<unsigned char>(encodingMajor);
-    	        ex.minor = static_cast<unsigned char>(encodingMinor);
-    	        throw ex;
+		throwUnsupportedEncodingException(__FILE__, __LINE__, eMajor, eMinor, encodingMajor, encodingMinor);
     	    }
     	    Byte messageType;
     	    is.read(messageType);
     	    if(messageType != validateConnectionMsg)
     	    {
-    	        throw ConnectionNotValidatedException(__FILE__, __LINE__);
+    	        throwConnectionNotValidatedException(__FILE__, __LINE__);
     	    }
             Byte compress;
             is.read(compress); // Ignore compression status for validate connection.
@@ -1189,7 +1235,7 @@ Ice::Connection::validate()
 	    is.read(size);
 	    if(size != headerSize)
 	    {
-	        throw IllegalMessageSizeException(__FILE__, __LINE__);
+	        throwIllegalMessageSizeException(__FILE__, __LINE__);
 	    }
 	    if(_traceLevels->protocol >= 1)
 	    {
@@ -1397,7 +1443,12 @@ Ice::Connection::initiateShutdown() const
     //
     // Before we shut down, we send a close connection message.
     //
-    BasicStream os(_instance.get(), _instance->messageSizeMax());
+    BasicStream os(_instance.get(), _instance->messageSizeMax()
+#ifdef ICEE_HAS_WSTRING
+                   , _instance->initializationData().stringConverter, _instance->initializationData().wstringConverter
+#endif
+                  );
+
     os.write(magic[0]);
     os.write(magic[1]);
     os.write(magic[2]);
@@ -1452,34 +1503,20 @@ Ice::Connection::readStreamAndParseMessage(IceInternal::BasicStream& stream, Int
     stream.readBlob(header, headerSize);
     if(header[0] != magic[0] || header[1] != magic[1] || header[2] != magic[2] || header[3] != magic[3])
     {
-	BadMagicException ex(__FILE__, __LINE__);
-	ex.badMagic = Ice::ByteSeq(&header[0], &header[0] + sizeof(magic));
-	throw ex;
+	throwBadMagicException(__FILE__, __LINE__, Ice::ByteSeq(&header[0], &header[0] + sizeof(magic)));
     }
     if(header[4] != protocolMajor)
     {
-	UnsupportedProtocolException ex(__FILE__, __LINE__);
-	ex.badMajor = static_cast<unsigned char>(header[4]);
-	ex.badMinor = static_cast<unsigned char>(header[5]);
-	ex.major = static_cast<unsigned char>(protocolMajor);
-	ex.minor = static_cast<unsigned char>(protocolMinor);
-	throw ex;
+	throwUnsupportedProtocolException(__FILE__, __LINE__, header[4], header[5], protocolMajor, protocolMinor);
     }
     if(header[6] != encodingMajor)
     {
-	UnsupportedEncodingException ex(__FILE__, __LINE__);
-	ex.badMajor = static_cast<unsigned char>(header[6]);
-	ex.badMinor = static_cast<unsigned char>(header[7]);
-	ex.major = static_cast<unsigned char>(encodingMajor);
-	ex.minor = static_cast<unsigned char>(encodingMinor);
-	throw ex;
+	throwUnsupportedEncodingException(__FILE__, __LINE__, header[6], header[7], encodingMajor, encodingMinor);
     }
     const Byte messageType = header[8];
     if(header[9] == 2)
     {
-	FeatureNotSupportedException ex(__FILE__, __LINE__);
-	ex.unsupportedFeature = "compression";
-	throw ex;
+	throw FeatureNotSupportedException(__FILE__, __LINE__, "compression");
     }
     
     Int size;
@@ -1487,11 +1524,11 @@ Ice::Connection::readStreamAndParseMessage(IceInternal::BasicStream& stream, Int
     stream.read(size);
     if(size < headerSize)
     {
-	throw IllegalMessageSizeException(__FILE__, __LINE__);
+	throwIllegalMessageSizeException(__FILE__, __LINE__);
     }
     if(size > static_cast<Int>(_instance->messageSizeMax()))
     {
-	throw MemoryLimitException(__FILE__, __LINE__);
+	throwMemoryLimitException(__FILE__, __LINE__);
     }
     if(size > static_cast<Int>(stream.b.size()))
     {
@@ -1551,7 +1588,7 @@ Ice::Connection::readStreamAndParseMessage(IceInternal::BasicStream& stream, Int
 	if(invokeNum < 0)
 	{
 	    invokeNum = 0;
-	    throw NegativeSizeException(__FILE__, __LINE__);
+	    throwNegativeSizeException(__FILE__, __LINE__);
 	}
 	break;
     }
@@ -1577,7 +1614,7 @@ Ice::Connection::readStreamAndParseMessage(IceInternal::BasicStream& stream, Int
 	{
 	    traceHeader("received unknown message\n(invalid, closing connection)", stream, _logger, _traceLevels);
 	}
-	throw UnknownMessageException(__FILE__, __LINE__);
+	throwUnknownMessageException(__FILE__, __LINE__);
 	break;
     }
     }
@@ -1709,7 +1746,7 @@ Ice::Connection::run()
 			
 			if(p == _requests.end())
 			{
-			    throw UnknownRequestIdException(__FILE__, __LINE__);
+			    throwUnknownRequestIdException(__FILE__, __LINE__);
 			}
 			
 			p->second->finished(_stream);
@@ -1795,14 +1832,14 @@ Ice::Connection::run()
 		    // Add the reply header and request id.
 		    //
 		    BasicStream* os = _in.os();
-		    os->writeBlob(&_replyHdr[0], headerSize);		
+		    os->writeBlob(replyHdr, sizeof(replyHdr));		
 		    os->write(requestId);
 		}
 	    
 		//
 		// Dispatch the incoming request.
 		//
-		_in.invoke(response);
+		_in.invoke(response, requestId);
 	    }
 	}
 	catch(const LocalException& ex)
@@ -1859,17 +1896,17 @@ Ice::Connection::ThreadPerConnection::run()
     }
     catch(const Exception& ex)
     {	
-	Error out(_connection->_instance->logger());
+	Error out(_connection->_logger);
 	out << "exception in thread per connection:\n" << _connection->toString() << ex.toString(); 
     }
     catch(const std::exception& ex)
     {
-	Error out(_connection->_instance->logger());
+	Error out(_connection->_logger);
 	out << "std::exception in thread per connection:\n" << _connection->toString() << ex.what();
     }
     catch(...)
     {
-	Error out(_connection->_instance->logger());
+	Error out(_connection->_logger);
 	out << "unknown exception in thread per connection:\n" << _connection->toString();
     }
 
