@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -13,5 +13,4 @@ using namespace std;
 using namespace Ice;
 using namespace IceInternal;
 
-void IceInternal::incRef(Endpoint* p) { p->__incRef(); }
-void IceInternal::decRef(Endpoint* p) { p->__decRef(); }
+IceUtil::Shared* IceInternal::upCast(Endpoint* p) { return p; }
