@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -20,7 +20,7 @@ namespace IceInternal
 
 class BasicStream;
 
-class ICE_API EndpointFactory : public ::IceUtil::Shared
+class EndpointFactory : public ::IceUtil::Shared
 {
 public:
 
