@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -9,7 +9,7 @@
 
 #include <IceE/LocalException.h>
 #include <IceE/SafeStdio.h>
-#include <IceE/IdentityUtil.h>
+#include <IceE/StringUtil.h>
 #include <IceE/Network.h>
 
 using namespace std;
@@ -21,6 +21,10 @@ Ice::LocalException::LocalException(const char* file, int line) :
 {
 }
 
+Ice::LocalException::~LocalException() throw()
+{
+}
+
 Ice::AlreadyRegisteredException::AlreadyRegisteredException(const char* __file, int __line) :
 #if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
     LocalException(__file, __line)
@@ -41,9 +45,13 @@ Ice::AlreadyRegisteredException::AlreadyRegisteredException(const char* __file,
 {
 }
 
+Ice::AlreadyRegisteredException::~AlreadyRegisteredException() throw()
+{
+}
+
 static const char* __Ice__AlreadyRegisteredException_name = "Ice::AlreadyRegisteredException";
 
-const ::std::string
+::std::string
 Ice::AlreadyRegisteredException::ice_name() const
 {
     return __Ice__AlreadyRegisteredException_name;
@@ -81,9 +89,13 @@ Ice::NotRegisteredException::NotRegisteredException(const char* __file, int __li
 {
 }
 
+Ice::NotRegisteredException::~NotRegisteredException() throw()
+{
+}
+
 static const char* __Ice__NotRegisteredException_name = "Ice::NotRegisteredException";
 
-const ::std::string
+::std::string
 Ice::NotRegisteredException::ice_name() const
 {
     return __Ice__NotRegisteredException_name;
@@ -120,9 +132,13 @@ Ice::TwowayOnlyException::TwowayOnlyException(const char* __file, int __line, co
 {
 }
 
+Ice::TwowayOnlyException::~TwowayOnlyException() throw()
+{
+}
+
 static const char* __Ice__TwowayOnlyException_name = "Ice::TwowayOnlyException";
 
-const ::std::string
+::std::string
 Ice::TwowayOnlyException::ice_name() const
 {
     return __Ice__TwowayOnlyException_name;
@@ -149,9 +165,13 @@ Ice::CloneNotImplementedException::CloneNotImplementedException(const char* __fi
 {
 }
 
+Ice::CloneNotImplementedException::~CloneNotImplementedException() throw()
+{
+}
+
 static const char* __Ice__CloneNotImplementedException_name = "Ice::CloneNotImplementedException";
 
-const ::std::string
+::std::string
 Ice::CloneNotImplementedException::ice_name() const
 {
     return __Ice__CloneNotImplementedException_name;
@@ -188,9 +208,13 @@ Ice::UnknownException::UnknownException(const char* __file, int __line, const ::
 {
 }
 
+Ice::UnknownException::~UnknownException() throw()
+{
+}
+
 static const char* __Ice__UnknownException_name = "Ice::UnknownException";
 
-const ::std::string
+::std::string
 Ice::UnknownException::ice_name() const
 {
     return __Ice__UnknownException_name;
@@ -226,9 +250,13 @@ Ice::UnknownLocalException::UnknownLocalException(const char* __file, int __line
 {
 }
 
+Ice::UnknownLocalException::~UnknownLocalException() throw()
+{
+}
+
 static const char* __Ice__UnknownLocalException_name = "Ice::UnknownLocalException";
 
-const ::std::string
+::std::string
 Ice::UnknownLocalException::ice_name() const
 {
     return __Ice__UnknownLocalException_name;
@@ -264,9 +292,13 @@ Ice::UnknownUserException::UnknownUserException(const char* __file, int __line,
 {
 }
 
+Ice::UnknownUserException::~UnknownUserException() throw()
+{
+}
+
 static const char* __Ice__UnknownUserException_name = "Ice::UnknownUserException";
 
-const ::std::string
+::std::string
 Ice::UnknownUserException::ice_name() const
 {
     return __Ice__UnknownUserException_name;
@@ -293,9 +325,13 @@ Ice::VersionMismatchException::VersionMismatchException(const char* __file, int
 {
 }
 
+Ice::VersionMismatchException::~VersionMismatchException() throw()
+{
+}
+
 static const char* __Ice__VersionMismatchException_name = "Ice::VersionMismatchException";
 
-const ::std::string
+::std::string
 Ice::VersionMismatchException::ice_name() const
 {
     return __Ice__VersionMismatchException_name;
@@ -322,9 +358,13 @@ Ice::CommunicatorDestroyedException::CommunicatorDestroyedException(const char*
 {
 }
 
+Ice::CommunicatorDestroyedException::~CommunicatorDestroyedException() throw()
+{
+}
+
 static const char* __Ice__CommunicatorDestroyedException_name = "Ice::CommunicatorDestroyedException";
 
-const ::std::string
+::std::string
 Ice::CommunicatorDestroyedException::ice_name() const
 {
     return __Ice__CommunicatorDestroyedException_name;
@@ -363,9 +403,13 @@ Ice::ObjectAdapterDeactivatedException::ObjectAdapterDeactivatedException(const
 {
 }
 
+Ice::ObjectAdapterDeactivatedException::~ObjectAdapterDeactivatedException() throw()
+{
+}
+
 static const char* __Ice__ObjectAdapterDeactivatedException_name = "Ice::ObjectAdapterDeactivatedException";
 
-const ::std::string
+::std::string
 Ice::ObjectAdapterDeactivatedException::ice_name() const
 {
     return __Ice__ObjectAdapterDeactivatedException_name;
@@ -411,9 +455,13 @@ Ice::ObjectAdapterIdInUseException::ObjectAdapterIdInUseException(const char* __
 {
 }
 
+Ice::ObjectAdapterIdInUseException::~ObjectAdapterIdInUseException() throw()
+{
+}
+
 static const char* __Ice__ObjectAdapterIdInUseException_name = "Ice::ObjectAdapterIdInUseException";
 
-const ::std::string
+::std::string
 Ice::ObjectAdapterIdInUseException::ice_name() const
 {
     return __Ice__ObjectAdapterIdInUseException_name;
@@ -466,9 +514,13 @@ Ice::NoEndpointException::NoEndpointException(const char* __file, int __line, co
 {
 }
 
+Ice::NoEndpointException::~NoEndpointException() throw()
+{
+}
+
 static const char* __Ice__NoEndpointException_name = "Ice::NoEndpointException";
 
-const ::std::string
+::std::string
 Ice::NoEndpointException::ice_name() const
 {
     return __Ice__NoEndpointException_name;
@@ -505,9 +557,13 @@ Ice::EndpointParseException::EndpointParseException(const char* __file, int __li
 {
 }
 
+Ice::EndpointParseException::~EndpointParseException() throw()
+{
+}
+
 static const char* __Ice__EndpointParseException_name = "Ice::EndpointParseException";
 
-const ::std::string
+::std::string
 Ice::EndpointParseException::ice_name() const
 {
     return __Ice__EndpointParseException_name;
@@ -544,9 +600,13 @@ Ice::IdentityParseException::IdentityParseException(const char* __file, int __li
 {
 }
 
+Ice::IdentityParseException::~IdentityParseException() throw()
+{
+}
+
 static const char* __Ice__IdentityParseException_name = "Ice::IdentityParseException";
 
-const ::std::string
+::std::string
 Ice::IdentityParseException::ice_name() const
 {
     return __Ice__IdentityParseException_name;
@@ -583,9 +643,13 @@ Ice::ProxyParseException::ProxyParseException(const char* __file, int __line, co
 {
 }
 
+Ice::ProxyParseException::~ProxyParseException() throw()
+{
+}
+
 static const char* __Ice__ProxyParseException_name = "Ice::ProxyParseException";
 
-const ::std::string
+::std::string
 Ice::ProxyParseException::ice_name() const
 {
     return __Ice__ProxyParseException_name;
@@ -622,9 +686,13 @@ Ice::IllegalIdentityException::IllegalIdentityException(const char* __file, int
 {
 }
 
+Ice::IllegalIdentityException::~IllegalIdentityException() throw()
+{
+}
+
 static const char* __Ice__IllegalIdentityException_name = "Ice::IllegalIdentityException";
 
-const ::std::string
+::std::string
 Ice::IllegalIdentityException::ice_name() const
 {
     return __Ice__IllegalIdentityException_name;
@@ -663,9 +731,13 @@ Ice::RequestFailedException::RequestFailedException(const char* __file, int __li
 {
 }
 
+Ice::RequestFailedException::~RequestFailedException() throw()
+{
+}
+
 static const char* __Ice__RequestFailedException_name = "Ice::RequestFailedException";
 
-const ::std::string
+::std::string
 Ice::RequestFailedException::ice_name() const
 {
     return __Ice__RequestFailedException_name;
@@ -701,9 +773,13 @@ Ice::ObjectNotExistException::ObjectNotExistException(const char* __file, int __
 {
 }
 
+Ice::ObjectNotExistException::~ObjectNotExistException() throw()
+{
+}
+
 static const char* __Ice__ObjectNotExistException_name = "Ice::ObjectNotExistException";
 
-const ::std::string
+::std::string
 Ice::ObjectNotExistException::ice_name() const
 {
     return __Ice__ObjectNotExistException_name;
@@ -739,9 +815,13 @@ Ice::FacetNotExistException::FacetNotExistException(const char* __file, int __li
 {
 }
 
+Ice::FacetNotExistException::~FacetNotExistException() throw()
+{
+}
+
 static const char* __Ice__FacetNotExistException_name = "Ice::FacetNotExistException";
 
-const ::std::string
+::std::string
 Ice::FacetNotExistException::ice_name() const
 {
     return __Ice__FacetNotExistException_name;
@@ -777,9 +857,13 @@ Ice::OperationNotExistException::OperationNotExistException(const char* __file,
 {
 }
 
+Ice::OperationNotExistException::~OperationNotExistException() throw()
+{
+}
+
 static const char* __Ice__OperationNotExistException_name = "Ice::OperationNotExistException";
 
-const ::std::string
+::std::string
 Ice::OperationNotExistException::ice_name() const
 {
     return __Ice__OperationNotExistException_name;
@@ -816,9 +900,13 @@ Ice::SyscallException::SyscallException(const char* __file, int __line, ::Ice::I
 {
 }
 
+Ice::SyscallException::~SyscallException() throw()
+{
+}
+
 static const char* __Ice__SyscallException_name = "Ice::SyscallException";
 
-const ::std::string
+::std::string
 Ice::SyscallException::ice_name() const
 {
     return __Ice__SyscallException_name;
@@ -854,9 +942,13 @@ Ice::SocketException::SocketException(const char* __file, int __line, ::Ice::Int
 {
 }
 
+Ice::SocketException::~SocketException() throw()
+{
+}
+
 static const char* __Ice__SocketException_name = "Ice::SocketException";
 
-const ::std::string
+::std::string
 Ice::SocketException::ice_name() const
 {
     return __Ice__SocketException_name;
@@ -893,9 +985,13 @@ Ice::FileException::FileException(const char* __file, int __line, ::Ice::Int __e
 {
 }
 
+Ice::FileException::~FileException() throw()
+{
+}
+
 static const char* __Ice__FileException_name = "Ice::FileException";
 
-const ::std::string
+::std::string
 Ice::FileException::ice_name() const
 {
     return __Ice__FileException_name;
@@ -931,9 +1027,13 @@ Ice::ConnectFailedException::ConnectFailedException(const char* __file, int __li
 {
 }
 
+Ice::ConnectFailedException::~ConnectFailedException() throw()
+{
+}
+
 static const char* __Ice__ConnectFailedException_name = "Ice::ConnectFailedException";
 
-const ::std::string
+::std::string
 Ice::ConnectFailedException::ice_name() const
 {
     return __Ice__ConnectFailedException_name;
@@ -969,9 +1069,13 @@ Ice::ConnectionRefusedException::ConnectionRefusedException(const char* __file,
 {
 }
 
+Ice::ConnectionRefusedException::~ConnectionRefusedException() throw()
+{
+}
+
 static const char* __Ice__ConnectionRefusedException_name = "Ice::ConnectionRefusedException";
 
-const ::std::string
+::std::string
 Ice::ConnectionRefusedException::ice_name() const
 {
     return __Ice__ConnectionRefusedException_name;
@@ -1007,9 +1111,13 @@ Ice::ConnectionLostException::ConnectionLostException(const char* __file, int __
 {
 }
 
+Ice::ConnectionLostException::~ConnectionLostException() throw()
+{
+}
+
 static const char* __Ice__ConnectionLostException_name = "Ice::ConnectionLostException";
 
-const ::std::string
+::std::string
 Ice::ConnectionLostException::ice_name() const
 {
     return __Ice__ConnectionLostException_name;
@@ -1047,9 +1155,13 @@ Ice::DNSException::DNSException(const char* __file, int __line, ::Ice::Int __err
 {
 }
 
+Ice::DNSException::~DNSException() throw()
+{
+}
+
 static const char* __Ice__DNSException_name = "Ice::DNSException";
 
-const ::std::string
+::std::string
 Ice::DNSException::ice_name() const
 {
     return __Ice__DNSException_name;
@@ -1076,9 +1188,13 @@ Ice::TimeoutException::TimeoutException(const char* __file, int __line) :
 {
 }
 
+Ice::TimeoutException::~TimeoutException() throw()
+{
+}
+
 static const char* __Ice__TimeoutException_name = "Ice::TimeoutException";
 
-const ::std::string
+::std::string
 Ice::TimeoutException::ice_name() const
 {
     return __Ice__TimeoutException_name;
@@ -1105,9 +1221,13 @@ Ice::ConnectTimeoutException::ConnectTimeoutException(const char* __file, int __
 {
 }
 
+Ice::ConnectTimeoutException::~ConnectTimeoutException() throw()
+{
+}
+
 static const char* __Ice__ConnectTimeoutException_name = "Ice::ConnectTimeoutException";
 
-const ::std::string
+::std::string
 Ice::ConnectTimeoutException::ice_name() const
 {
     return __Ice__ConnectTimeoutException_name;
@@ -1134,9 +1254,13 @@ Ice::CloseTimeoutException::CloseTimeoutException(const char* __file, int __line
 {
 }
 
+Ice::CloseTimeoutException::~CloseTimeoutException() throw()
+{
+}
+
 static const char* __Ice__CloseTimeoutException_name = "Ice::CloseTimeoutException";
 
-const ::std::string
+::std::string
 Ice::CloseTimeoutException::ice_name() const
 {
     return __Ice__CloseTimeoutException_name;
@@ -1163,9 +1287,23 @@ Ice::ProtocolException::ProtocolException(const char* __file, int __line) :
 {
 }
 
+Ice::ProtocolException::ProtocolException(const char* __file, int __line, const ::std::string& __reason) :
+#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
+    LocalException(__file, __line),
+#else
+    ::Ice::LocalException(__file, __line),
+#endif
+    reason(__reason)
+{
+}
+
+Ice::ProtocolException::~ProtocolException() throw()
+{
+}
+
 static const char* __Ice__ProtocolException_name = "Ice::ProtocolException";
 
-const ::std::string
+::std::string
 Ice::ProtocolException::ice_name() const
 {
     return __Ice__ProtocolException_name;
@@ -1183,245 +1321,6 @@ Ice::ProtocolException::ice_throw() const
     throw *this;
 }
 
-Ice::BadMagicException::BadMagicException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line)
-#else
-    ::Ice::ProtocolException(__file, __line)
-#endif
-{
-}
-
-Ice::BadMagicException::BadMagicException(const char* __file, int __line, const ::Ice::ByteSeq& __badMagic) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line),
-#else
-    ::Ice::ProtocolException(__file, __line),
-#endif
-    badMagic(__badMagic)
-{
-}
-
-static const char* __Ice__BadMagicException_name = "Ice::BadMagicException";
-
-const ::std::string
-Ice::BadMagicException::ice_name() const
-{
-    return __Ice__BadMagicException_name;
-}
-
-::Ice::Exception*
-Ice::BadMagicException::ice_clone() const
-{
-    return new BadMagicException(*this);
-}
-
-void
-Ice::BadMagicException::ice_throw() const
-{
-    throw *this;
-}
-
-Ice::UnsupportedProtocolException::UnsupportedProtocolException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line)
-#else
-    ::Ice::ProtocolException(__file, __line)
-#endif
-{
-}
-
-Ice::UnsupportedProtocolException::UnsupportedProtocolException(const char* __file, int __line, ::Ice::Int __badMajor, ::Ice::Int __badMinor, ::Ice::Int __major, ::Ice::Int __minor) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line),
-#else
-    ::Ice::ProtocolException(__file, __line),
-#endif
-    badMajor(__badMajor),
-    badMinor(__badMinor),
-    major(__major),
-    minor(__minor)
-{
-}
-
-static const char* __Ice__UnsupportedProtocolException_name = "Ice::UnsupportedProtocolException";
-
-const ::std::string
-Ice::UnsupportedProtocolException::ice_name() const
-{
-    return __Ice__UnsupportedProtocolException_name;
-}
-
-::Ice::Exception*
-Ice::UnsupportedProtocolException::ice_clone() const
-{
-    return new UnsupportedProtocolException(*this);
-}
-
-void
-Ice::UnsupportedProtocolException::ice_throw() const
-{
-    throw *this;
-}
-
-Ice::UnsupportedEncodingException::UnsupportedEncodingException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line)
-#else
-    ::Ice::ProtocolException(__file, __line)
-#endif
-{
-}
-
-Ice::UnsupportedEncodingException::UnsupportedEncodingException(const char* __file, int __line, ::Ice::Int __badMajor, ::Ice::Int __badMinor, ::Ice::Int __major, ::Ice::Int __minor) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line),
-#else
-    ::Ice::ProtocolException(__file, __line),
-#endif
-    badMajor(__badMajor),
-    badMinor(__badMinor),
-    major(__major),
-    minor(__minor)
-{
-}
-
-static const char* __Ice__UnsupportedEncodingException_name = "Ice::UnsupportedEncodingException";
-
-const ::std::string
-Ice::UnsupportedEncodingException::ice_name() const
-{
-    return __Ice__UnsupportedEncodingException_name;
-}
-
-::Ice::Exception*
-Ice::UnsupportedEncodingException::ice_clone() const
-{
-    return new UnsupportedEncodingException(*this);
-}
-
-void
-Ice::UnsupportedEncodingException::ice_throw() const
-{
-    throw *this;
-}
-
-Ice::UnknownMessageException::UnknownMessageException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line)
-#else
-    ::Ice::ProtocolException(__file, __line)
-#endif
-{
-}
-
-static const char* __Ice__UnknownMessageException_name = "Ice::UnknownMessageException";
-
-const ::std::string
-Ice::UnknownMessageException::ice_name() const
-{
-    return __Ice__UnknownMessageException_name;
-}
-
-::Ice::Exception*
-Ice::UnknownMessageException::ice_clone() const
-{
-    return new UnknownMessageException(*this);
-}
-
-void
-Ice::UnknownMessageException::ice_throw() const
-{
-    throw *this;
-}
-
-Ice::ConnectionNotValidatedException::ConnectionNotValidatedException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line)
-#else
-    ::Ice::ProtocolException(__file, __line)
-#endif
-{
-}
-
-static const char* __Ice__ConnectionNotValidatedException_name = "Ice::ConnectionNotValidatedException";
-
-const ::std::string
-Ice::ConnectionNotValidatedException::ice_name() const
-{
-    return __Ice__ConnectionNotValidatedException_name;
-}
-
-::Ice::Exception*
-Ice::ConnectionNotValidatedException::ice_clone() const
-{
-    return new ConnectionNotValidatedException(*this);
-}
-
-void
-Ice::ConnectionNotValidatedException::ice_throw() const
-{
-    throw *this;
-}
-
-Ice::UnknownRequestIdException::UnknownRequestIdException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line)
-#else
-    ::Ice::ProtocolException(__file, __line)
-#endif
-{
-}
-
-static const char* __Ice__UnknownRequestIdException_name = "Ice::UnknownRequestIdException";
-
-const ::std::string
-Ice::UnknownRequestIdException::ice_name() const
-{
-    return __Ice__UnknownRequestIdException_name;
-}
-
-::Ice::Exception*
-Ice::UnknownRequestIdException::ice_clone() const
-{
-    return new UnknownRequestIdException(*this);
-}
-
-void
-Ice::UnknownRequestIdException::ice_throw() const
-{
-    throw *this;
-}
-
-Ice::UnknownReplyStatusException::UnknownReplyStatusException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line)
-#else
-    ::Ice::ProtocolException(__file, __line)
-#endif
-{
-}
-
-static const char* __Ice__UnknownReplyStatusException_name = "Ice::UnknownReplyStatusException";
-
-const ::std::string
-Ice::UnknownReplyStatusException::ice_name() const
-{
-    return __Ice__UnknownReplyStatusException_name;
-}
-
-::Ice::Exception*
-Ice::UnknownReplyStatusException::ice_clone() const
-{
-    return new UnknownReplyStatusException(*this);
-}
-
-void
-Ice::UnknownReplyStatusException::ice_throw() const
-{
-    throw *this;
-}
-
 Ice::CloseConnectionException::CloseConnectionException(const char* __file, int __line) :
 #if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
     ProtocolException(__file, __line)
@@ -1431,9 +1330,13 @@ Ice::CloseConnectionException::CloseConnectionException(const char* __file, int
 {
 }
 
+Ice::CloseConnectionException::~CloseConnectionException() throw()
+{
+}
+
 static const char* __Ice__CloseConnectionException_name = "Ice::CloseConnectionException";
 
-const ::std::string
+::std::string
 Ice::CloseConnectionException::ice_name() const
 {
     return __Ice__CloseConnectionException_name;
@@ -1460,9 +1363,13 @@ Ice::ForcedCloseConnectionException::ForcedCloseConnectionException(const char*
 {
 }
 
+Ice::ForcedCloseConnectionException::~ForcedCloseConnectionException() throw()
+{
+}
+
 static const char* __Ice__ForcedCloseConnectionException_name = "Ice::ForcedCloseConnectionException";
 
-const ::std::string
+::std::string
 Ice::ForcedCloseConnectionException::ice_name() const
 {
     return __Ice__ForcedCloseConnectionException_name;
@@ -1480,62 +1387,32 @@ Ice::ForcedCloseConnectionException::ice_throw() const
     throw *this;
 }
 
-Ice::IllegalMessageSizeException::IllegalMessageSizeException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line)
-#else
-    ::Ice::ProtocolException(__file, __line)
-#endif
+void
+Ice::throwBadMagicException(const char* file, int line, const ::Ice::ByteSeq& badMagic)
 {
-}
-
-static const char* __Ice__IllegalMessageSizeException_name = "Ice::IllegalMessageSizeException";
-
-const ::std::string
-Ice::IllegalMessageSizeException::ice_name() const
-{
-    return __Ice__IllegalMessageSizeException_name;
-}
-
-::Ice::Exception*
-Ice::IllegalMessageSizeException::ice_clone() const
-{
-    return new IllegalMessageSizeException(*this);
+    string out = "unknown magic number: ";
+    out += Ice::printfToString("0x%2X, 0x%2X, 0x%2X, 0x%2X", badMagic[0], badMagic[1], badMagic[2], badMagic[3]);
+    throw ProtocolException(__FILE__, __LINE__, out);
 }
 
 void
-Ice::IllegalMessageSizeException::ice_throw() const
+Ice::throwUnsupportedProtocolException(const char* file, int line, Int badMajor, Int badMinor, Int major, Int minor)
 {
-    throw *this;
-}
-
-Ice::FeatureNotSupportedException::FeatureNotSupportedException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    LocalException(__file, __line)
-#else
-    ::Ice::LocalException(__file, __line)
-#endif
-{
-}
-
-static const char* __Ice__FeatureNotSupportedException_name = "Ice::FeatureNotSupportedException";
-
-const ::std::string
-Ice::FeatureNotSupportedException::ice_name() const
-{
-    return __Ice__FeatureNotSupportedException_name;
-}
-
-::Ice::Exception*
-Ice::FeatureNotSupportedException::ice_clone() const
-{
-    return new FeatureNotSupportedException(*this);
+    string out = "unsupported protocol version: ";
+    out += Ice::printfToString("%d.%d", badMajor, badMinor);
+    out += "\n(can only support protocols compatible with version ";
+    out += Ice::printfToString("%d.%d", major, minor) + ")";
+    throw ProtocolException(__FILE__, __LINE__, out);
 }
 
 void
-Ice::FeatureNotSupportedException::ice_throw() const
+Ice::throwUnsupportedEncodingException(const char* file, int line, Int badMajor, Int badMinor, Int major, Int minor)
 {
-    throw *this;
+    string out = "unsupported encoding version: ";
+    out += Ice::printfToString("%d.%d", badMajor, badMinor);
+    out += "\n(can only support protocols compatible with version ";
+    out += Ice::printfToString("%d.%d", major, minor) + ")";
+    throw ProtocolException(__FILE__, __LINE__, out);
 }
 
 Ice::MarshalException::MarshalException(const char* __file, int __line) :
@@ -1549,17 +1426,20 @@ Ice::MarshalException::MarshalException(const char* __file, int __line) :
 
 Ice::MarshalException::MarshalException(const char* __file, int __line, const ::std::string& __reason) :
 #if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    ProtocolException(__file, __line),
+    ProtocolException(__file, __line, __reason)
 #else
-    ::Ice::ProtocolException(__file, __line),
+    ::Ice::ProtocolException(__file, __line, __reason)
 #endif
-    reason(__reason)
+{
+}
+
+Ice::MarshalException::~MarshalException() throw()
 {
 }
 
 static const char* __Ice__MarshalException_name = "Ice::MarshalException";
 
-const ::std::string
+::std::string
 Ice::MarshalException::ice_name() const
 {
     return __Ice__MarshalException_name;
@@ -1577,82 +1457,6 @@ Ice::MarshalException::ice_throw() const
     throw *this;
 }
 
-Ice::ProxyUnmarshalException::ProxyUnmarshalException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    MarshalException(__file, __line)
-#else
-    ::Ice::MarshalException(__file, __line)
-#endif
-{
-}
-
-Ice::ProxyUnmarshalException::ProxyUnmarshalException(const char* __file, int __line, const ::std::string& __reason) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    MarshalException(__file, __line, __reason)
-#else
-    ::Ice::MarshalException(__file, __line, __reason)
-#endif
-{
-}
-
-static const char* __Ice__ProxyUnmarshalException_name = "Ice::ProxyUnmarshalException";
-
-const ::std::string
-Ice::ProxyUnmarshalException::ice_name() const
-{
-    return __Ice__ProxyUnmarshalException_name;
-}
-
-::Ice::Exception*
-Ice::ProxyUnmarshalException::ice_clone() const
-{
-    return new ProxyUnmarshalException(*this);
-}
-
-void
-Ice::ProxyUnmarshalException::ice_throw() const
-{
-    throw *this;
-}
-
-Ice::UnmarshalOutOfBoundsException::UnmarshalOutOfBoundsException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    MarshalException(__file, __line)
-#else
-    ::Ice::MarshalException(__file, __line)
-#endif
-{
-}
-
-Ice::UnmarshalOutOfBoundsException::UnmarshalOutOfBoundsException(const char* __file, int __line, const ::std::string& __reason) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    MarshalException(__file, __line, __reason)
-#else
-    ::Ice::MarshalException(__file, __line, __reason)
-#endif
-{
-}
-
-static const char* __Ice__UnmarshalOutOfBoundsException_name = "Ice::UnmarshalOutOfBoundsException";
-
-const ::std::string
-Ice::UnmarshalOutOfBoundsException::ice_name() const
-{
-    return __Ice__UnmarshalOutOfBoundsException_name;
-}
-
-::Ice::Exception*
-Ice::UnmarshalOutOfBoundsException::ice_clone() const
-{
-    return new UnmarshalOutOfBoundsException(*this);
-}
-
-void
-Ice::UnmarshalOutOfBoundsException::ice_throw() const
-{
-    throw *this;
-}
-
 Ice::MemoryLimitException::MemoryLimitException(const char* __file, int __line) :
 #if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
     MarshalException(__file, __line)
@@ -1671,9 +1475,13 @@ Ice::MemoryLimitException::MemoryLimitException(const char* __file, int __line,
 {
 }
 
+Ice::MemoryLimitException::~MemoryLimitException() throw()
+{
+}
+
 static const char* __Ice__MemoryLimitException_name = "Ice::MemoryLimitException";
 
-const ::std::string
+::std::string
 Ice::MemoryLimitException::ice_name() const
 {
     return __Ice__MemoryLimitException_name;
@@ -1691,7 +1499,102 @@ Ice::MemoryLimitException::ice_throw() const
     throw *this;
 }
 
-Ice::EncapsulationException::EncapsulationException(const char* __file, int __line) :
+void
+Ice::throwMemoryLimitException(const char* file, int line)
+{
+     throw MemoryLimitException(file, line);
+}
+
+void
+Ice::throwUnmarshalOutOfBoundsException(const char* file, int line)
+{
+     throw MarshalException(file, line, "out of bounds during unmarshaling");
+}
+
+void
+Ice::throwNegativeSizeException(const char* file, int line)
+{
+    throw MarshalException(file, line, "negative size for sequence, dictionary, etc.");
+}
+
+Ice::FeatureNotSupportedException::FeatureNotSupportedException(const char* __file, int __line) :
+#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
+    LocalException(__file, __line)
+#else
+    ::Ice::LocalException(__file, __line)
+#endif
+{
+}
+
+Ice::FeatureNotSupportedException::FeatureNotSupportedException(const char* __file, int __line, const string& __unsupportedFeature) :
+#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
+    LocalException(__file, __line),
+#else
+    ::Ice::LocalException(__file, __line),
+#endif
+    unsupportedFeature(__unsupportedFeature)
+{
+}
+
+Ice::FeatureNotSupportedException::~FeatureNotSupportedException() throw()
+{
+}
+
+static const char* __Ice__FeatureNotSupportedException_name = "Ice::FeatureNotSupportedException";
+
+::std::string
+Ice::FeatureNotSupportedException::ice_name() const
+{
+    return __Ice__FeatureNotSupportedException_name;
+}
+
+::Ice::Exception*
+Ice::FeatureNotSupportedException::ice_clone() const
+{
+    return new FeatureNotSupportedException(*this);
+}
+
+void
+Ice::FeatureNotSupportedException::ice_throw() const
+{
+    throw *this;
+}
+
+Ice::FixedProxyException::FixedProxyException(const char* __file, int __line) :
+#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
+    LocalException(__file, __line)
+#else
+    ::Ice::LocalException(__file, __line)
+#endif
+{
+}
+
+Ice::FixedProxyException::~FixedProxyException() throw()
+{
+}
+
+static const char* __Ice__FixedProxyException_name = "Ice::FixedProxyException";
+
+::std::string
+Ice::FixedProxyException::ice_name() const
+{
+    return __Ice__FixedProxyException_name;
+}
+
+::Ice::Exception*
+Ice::FixedProxyException::ice_clone() const
+{
+    return new FixedProxyException(*this);
+}
+
+void
+Ice::FixedProxyException::ice_throw() const
+{
+    throw *this;
+}
+
+#ifdef ICEE_HAS_WSTRING
+Ice::StringConversionException::StringConversionException(const char* __file, int __line) :
 #if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
     MarshalException(__file, __line)
 #else
@@ -1700,7 +1603,7 @@ Ice::EncapsulationException::EncapsulationException(const char* __file, int __li
 {
 }
 
-Ice::EncapsulationException::EncapsulationException(const char* __file, int __line, const ::std::string& __reason) :
+Ice::StringConversionException::StringConversionException(const char* __file, int __line, const ::std::string& __reason) :  
 #if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
     MarshalException(__file, __line, __reason)
 #else
@@ -1709,63 +1612,30 @@ Ice::EncapsulationException::EncapsulationException(const char* __file, int __li
 {
 }
 
-static const char* __Ice__EncapsulationException_name = "Ice::EncapsulationException";
-
-const ::std::string
-Ice::EncapsulationException::ice_name() const
+Ice::StringConversionException::~StringConversionException() throw()
 {
-    return __Ice__EncapsulationException_name;
+}
+
+static const char* __Ice__StringConversionException_name = "Ice::StringConversionException";
+
+::std::string
+Ice::StringConversionException::ice_name() const
+{
+    return __Ice__StringConversionException_name;
 }
 
 ::Ice::Exception*
-Ice::EncapsulationException::ice_clone() const
+Ice::StringConversionException::ice_clone() const
 {
-    return new EncapsulationException(*this);
+    return new StringConversionException(*this);
 }
 
 void
-Ice::EncapsulationException::ice_throw() const
+Ice::StringConversionException::ice_throw() const
 {
     throw *this;
 }
-
-Ice::NegativeSizeException::NegativeSizeException(const char* __file, int __line) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    MarshalException(__file, __line)
-#else
-    ::Ice::MarshalException(__file, __line)
 #endif
-{
-}
-
-Ice::NegativeSizeException::NegativeSizeException(const char* __file, int __line, const ::std::string& __reason) :
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    MarshalException(__file, __line, __reason)
-#else
-    ::Ice::MarshalException(__file, __line, __reason)
-#endif
-{
-}
-
-static const char* __Ice__NegativeSizeException_name = "Ice::NegativeSizeException";
-
-const ::std::string
-Ice::NegativeSizeException::ice_name() const
-{
-    return __Ice__NegativeSizeException_name;
-}
-
-::Ice::Exception*
-Ice::NegativeSizeException::ice_clone() const
-{
-    return new NegativeSizeException(*this);
-}
-
-void
-Ice::NegativeSizeException::ice_throw() const
-{
-    throw *this;
-}
 
 string
 Ice::UnknownException::toString() const
@@ -1871,7 +1741,14 @@ Ice::IllegalIdentityException::toString() const
 {
     string out = Exception::toString();
     out += ":\nillegal identity: `";
-    out += identityToString(id);
+    if(id.category.empty())
+    {
+        out += IceUtil::escapeString(id.name, "/");
+    }
+    else
+    {
+        out += IceUtil::escapeString(id.category, "/") + '/' + IceUtil::escapeString(id.name, "/");
+    }
     out += "'";
     return out;
 }
@@ -1880,7 +1757,14 @@ static void
 printFailedRequestData(string& out, const RequestFailedException& ex)
 {
     out += "\nidentity: ";
-    out += identityToString(ex.id);
+    if(ex.id.category.empty())
+    {
+        out += IceUtil::escapeString(ex.id.name, "/");
+    }
+    else
+    {
+        out += IceUtil::escapeString(ex.id.category, "/") + '/' + IceUtil::escapeString(ex.id.name, "/");
+    }
     out += "\nfacet: ";
     out += ex.facet;
     out += "\noperation: ";
@@ -2041,76 +1925,15 @@ string
 Ice::ProtocolException::toString() const
 {
     string out = Exception::toString();
-    out += ":\nunknown protocol exception";
-    return out;
-}
-
-string
-Ice::BadMagicException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nunknown magic number: ";
-    out += Ice::printfToString("0x%2X, 0x%2X, 0x%2X, 0x%2X", badMagic[0], badMagic[1], badMagic[2], badMagic[3]);
-
-    return out;
-}
-
-string
-Ice::UnsupportedProtocolException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: unsupported protocol version: ";
-    out += Ice::printfToString("%d.%d", badMajor, badMinor);
-    out += "\n(can only support protocols compatible with version ";
-    out += Ice::printfToString("%d.%d", major, minor);
-    out += ")";
-    return out;
-}
-
-string
-Ice::UnsupportedEncodingException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: unsupported encoding version: ";
-    out += badMajor;
-    out += ".";
-    out += badMinor;
-    out += "\n(can only support encodings compatible with version ";
-    out += major;
-    out += ".";
-    out += minor;
-    return out;
-}
-
-string
-Ice::UnknownMessageException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: unknown message type";
-    return out;
-}
-
-string
-Ice::ConnectionNotValidatedException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: received message over unvalidated connection";
-    return out;
-}
-
-string
-Ice::UnknownRequestIdException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: unknown request id";
-    return out;
-}
-
-string
-Ice::UnknownReplyStatusException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: unknown reply status";
+    out += ":\nprotocol error: ";
+    if(!reason.empty())
+    {
+	out += reason;
+    }
+    else
+    {
+	out += "unknown protocol exception";
+    }
     return out;
 }
 
@@ -2130,21 +1953,16 @@ Ice::ForcedCloseConnectionException::toString() const
     return out;
 }
 
-string
-Ice::IllegalMessageSizeException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: illegal message size";
-    return out;
-}
-
 string
 Ice::FeatureNotSupportedException::toString() const
 {
     string out = Exception::toString();
-    out += ":\nfeature `";
-    out += unsupportedFeature;
-    out += "' is not supported.";
+    out += ":\nfeature not supported";
+    if(!unsupportedFeature.empty())
+    {
+        out += ": ";
+        out += unsupportedFeature;
+    }
     return out;
 }
 
@@ -2161,22 +1979,6 @@ Ice::MarshalException::toString() const
     return out;
 }
 
-string
-Ice::UnmarshalOutOfBoundsException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: out of bounds during unmarshaling";
-    return out;
-}
-
-string
-Ice::ProxyUnmarshalException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: inconsistent proxy data during unmarshaling";
-    return out;
-}
-
 string
 Ice::MemoryLimitException::toString() const
 {
@@ -2185,22 +1987,6 @@ Ice::MemoryLimitException::toString() const
     return out;
 }
 
-string
-Ice::EncapsulationException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: illegal encapsulation";
-    return out;
-}
-
-string
-Ice::NegativeSizeException::toString() const
-{
-    string out = Exception::toString();
-    out += ":\nprotocol error: negative size for sequence, dictionary, etc.";
-    return out;
-}
-
 string
 Ice::AlreadyRegisteredException::toString() const
 {
@@ -2242,3 +2028,21 @@ Ice::CloneNotImplementedException::toString() const
     out += ":\n ice_clone() must be implemented in classes derived from abstract base classes";
     return out;
 }
+
+string
+Ice::FixedProxyException::toString() const
+{
+    string out = Exception::toString();
+    out += ":\nfixed proxy exception";
+    return out;
+}
+
+#ifdef ICEE_HAS_WSTRING
+string
+Ice::StringConversionException::toString() const
+{
+    string out = Exception::toString();
+    out += ":\nprotocol error: string conversion failed";
+    return out;
+}
+#endif
