@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -14,7 +14,7 @@
 
 #if defined(_WIN32)
 #  include <winsock2.h>
-#elif defined(__linux) || defined(__APPLE__) || defined(__FreeBSD__)
+#elif defined(__APPLE__) || defined(__FreeBSD__)
 #  include <ifaddrs.h>
 #else
 #  include <sys/ioctl.h>
@@ -344,7 +344,7 @@ IceInternal::setBlock(SOCKET fd, bool block)
     }
 }
 
-#ifndef ICEE_USE_SELECT_FOR_TIMEOUTS
+#ifndef ICEE_USE_SELECT_OR_POLL_FOR_TIMEOUTS
 void
 IceInternal::setTimeout(SOCKET fd, bool recv, int timeout)
 {
@@ -549,30 +549,11 @@ repeatConnect:
 	    assert(nevents.lNetworkEvents & FD_CONNECT);
 	    val = nevents.iErrorCode[FD_CONNECT_BIT];
 #else
-	repeatSelect:
-
-	    int ret;
-	    fd_set wFdSet;
-	    FD_ZERO(&wFdSet);
-	    FD_SET(fd, &wFdSet);
-	    //
-	    // Note that although we use a different mechanism for
-	    // WIN32, winsock notifies about connection failures
-	    // through the exception filedescriptors
-	    //
-	    if(timeout >= 0)
-	    {
-		struct timeval tv;
-		tv.tv_sec = timeout / 1000;
-		tv.tv_usec = (timeout - tv.tv_sec * 1000) * 1000;
-		ret = ::select(fd + 1, 0, &wFdSet, 0, &tv);
-	    }
-	    else
-	    {
-
-		ret = ::select(fd + 1, 0, &wFdSet, 0, 0);
-	    }
-	    
+        repeatPoll:
+            struct pollfd pollFd[1];
+            pollFd[0].fd = fd;
+            pollFd[0].events = POLLOUT;
+            int ret = ::poll(pollFd, 1, timeout);
 	    if(ret == 0)
 	    {
 		closeSocketNoThrow(fd);
@@ -582,7 +563,7 @@ repeatConnect:
 	    {
 		if(interrupted())
 		{
-		    goto repeatSelect;
+		    goto repeatPoll;
 		}
 		
 		SocketException ex(__FILE__, __LINE__);
@@ -590,12 +571,12 @@ repeatConnect:
 		throw ex;
 	    }
 	    
-	    //
-	    // Strange windows bug: The following call to Sleep() is
-	    // necessary, otherwise no error is reported through
-	    // getsockopt.
-	    //
-	    //Sleep(0);
+            //
+            // Strange windows bug: The following call to Sleep() is
+            // necessary, otherwise no error is reported through
+            // getsockopt.
+            //
+            //Sleep(0);
 	    socklen_t len = static_cast<socklen_t>(sizeof(int));
 	    if(getsockopt(fd, SOL_SOCKET, SO_ERROR, reinterpret_cast<char*>(&val), &len) == SOCKET_ERROR)
 	    {
@@ -729,94 +710,6 @@ IceInternal::getAddress(const string& host, int port, struct sockaddr_in& addr)
     }
 }
 
-string
-IceInternal::getLocalHost(bool numeric)
-{
-    char host[1024 + 1];
-    if(gethostname(host, 1024) == SOCKET_ERROR)
-    {
-	SyscallException ex(__FILE__, __LINE__);
-	ex.error = getSystemErrno();
-	throw ex;
-    }
-    
-#ifdef _WIN32
-
-    //
-    // Windows XP has getaddrinfo(), but we don't want to require XP to run IceE.
-    //
-	
-    //
-    // gethostbyname() is thread safe on Windows, with a separate hostent per thread
-    //
-    struct hostent* entry;
-    int retry = 5;
-    do
-    {
-	entry = gethostbyname(host);
-    }
-    while(entry == 0 && WSAGetLastError() == WSATRY_AGAIN && --retry >= 0);
-    
-    if(entry == 0)
-    {
-	DNSException ex(__FILE__, __LINE__);
-	ex.error = WSAGetLastError();
-	ex.host = host;
-	throw ex;
-    }
-
-    if(numeric)
-    {
-	struct sockaddr_in addr;
-	memset(&addr, 0, sizeof(struct sockaddr_in));
-	memcpy(&addr.sin_addr, entry->h_addr, entry->h_length);
-	return inetAddrToString(addr.sin_addr);
-    }
-    else
-    {
-	return string(entry->h_name);
-    }
-
-#else
-    
-    struct addrinfo* info = 0;
-    int retry = 5;
-    
-    struct addrinfo hints = { 0 };
-    hints.ai_family = PF_INET;
-    
-    int rs = 0;
-    do
-    {
-	rs = getaddrinfo(host, 0, &hints, &info);    
-    }
-    while(info == 0 && rs == EAI_AGAIN && --retry >= 0);
-    
-    if(rs != 0)
-    {
-	DNSException ex(__FILE__, __LINE__);
-	ex.error = rs;
-	ex.host = host;
-	throw ex;
-    }
-    
-    string result;
-    if(numeric)
-    {
-	assert(info->ai_family == PF_INET);
-	struct sockaddr_in* sin = reinterpret_cast<sockaddr_in*>(info->ai_addr);
-	result = inetAddrToString(sin->sin_addr);
-    }
-    else
-    {
-	result = info->ai_canonname;
-    }
-    freeaddrinfo(info);
-    return result;
-
-#endif
-}
-
 bool
 IceInternal::compareAddress(const struct sockaddr_in& addr1, const struct sockaddr_in& addr2)
 {
@@ -1034,7 +927,7 @@ IceInternal::getLocalHosts()
     {
         result.push_back(inetAddrToString(addrs[i].sin_addr));
     }
-#elif defined(__linux) || defined(__APPLE__) || defined(__FreeBSD__)
+#elif defined(__APPLE__) || defined(__FreeBSD__)
     struct ifaddrs* ifap;
     if(::getifaddrs(&ifap) == SOCKET_ERROR)
     {
@@ -1060,7 +953,7 @@ IceInternal::getLocalHosts()
 
     ::freeifaddrs(ifap);
 #else
-    SOCKET fd = createSocket(false);
+    SOCKET fd = createSocket();
 
 #ifdef _AIX
     int cmd = CSIOCGIFCONF;
@@ -1082,10 +975,10 @@ IceInternal::getLocalHosts()
         int bufsize = numaddrs * sizeof(struct ifreq);
         ifc.ifc_len = bufsize;
         ifc.ifc_buf = (char*)malloc(bufsize);
-
         int rs = ioctl(fd, cmd, &ifc);
         if(rs == SOCKET_ERROR)
         {
+            free(ifc.ifc_buf);
             closeSocketNoThrow(fd);
             SocketException ex(__FILE__, __LINE__);
             ex.error = getSocketErrno();
