@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -8,6 +8,7 @@
 // **********************************************************************
 
 #include <IceE/ObjectAdapter.h>
+#include <IceE/ObjectAdapterFactory.h>
 #include <IceE/UUID.h>
 #include <IceE/Instance.h>
 #include <IceE/ProxyFactory.h>
@@ -29,41 +30,14 @@
 #endif
 #include <IceE/Endpoint.h>
 #include <IceE/LoggerUtil.h>
+#include <IceE/TraceLevels.h>
 #include <ctype.h>
 
 using namespace std;
 using namespace Ice;
 using namespace IceInternal;
 
-bool
-Ice::operator==(const ::Ice::ObjectAdapter& l, const ::Ice::ObjectAdapter& r)
-{
-    return &l == &r;
-}
-
-bool
-Ice::operator!=(const ::Ice::ObjectAdapter& l, const ::Ice::ObjectAdapter& r)
-{
-    return &l != &r;
-}
-
-bool
-Ice::operator<(const ::Ice::ObjectAdapter& l, const ::Ice::ObjectAdapter& r)
-{
-    return &l < &r;
-}
-
-void
-IceInternal::incRef(::Ice::ObjectAdapter* p)
-{
-    p->__incRef();
-}
-
-void
-IceInternal::decRef(::Ice::ObjectAdapter* p)
-{
-    p->__decRef();
-}
+IceUtil::Shared* IceInternal::upCast(::Ice::ObjectAdapter* p) { return p; }
 
 string
 Ice::ObjectAdapter::getName() const
@@ -97,78 +71,53 @@ Ice::ObjectAdapter::activate()
 	
 	checkForDeactivation();
 
-	if(!_printAdapterReadyDone)
+	//
+	// If the one off initializations of the adapter are already
+	// done, we just need to activate the incoming connection
+	// factories and we're done.
+	//
+	if(_activateOneOffDone)
 	{
-#ifdef ICEE_HAS_LOCATOR
-	    locatorInfo = _locatorInfo;
-#endif
-	    printAdapterReady = _instance->properties()->getPropertyAsInt("Ice.PrintAdapterReady") > 0;
-	    _printAdapterReadyDone = true;
+	    for_each(_incomingConnectionFactories.begin(), _incomingConnectionFactories.end(),
+		     Ice::voidMemFun(&IncomingConnectionFactory::activate));	
+	    return;
 	}
-	
-	for_each(_incomingConnectionFactories.begin(), _incomingConnectionFactories.end(),
-		 Ice::voidMemFun(&IncomingConnectionFactory::activate));	
+
+	//
+	// One off initializations of the adapter: update the locator
+	// registry and print the "adapter ready" message. We set the
+	// _waitForActivate flag to prevent deactivation from other
+	// threads while these one off initializations are done.
+	//
+	_waitForActivate = true;
+
+#ifdef ICEE_HAS_LOCATOR
+	locatorInfo = _locatorInfo;
+#endif
+	printAdapterReady = _instance->initializationData().properties->getPropertyAsInt("Ice.PrintAdapterReady") > 0;
     }
 
 #ifdef ICEE_HAS_LOCATOR
-    if(!_id.empty())
+    try
+    {
+	Ice::Identity dummy;
+	dummy.name = "dummy";
+	updateLocatorRegistry(locatorInfo, createDirectProxy(dummy));
+    }
+    catch(const Ice::LocalException&)
     {
 	//
-	// We must call on the locator registry outside the thread
-	// synchronization, to avoid deadlocks.
+	// If we couldn't update the locator registry, we let the
+	// exception go through and don't activate the adapter to
+	// allow to user code to retry activating the adapter
+	// later.
 	//
-	LocatorRegistryPrx locatorRegistry;
-	if(locatorInfo)
 	{
-	    //
-	    // TODO: This might throw if we can't connect to the
-	    // locator. Shall we raise a special exception for the
-	    // activate operation instead of a non obvious network
-	    // exception?
-	    //
-	    locatorRegistry = locatorInfo->getLocatorRegistry();
-	}
-
-	if(locatorRegistry)
-	{
-	    try
-	    {
-		Identity ident;
-		ident.name = "dummy";
-		if(_replicaGroupId.empty())
-		{
-		    locatorRegistry->setAdapterDirectProxy(_id, createDirectProxy(ident));
-		}
-		else
-		{
-		    locatorRegistry->setReplicatedAdapterDirectProxy(_id, _replicaGroupId, createDirectProxy(ident));
-		}
-	    }
-	    catch(const ObjectAdapterDeactivatedException&)
-	    {
-		// IGNORE: The object adapter is already inactive.
-	    }
-	    catch(const AdapterNotFoundException&)
-	    {
-		NotRegisteredException ex(__FILE__, __LINE__);
-		ex.kindOfObject = "object adapter";
-		ex.id = _id;
-		throw ex;
-	    }
-	    catch(const InvalidReplicaGroupIdException&)
-	    {
-		NotRegisteredException ex(__FILE__, __LINE__);
-		ex.kindOfObject = "replica group";
-		ex.id = _replicaGroupId;
-		throw ex;
-	    }
-	    catch(const AdapterAlreadyActiveException&)
-	    {
-		ObjectAdapterIdInUseException ex(__FILE__, __LINE__);
-		ex.id = _id;
-		throw ex;
-	    }
+	    IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
+	    _waitForActivate = false;
+	    notifyAll();
 	}
+	throw;
     }
 #endif
 
@@ -177,6 +126,22 @@ Ice::ObjectAdapter::activate()
 	printf("%s ready\n", _name.c_str());
 	fflush(stdout);
     }
+
+    {
+	IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
+	assert(!_deactivated); // Not possible if _waitForActivate = true;
+
+	//
+	// Signal threads waiting for the activation.
+	//
+	_waitForActivate = false;
+	notifyAll();
+
+	_activateOneOffDone = true;
+
+	for_each(_incomingConnectionFactories.begin(), _incomingConnectionFactories.end(),
+		 Ice::voidMemFun(&IncomingConnectionFactory::activate));
+    }
 }
 
 void
@@ -206,6 +171,9 @@ Ice::ObjectAdapter::deactivate()
 {
     vector<IncomingConnectionFactoryPtr> incomingConnectionFactories;
     OutgoingConnectionFactoryPtr outgoingConnectionFactory;
+#ifdef ICEE_HAS_LOCATOR
+    LocatorInfoPtr locatorInfo;
+#endif
 
     {
 	IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
@@ -219,14 +187,55 @@ Ice::ObjectAdapter::deactivate()
 	    return;
 	}
 
+	//
+	// Wait for activation to complete. This is necessary to not 
+	// get out of order locator updates.
+	//
+	while(_waitForActivate)
+	{
+	    wait();
+	}
+
+#ifdef ICEE_HAS_ROUTER
+        if(_routerInfo)
+        {
+            //
+            // Remove entry from the router manager.
+            //
+            _instance->routerManager()->erase(_routerInfo->getRouter());
+
+            //
+            //  Clear this object adapter with the router.
+            //
+            _routerInfo->setAdapter(0);
+        }
+#endif
+
         incomingConnectionFactories = _incomingConnectionFactories;
 	outgoingConnectionFactory = _instance->outgoingConnectionFactory();
-    
+#ifdef ICEE_HAS_LOCATOR
+	locatorInfo = _locatorInfo;
+#endif
+
 	_deactivated = true;
 	
 	notifyAll();
     }
 
+#ifdef ICEE_HAS_LOCATOR
+    try
+    {
+	updateLocatorRegistry(locatorInfo, 0);
+    }
+    catch(const Ice::LocalException&)
+    {
+	//
+	// We can't throw exceptions in deactivate so we ignore
+	// failures to update the locator registry.
+	//
+    }
+#endif
+
     //
     // Must be called outside the thread synchronization, because
     // Connection::destroy() might block when sending a CloseConnection
@@ -246,66 +255,121 @@ Ice::ObjectAdapter::deactivate()
 void
 Ice::ObjectAdapter::waitForDeactivate()
 {
+    vector<IceInternal::IncomingConnectionFactoryPtr> incomingConnectionFactories;
+
     {
-	IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
+        IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
 
-	//
-	// First we wait for deactivation of the adapter itself, and for
-	// the return of all direct method calls using this adapter.
-	//
-	while(!_deactivated || _directCount > 0)
-	{
-	    wait();
-	}
+        if(_destroyed)
+        {
+            return;
+        }
 
-	//
-	// If some other thread is currently deactivating, we wait
-	// until this thread is finished.
-	//
-	while(_waitForDeactivate)
-	{
-	    wait();
-	}
-	_waitForDeactivate = true;
+        //
+        // Wait for deactivation of the adapter itself, and for
+        // the return of all direct method calls using this adapter.
+        //
+        while(!_deactivated || _directCount > 0)
+        {
+            wait();
+        }
+
+        incomingConnectionFactories = _incomingConnectionFactories;
     }
 
     //
     // Now we wait until all incoming connection factories are
     // finished.
     //
-    for_each(_incomingConnectionFactories.begin(), _incomingConnectionFactories.end(),
-	     Ice::voidMemFun(&IncomingConnectionFactory::waitUntilFinished));
+    for_each(incomingConnectionFactories.begin(), incomingConnectionFactories.end(),
+             Ice::voidMemFun(&IncomingConnectionFactory::waitUntilFinished));
+}
+
+bool
+Ice::ObjectAdapter::isDeactivated() const
+{
+    IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
+
+    return _deactivated;
+}
+
+void
+Ice::ObjectAdapter::destroy()
+{
+    {
+        IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
+
+        //
+        // Another thread is in the process of destroying the object
+        // adapter. Wait for it to finish.
+        //
+        while(_destroying)
+        {
+            wait();
+        }
+
+        //
+        // Object adapter is already destroyed.
+        //
+        if(_destroyed)
+        {
+            return;
+        }
+
+        _destroying = true;
+    }
+
+    //
+    // Deactivate and wait for completion.
+    //
+    deactivate();
+    waitForDeactivate();
 
     //
     // Now it's also time to clean up our servants and servant
     // locators.
     //
-    if(_servantManager)
+    _servantManager->destroy();
+
+    ObjectAdapterFactoryPtr objectAdapterFactory;
+
     {
-	_servantManager->destroy();
+        IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
+
+        //
+        // Signal that destroy is complete.
+        //
+        _destroying = false;
+        _destroyed = true;
+        notifyAll();
+
+        //
+        // We're done, now we can throw away all incoming connection
+        // factories.
+        //
+        _incomingConnectionFactories.clear();
+
+        //
+        // Remove object references (some of them cyclic).
+        //
+        _instance = 0;
+        _communicator = 0;
+#ifdef ICEE_HAS_ROUTER
+        _routerEndpoints.clear();
+        _routerInfo = 0;
+#endif
+        _publishedEndpoints.clear();
+#ifdef ICEE_HAS_LOCATOR
+        _locatorInfo = 0;
+#endif
+
+        objectAdapterFactory = _objectAdapterFactory;
+        _objectAdapterFactory = 0;
     }
 
+    if(objectAdapterFactory)
     {
-	IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
-
-	//
-	// Signal that waiting is complete.
-	//
-	_waitForDeactivate = false;
-	notifyAll();
-
-	//
-	// We're done, now we can throw away all incoming connection
-	// factories.
-	//
-	_incomingConnectionFactories.clear();
-
-	//
-	// Remove object references (some of them cyclic).
-	//
-	_instance = 0;
-	_servantManager = 0;
-	_communicator = 0;
+        objectAdapterFactory->removeObjectAdapter(_name);
     }
 }
 
@@ -468,96 +532,11 @@ Ice::ObjectAdapter::createReverseProxy(const Identity& ident) const
     // reference.
     //
     vector<EndpointPtr> endpoints;
-    ReferencePtr ref = _instance->referenceFactory()->create(ident, _instance->getDefaultContext(), "",
-    							     Reference::ModeTwoway, connections);
+    ReferencePtr ref = _instance->referenceFactory()->create(ident, Ice::Context(), "", ReferenceModeTwoway, 
+							     connections);
     return _instance->proxyFactory()->referenceToProxy(ref);
 }
 
-#ifdef ICEE_HAS_ROUTER
-void
-Ice::ObjectAdapter::addRouter(const RouterPrx& router)
-{
-    IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
-    
-    checkForDeactivation();
-
-    RouterInfoPtr routerInfo = _instance->routerManager()->get(router);
-    if(routerInfo)
-    {
-	_routerInfos.push_back(routerInfo);
-
-	//
-	// Add the router's server proxy endpoints to this object
-	// adapter.
-	//
-	ObjectPrx proxy = routerInfo->getServerProxy();
-	vector<EndpointPtr> endpoints = proxy->__reference()->getEndpoints();
-	copy(endpoints.begin(), endpoints.end(), back_inserter(_routerEndpoints));
-	sort(_routerEndpoints.begin(), _routerEndpoints.end()); // Must be sorted.
-	_routerEndpoints.erase(unique(_routerEndpoints.begin(), _routerEndpoints.end()), _routerEndpoints.end());
-
-	//
-	// Associate this object adapter with the router. This way,
-	// new outgoing connections to the router's client proxy will
-	// use this object adapter for callbacks.
-	//
-	routerInfo->setAdapter(this);
-
-	//
-	// Also modify all existing outgoing connections to the
-	// router's client proxy to use this object adapter for
-	// callbacks.
-	//	
-	_instance->outgoingConnectionFactory()->setRouterInfo(routerInfo);
-    }
-}
-
-void
-Ice::ObjectAdapter::removeRouter(const RouterPrx& router)
-{
-    IceUtil::Monitor<IceUtil::RecMutex>::Lock sync(*this);
-    
-    checkForDeactivation();
-
-    RouterInfoPtr routerInfo = _instance->routerManager()->erase(router);
-    if(routerInfo)
-    {
-	//
-	// Rebuild the router endpoints from our set of router infos.
-	//
-	_routerEndpoints.clear();
-	vector<RouterInfoPtr>::iterator p = _routerInfos.begin();
-	while(p != _routerInfos.end())
-	{
-	    if(*p == routerInfo)
-	    {
-		p = _routerInfos.erase(p);
-		continue;
-	    }
-	    ObjectPrx proxy = (*p)->getServerProxy();
-	    vector<EndpointPtr> endpoints = proxy->__reference()->getEndpoints();
-	    copy(endpoints.begin(), endpoints.end(), back_inserter(_routerEndpoints));
-	    ++p;
-	}
-
-	sort(_routerEndpoints.begin(), _routerEndpoints.end()); // Must be sorted.
-	_routerEndpoints.erase(unique(_routerEndpoints.begin(), _routerEndpoints.end()), _routerEndpoints.end());
-
-	//
-	// Clear this object adapter with the router.
-	//
-	routerInfo->setAdapter(0);
-
-	//
-	// Also modify all existing outgoing connections to the
-	// router's client proxy to use this object adapter for
-	// callbacks.
-	//	
-	_instance->outgoingConnectionFactory()->setRouterInfo(routerInfo);
-    }
-}
-#endif
-
 #ifdef ICEE_HAS_LOCATOR
 void
 Ice::ObjectAdapter::setLocator(const LocatorPrx& locator)
@@ -611,74 +590,131 @@ Ice::ObjectAdapter::decDirectCount()
 ServantManagerPtr
 Ice::ObjectAdapter::getServantManager() const
 {
-    // No mutex lock necessary, _instance is
-    // immutable after creation until it is removed in
-    // waitForDeactivate().
-
-    // Not check for deactivation here!
-
-    assert(_instance); // Must not be called after waitForDeactivate().
-
+    //
+    // No mutex lock necessary, _servantManager is immutable.
+    //
     return _servantManager;
 }
 
 Ice::ObjectAdapter::ObjectAdapter(const InstancePtr& instance, const CommunicatorPtr& communicator,
-				  const string& name, const string& endpointInfo) :
+				  const ObjectAdapterFactoryPtr& objectAdapterFactory, 
+				  const string& name, const string& endpointInfo
+#ifdef ICEE_HAS_ROUTER
+				  , const RouterPrx& router
+#endif
+				  ) :
     _deactivated(false),
     _instance(instance),
     _communicator(communicator),
+    _objectAdapterFactory(objectAdapterFactory),
     _servantManager(new ServantManager(instance, name)),
-    _printAdapterReadyDone(false),
+    _activateOneOffDone(false),
     _name(name),
 #ifdef ICEE_HAS_LOCATOR
-    _id(instance->properties()->getProperty(name + ".AdapterId")),
-    _replicaGroupId(instance->properties()->getProperty(name + ".ReplicaGroupId")),
+    _id(instance->initializationData().properties->getProperty(name + ".AdapterId")),
+    _replicaGroupId(instance->initializationData().properties->getProperty(name + ".ReplicaGroupId")),
 #endif
     _directCount(0),
-    _waitForDeactivate(false)
+    _waitForActivate(false),
+    _destroying(false),
+    _destroyed(false)
 {
     __setNoDelete(true);
     try
     {
-	//
-	// Parse the endpoints, but don't store them in the adapter.
-	// The connection factory might change it, for example, to
-	// fill in the real port number.
-	//
-	vector<EndpointPtr> endpoints = parseEndpoints(endpointInfo);
-	for(vector<EndpointPtr>::iterator p = endpoints.begin(); p != endpoints.end(); ++p)
-	{
-	    _incomingConnectionFactories.push_back(new IncomingConnectionFactory(_instance, *p, this));
-	}
-
-	//
-	// Parse published endpoints. These are used in proxies
-	// instead of the connection factory endpoints.
-	//
-	string endpts = _instance->properties()->getProperty(name + ".PublishedEndpoints");
-	_publishedEndpoints = parseEndpoints(endpts);
-        if(_publishedEndpoints.empty())
-        {
-            transform(_incomingConnectionFactories.begin(), _incomingConnectionFactories.end(),
-                      back_inserter(_publishedEndpoints), Ice::constMemFun(&IncomingConnectionFactory::endpoint));
-        }
-
-        //
-        // Filter out any endpoints that are not meant to be published.
-        //
-        _publishedEndpoints.erase(remove_if(_publishedEndpoints.begin(), _publishedEndpoints.end(),
-                                  not1(Ice::constMemFun(&Endpoint::publish))), _publishedEndpoints.end());
-
 #ifdef ICEE_HAS_ROUTER
-	string router = _instance->properties()->getProperty(_name + ".Router");
-	if(!router.empty())
-	{
-	    addRouter(RouterPrx::uncheckedCast(_instance->proxyFactory()->stringToProxy(router)));
+        if(!router)
+        {
+            string routerStr = _instance->initializationData().properties->getProperty(_name + ".Router");
+            if(!routerStr.empty())
+            {
+                const_cast<RouterPrx&>(router) =
+                    RouterPrx::uncheckedCast(_instance->proxyFactory()->stringToProxy(routerStr));
+            }
+        }
+        if(router)
+        {
+            _routerInfo = _instance->routerManager()->get(router);
+            if(_routerInfo)
+            {
+                //
+                // Make sure this router is not already registered with another adapter.
+                //
+                if(_routerInfo->getAdapter())
+                {
+                    throw AlreadyRegisteredException(__FILE__, __LINE__, "object adapter with router",
+                                                     _instance->identityToString(router->ice_getIdentity()));
+                }
+
+                //
+                // Add the router's server proxy endpoints to this object
+                // adapter.
+                //
+                vector<EndpointPtr> endpoints = _routerInfo->getServerEndpoints();
+                copy(endpoints.begin(), endpoints.end(), back_inserter(_routerEndpoints));
+                sort(_routerEndpoints.begin(), _routerEndpoints.end()); // Must be sorted.
+                _routerEndpoints.erase(unique(_routerEndpoints.begin(), _routerEndpoints.end()),
+                                       _routerEndpoints.end());
+
+                //
+                // Associate this object adapter with the router. This way,
+                // new outgoing connections to the router's client proxy will
+                // use this object adapter for callbacks.
+                //
+                _routerInfo->setAdapter(this);
+
+                //
+                // Also modify all existing outgoing connections to the
+                // router's client proxy to use this object adapter for
+                // callbacks.
+                //      
+                _instance->outgoingConnectionFactory()->setRouterInfo(_routerInfo);
+            }
 	}
+	else
 #endif
-	
+	{
+	    //
+	    // Parse the endpoints, but don't store them in the adapter.
+	    // The connection factory might change it, for example, to
+	    // fill in the real port number.
+	    //
+	    vector<EndpointPtr> endpoints = parseEndpoints(endpointInfo);
+	    for(vector<EndpointPtr>::iterator p = endpoints.begin(); p != endpoints.end(); ++p)
+	    {
+	        _incomingConnectionFactories.push_back(new IncomingConnectionFactory(_instance, *p, this));
+	    }
+	    if(endpoints.empty())
+	    {
+		TraceLevelsPtr tl = _instance->traceLevels();
+		if(tl->network >= 2)
+		{
+		    Trace out(_instance->initializationData().logger, tl->networkCat);
+		    out << "created adapter `" << name << "' without endpoints";
+		}
+	    }
+
+	    //
+	    // Parse published endpoints. These are used in proxies
+	    // instead of the connection factory endpoints.
+	    //
+	    string endpts = _instance->initializationData().properties->getProperty(name + ".PublishedEndpoints");
+	    _publishedEndpoints = parseEndpoints(endpts);
+            if(_publishedEndpoints.empty())
+            {
+                transform(_incomingConnectionFactories.begin(), _incomingConnectionFactories.end(),
+                          back_inserter(_publishedEndpoints), Ice::constMemFun(&IncomingConnectionFactory::endpoint));
+            }
+
+            //
+            // Filter out any endpoints that are not meant to be published.
+            //
+            _publishedEndpoints.erase(remove_if(_publishedEndpoints.begin(), _publishedEndpoints.end(),
+                                      not1(Ice::constMemFun(&Endpoint::publish))), _publishedEndpoints.end());
+	}
+
 #ifdef ICEE_HAS_LOCATOR
-	string locator = _instance->properties()->getProperty(_name + ".Locator");
+	string locator = _instance->initializationData().properties->getProperty(_name + ".Locator");
 	if(!locator.empty())
 	{
 	    setLocator(LocatorPrx::uncheckedCast(_instance->proxyFactory()->stringToProxy(locator)));
@@ -703,21 +739,21 @@ Ice::ObjectAdapter::~ObjectAdapter()
 {
     if(!_deactivated)
     {
-	Warning out(_instance->logger());
+	Warning out(_instance->initializationData().logger);
 	out << "object adapter `" << _name << "' has not been deactivated";
     }
-    else if(_instance)
+    else if(!_destroyed)
     {
-	Warning out(_instance->logger());
-	out << "object adapter `" << _name << "' deactivation had not been waited for";
+	Warning out(_instance->initializationData().logger);
+	out << "object adapter `" << _name << "' has not been destroyed";
     }
     else
     {
-	assert(!_servantManager);
+	//assert(!_servantManager); // We don't clear this reference, it needs to be immutable.
 	assert(!_communicator);
 	assert(_incomingConnectionFactories.empty());
 	assert(_directCount == 0);
-	assert(!_waitForDeactivate);
+	assert(!_waitForActivate);
     }
 }
 
@@ -760,11 +796,11 @@ Ice::ObjectAdapter::newDirectProxy(const Identity& ident, const string& facet) c
     // Create a reference and return a proxy for this reference.
     //
 #ifdef ICEE_HAS_ROUTER
-    ReferencePtr ref = _instance->referenceFactory()->create(ident, _instance->getDefaultContext(), facet,
-    							     Reference::ModeTwoway, false, endpoints, 0);
+    ReferencePtr ref = _instance->referenceFactory()->create(ident, Ice::Context(), facet, ReferenceModeTwoway,
+							     false, endpoints, 0);
 #else
-    ReferencePtr ref = _instance->referenceFactory()->create(ident, _instance->getDefaultContext(), facet,
-    							     Reference::ModeTwoway, false, endpoints);
+    ReferencePtr ref = _instance->referenceFactory()->create(ident, Ice::Context(), facet, ReferenceModeTwoway,
+							     false, endpoints);
 #endif
     return _instance->proxyFactory()->referenceToProxy(ref);
 
@@ -778,12 +814,13 @@ Ice::ObjectAdapter::newIndirectProxy(const Identity& ident, const string& facet,
     // Create a reference with the adapter id.
     //
 #ifdef ICEE_HAS_ROUTER
-    ReferencePtr ref = _instance->referenceFactory()->create(ident, _instance->getDefaultContext(), facet,
-    							     Reference::ModeTwoway, false, id, 0, _locatorInfo);
-    
+    ReferencePtr ref =
+        _instance->referenceFactory()->create(ident, Ice::Context(), facet, ReferenceModeTwoway, false, id, 0,
+					      _locatorInfo);
 #else
-    ReferencePtr ref = _instance->referenceFactory()->create(ident, _instance->getDefaultContext(), facet,
-    							     Reference::ModeTwoway, false, id, _locatorInfo);
+    ReferencePtr ref =
+        _instance->referenceFactory()->create(ident, Ice::Context(), facet, ReferenceModeTwoway, false, id, 
+					      _locatorInfo);
 #endif
 
     //
@@ -863,3 +900,66 @@ Ice::ObjectAdapter::parseEndpoints(const string& str) const
 
     return endpoints;
 }
+
+#ifdef ICEE_HAS_LOCATOR
+void
+ObjectAdapter::updateLocatorRegistry(const IceInternal::LocatorInfoPtr& locatorInfo, const Ice::ObjectPrx& proxy)
+{
+    if(_id.empty())
+    {
+	return; // Nothing to update.
+    }
+
+    //
+    // We must get and call on the locator registry outside the thread
+    // synchronization to avoid deadlocks. (we can't make remote calls
+    // within the OA synchronization because the remote call will
+    // indirectly call isLocal() on this OA with the OA factory
+    // locked).
+    //
+    // TODO: This might throw if we can't connect to the
+    // locator. Shall we raise a special exception for the activate
+    // operation instead of a non obvious network exception?
+    //
+    LocatorRegistryPrx locatorRegistry = locatorInfo ? locatorInfo->getLocatorRegistry() : LocatorRegistryPrx();
+    if(!locatorRegistry)
+    {
+	return;
+    }
+
+    if(!_id.empty())
+    {
+	try
+	{
+	    if(_replicaGroupId.empty())
+	    {
+		locatorRegistry->setAdapterDirectProxy(_id, proxy);
+	    }
+	    else
+	    {
+		locatorRegistry->setReplicatedAdapterDirectProxy(_id, _replicaGroupId, proxy);
+	    }
+	}
+	catch(const AdapterNotFoundException&)
+	{
+	    NotRegisteredException ex(__FILE__, __LINE__);
+	    ex.kindOfObject = "object adapter";
+	    ex.id = _id;
+	    throw ex;
+	}
+	catch(const InvalidReplicaGroupIdException&)
+	{
+	    NotRegisteredException ex(__FILE__, __LINE__);
+	    ex.kindOfObject = "replica group";
+	    ex.id = _replicaGroupId;
+	    throw ex;
+	}
+	catch(const AdapterAlreadyActiveException&)
+	{
+	    ObjectAdapterIdInUseException ex(__FILE__, __LINE__);
+	    ex.id = _id;
+	    throw ex;
+	}
+    }
+}
+#endif
