@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,6 +11,9 @@
 #define ICEE_OBJECT_ADAPTER_FACTORY_H
 
 #include <IceE/ObjectAdapterF.h>
+#ifdef ICEE_HAS_ROUTER
+# include <IceE/RouterF.h>
+#endif
 #include <IceE/ProxyF.h>
 #include <IceE/CommunicatorF.h>
 #include <IceE/InstanceF.h>
@@ -28,8 +31,15 @@ public:
 
     void shutdown();
     void waitForShutdown();
+    bool isShutdown() const;
+    void destroy();
 
-    ::Ice::ObjectAdapterPtr createObjectAdapter(const std::string&, const std::string&);
+    ::Ice::ObjectAdapterPtr createObjectAdapter(const std::string&, const std::string&
+#ifdef ICEE_HAS_ROUTER
+						, const ::Ice::RouterPrx&
+#endif
+    					        );
+    void removeObjectAdapter(const std::string&);
     void flushBatchRequests() const;
 
 private:
