@@ -1,12 +1,13 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
+#include <IceE/Communicator.h>
 #include <IceE/Proxy.h>
 #include <IceE/ProxyFactory.h>
 #include <IceE/Outgoing.h>
@@ -17,15 +18,17 @@
 #include <IceE/LocalException.h>
 #ifdef ICEE_HAS_ROUTER
 #   include <IceE/RouterInfo.h>
+#   include <IceE/Router.h>
+#endif
+#ifdef ICEE_HAS_LOCATOR
+#   include <IceE/LocatorInfo.h>
+#   include <IceE/Locator.h>
 #endif
 
 using namespace std;
 using namespace Ice;
 using namespace IceInternal;
 
-void IceInternal::incRef(::IceProxy::Ice::Object* p) { p->__incRef(); }
-void IceInternal::decRef(::IceProxy::Ice::Object* p) { p->__decRef(); }
-
 void
 Ice::__write(::IceInternal::BasicStream* __os, const ::Ice::Context& v, ::Ice::__U__Context)
 {
@@ -65,7 +68,7 @@ IceInternal::checkedCastImpl(const ObjectPrx& b, const string& f, const string&
 
     if(b)
     {
-	ObjectPrx bb = b->ice_newFacet(f);
+	ObjectPrx bb = b->ice_facet(f);
 	try
 	{
 	    if(bb->ice_isA(typeId))
@@ -99,7 +102,7 @@ IceInternal::checkedCastImpl(const ObjectPrx& b, const string& f, const string&
 
     if(b)
     {
-	ObjectPrx bb = b->ice_newFacet(f);
+	ObjectPrx bb = b->ice_facet(f);
 	try
 	{
 	    if(bb->ice_isA(typeId, ctx))
@@ -126,12 +129,6 @@ IceProxy::Ice::Object::operator==(const Object& r) const
     return _reference == r._reference;
 }
 
-bool
-IceProxy::Ice::Object::operator!=(const Object& r) const
-{
-    return _reference != r._reference;
-}
-
 bool
 IceProxy::Ice::Object::operator<(const Object& r) const
 {
@@ -139,13 +136,13 @@ IceProxy::Ice::Object::operator<(const Object& r) const
 }
 
 Int
-IceProxy::Ice::Object::ice_hash() const
+IceProxy::Ice::Object::ice_getHash() const
 {
     return _reference->hash();
 }
 
 CommunicatorPtr
-IceProxy::Ice::Object::ice_communicator() const
+IceProxy::Ice::Object::ice_getCommunicator() const
 {
     return _reference->getCommunicator();
 }
@@ -157,27 +154,22 @@ IceProxy::Ice::Object::ice_toString() const
 }
 
 bool
-IceProxy::Ice::Object::ice_isA(const string& __id)
-{
-    return ice_isA(__id, _reference->getContext());
-}
-
-bool
-IceProxy::Ice::Object::ice_isA(const string& __id, const Context& __context)
+IceProxy::Ice::Object::ice_isA(const string& __id, const Context* __context)
 {
     int __cnt = 0;
     while(true)
     {
+        ConnectionPtr __connection;
 	try
 	{
 	    __checkTwowayOnly("ice_isA");
 	    static const string __operation("ice_isA");
-	    ConnectionPtr __connection = ice_connection();
+	    __connection = ice_getConnection();
             Outgoing __og(__connection.get(), _reference.get(), __operation, ::Ice::Nonmutating, __context);
             BasicStream* __stream = __og.stream();
             try
             {
-                __stream->write(__id);
+                __stream->write(__id, false);
             }
             catch(const ::Ice::LocalException& __ex)
             {
@@ -214,11 +206,11 @@ IceProxy::Ice::Object::ice_isA(const string& __id, const Context& __context)
 	}
 	catch(const LocalExceptionWrapper& __ex)
 	{
-	    __handleExceptionWrapperRelaxed(__ex, __cnt);
+	    __handleExceptionWrapperRelaxed(__connection, __ex, __cnt);
 	}
 	catch(const LocalException& __ex)
 	{
-	    __handleException(__ex, __cnt);
+	    __handleException(__connection, __ex, __cnt);
 	}
 #if defined(_MSC_VER) && defined(_M_ARM) // ARM bug.
 	catch(...)
@@ -230,21 +222,16 @@ IceProxy::Ice::Object::ice_isA(const string& __id, const Context& __context)
 }
 
 void
-IceProxy::Ice::Object::ice_ping()
-{
-    ice_ping(_reference->getContext());
-}
-
-void
-IceProxy::Ice::Object::ice_ping(const Context& __context)
+IceProxy::Ice::Object::ice_ping(const Context* __context)
 {
     int __cnt = 0;
     while(true)
     {
+        ConnectionPtr __connection;
 	try
 	{
             static const string __operation("ice_ping");
-	    ConnectionPtr __connection = ice_connection();
+	    __connection = ice_getConnection();
             Outgoing __og(__connection.get(), _reference.get(), __operation, ::Ice::Nonmutating, __context);
             bool __ok = __og.invoke();
             try
@@ -276,11 +263,11 @@ IceProxy::Ice::Object::ice_ping(const Context& __context)
 	}
 	catch(const LocalExceptionWrapper& __ex)
 	{
-	    __handleExceptionWrapperRelaxed(__ex, __cnt);
+	    __handleExceptionWrapperRelaxed(__connection, __ex, __cnt);
 	}
 	catch(const LocalException& __ex)
 	{
-	    __handleException(__ex, __cnt);
+	    __handleException(__connection, __ex, __cnt);
 	}
 #if defined(_MSC_VER) && defined(_M_ARM) // ARM bug.
 	catch(...)
@@ -292,22 +279,17 @@ IceProxy::Ice::Object::ice_ping(const Context& __context)
 }
 
 vector<string>
-IceProxy::Ice::Object::ice_ids()
-{
-    return ice_ids(_reference->getContext());
-}
-
-vector<string>
-IceProxy::Ice::Object::ice_ids(const Context& __context)
+IceProxy::Ice::Object::ice_ids(const Context* __context)
 {
     int __cnt = 0;
     while(true)
     {
+        ConnectionPtr __connection;
 	try
 	{
 	    __checkTwowayOnly("ice_ids");
             static const string __operation("ice_ids");
-	    ConnectionPtr __connection = ice_connection();
+	    __connection = ice_getConnection();
             Outgoing __og(__connection.get(), _reference.get(), __operation, ::Ice::Nonmutating, __context);
             vector<string> __ret;
             bool __ok = __og.invoke();
@@ -325,7 +307,7 @@ IceProxy::Ice::Object::ice_ids(const Context& __context)
 		        throw ::Ice::UnknownUserException(__FILE__, __LINE__, __ex.ice_name());
 		    }
                 }
-                __is->read(__ret);
+                __is->read(__ret, false);
             }
             catch(const ::Ice::LocalException& __ex)
             {
@@ -341,11 +323,11 @@ IceProxy::Ice::Object::ice_ids(const Context& __context)
 	}
 	catch(const LocalExceptionWrapper& __ex)
 	{
-	    __handleExceptionWrapperRelaxed(__ex, __cnt);
+	    __handleExceptionWrapperRelaxed(__connection, __ex, __cnt);
 	}
 	catch(const LocalException& __ex)
 	{
-	    __handleException(__ex, __cnt);
+	    __handleException(__connection, __ex, __cnt);
 	}
 #if defined(_MSC_VER) && defined(_M_ARM) // ARM bug.
 	catch(...)
@@ -357,22 +339,17 @@ IceProxy::Ice::Object::ice_ids(const Context& __context)
 }
 
 string
-IceProxy::Ice::Object::ice_id()
-{
-    return ice_id(_reference->getContext());
-}
-
-string
-IceProxy::Ice::Object::ice_id(const Context& __context)
+IceProxy::Ice::Object::ice_id(const Context* __context)
 {
     int __cnt = 0;
     while(true)
     {
+        ConnectionPtr __connection;
 	try
 	{
 	    __checkTwowayOnly("ice_id");
             static const string __operation("ice_id");
-	    ConnectionPtr __connection = ice_connection();
+	    __connection = ice_getConnection();
             Outgoing __og(__connection.get(), _reference.get(), __operation, ::Ice::Nonmutating, __context);
             string __ret;
             bool __ok = __og.invoke();
@@ -390,7 +367,7 @@ IceProxy::Ice::Object::ice_id(const Context& __context)
 		        throw ::Ice::UnknownUserException(__FILE__, __LINE__, __ex.ice_name());
 		    }
                 }
-                __is->read(__ret);
+                __is->read(__ret, false);
             }
             catch(const ::Ice::LocalException& __ex)
             {
@@ -406,11 +383,11 @@ IceProxy::Ice::Object::ice_id(const Context& __context)
 	}
 	catch(const LocalExceptionWrapper& __ex)
 	{
-	    __handleExceptionWrapperRelaxed(__ex, __cnt);
+	    __handleExceptionWrapperRelaxed(__connection, __ex, __cnt);
 	}
 	catch(const LocalException& __ex)
 	{
-	    __handleException(__ex, __cnt);
+	    __handleException(__connection, __ex, __cnt);
 	}
 #if defined(_MSC_VER) && defined(_M_ARM) // ARM bug.
 	catch(...)
@@ -424,25 +401,17 @@ IceProxy::Ice::Object::ice_id(const Context& __context)
 Context
 IceProxy::Ice::Object::ice_getContext() const
 {
-    return _reference->getContext();
+    return *_reference->getContext();
 }
 
 ObjectPrx
-IceProxy::Ice::Object::ice_newContext(const Context& newContext) const
+IceProxy::Ice::Object::ice_context(const Context& newContext) const
 {
     ObjectPrx proxy(new ::IceProxy::Ice::Object());
     proxy->setup(_reference->changeContext(newContext));
     return proxy;
 }
 
-ObjectPrx
-IceProxy::Ice::Object::ice_defaultContext() const
-{
-    ObjectPrx proxy(new ::IceProxy::Ice::Object());
-    proxy->setup(_reference->defaultContext());
-    return proxy;
-}
-
 Identity
 IceProxy::Ice::Object::ice_getIdentity() const
 {
@@ -450,8 +419,12 @@ IceProxy::Ice::Object::ice_getIdentity() const
 }
 
 ObjectPrx
-IceProxy::Ice::Object::ice_newIdentity(const Identity& newIdentity) const
+IceProxy::Ice::Object::ice_identity(const Identity& newIdentity) const
 {
+    if(newIdentity.name.empty())
+    {
+        throw IllegalIdentityException(__FILE__, __LINE__);
+    }
     if(newIdentity == _reference->getIdentity())
     {
 	return ObjectPrx(const_cast< ::IceProxy::Ice::Object*>(this));
@@ -471,7 +444,7 @@ IceProxy::Ice::Object::ice_getFacet() const
 }
 
 ObjectPrx
-IceProxy::Ice::Object::ice_newFacet(const string& newFacet) const
+IceProxy::Ice::Object::ice_facet(const string& newFacet) const
 {
     if(newFacet == _reference->getFacet())
     {
@@ -485,92 +458,15 @@ IceProxy::Ice::Object::ice_newFacet(const string& newFacet) const
     }
 }
 
-ObjectPrx
-IceProxy::Ice::Object::ice_twoway() const
-{
-    ReferencePtr ref = _reference->changeMode(Reference::ModeTwoway);
-    if(ref == _reference)
-    {
-	return ObjectPrx(const_cast< ::IceProxy::Ice::Object*>(this));
-    }
-    else
-    {
-	ObjectPrx proxy(new ::IceProxy::Ice::Object());
-	proxy->setup(ref);
-	return proxy;
-    }
-}
-
-bool
-IceProxy::Ice::Object::ice_isTwoway() const
-{
-    return _reference->getMode() == Reference::ModeTwoway;
-}
-
-ObjectPrx
-IceProxy::Ice::Object::ice_oneway() const
-{
-    ReferencePtr ref = _reference->changeMode(Reference::ModeOneway);
-    if(ref == _reference)
-    {
-	return ObjectPrx(const_cast< ::IceProxy::Ice::Object*>(this));
-    }
-    else
-    {
-	ObjectPrx proxy(new ::IceProxy::Ice::Object());
-	proxy->setup(ref);
-	return proxy;
-    }
-}
-
-bool
-IceProxy::Ice::Object::ice_isOneway() const
-{
-    return _reference->getMode() == Reference::ModeOneway;
-}
-
-#ifdef ICEE_HAS_BATCH
-ObjectPrx
-IceProxy::Ice::Object::ice_batchOneway() const
-{
-    ReferencePtr ref = _reference->changeMode(Reference::ModeBatchOneway);
-    if(ref == _reference)
-    {
-	return ObjectPrx(const_cast< ::IceProxy::Ice::Object*>(this));
-    }
-    else
-    {
-	ObjectPrx proxy(new ::IceProxy::Ice::Object());
-	proxy->setup(ref);
-	return proxy;
-    }
-}
-
-bool
-IceProxy::Ice::Object::ice_isBatchOneway() const
-{
-    return _reference->getMode() == Reference::ModeBatchOneway;
-}
-#endif
-
-ObjectPrx
-IceProxy::Ice::Object::ice_timeout(int t) const
-{
-    ReferencePtr ref = _reference->changeTimeout(t);
-    if(ref == _reference)
-    {
-	return ObjectPrx(const_cast< ::IceProxy::Ice::Object*>(this));
-    }
-    else
-    {
-	ObjectPrx proxy(new ::IceProxy::Ice::Object());
-	proxy->setup(ref);
-	return proxy;
-    }
-}
-
 #ifdef ICEE_HAS_ROUTER
 
+RouterPrx
+IceProxy::Ice::Object::ice_getRouter() const
+{
+    RouterInfoPtr ri = _reference->getRouterInfo();
+    return ri ? ri->getRouter() : RouterPrx();    
+}
+
 ObjectPrx
 IceProxy::Ice::Object::ice_router(const RouterPrx& router) const
 {
@@ -591,6 +487,35 @@ IceProxy::Ice::Object::ice_router(const RouterPrx& router) const
 
 #ifdef ICEE_HAS_LOCATOR
 
+string
+IceProxy::Ice::Object::ice_getAdapterId() const
+{
+    return _reference->getAdapterId();
+}
+
+ObjectPrx
+IceProxy::Ice::Object::ice_adapterId(const string& adapterId) const
+{
+    ReferencePtr ref = _reference->changeAdapterId(adapterId);
+    if(ref == _reference)
+    {
+	return ObjectPrx(const_cast< ::IceProxy::Ice::Object*>(this));
+    }
+    else
+    {
+	ObjectPrx proxy(new ::IceProxy::Ice::Object());
+	proxy->setup(ref);
+	return proxy;
+    }
+}
+
+LocatorPrx
+IceProxy::Ice::Object::ice_getLocator() const
+{
+    LocatorInfoPtr ri = _reference->getLocatorInfo();
+    return ri ? ri->getLocator() : LocatorPrx();    
+}
+
 ObjectPrx
 IceProxy::Ice::Object::ice_locator(const LocatorPrx& locator) const
 {
@@ -609,8 +534,45 @@ IceProxy::Ice::Object::ice_locator(const LocatorPrx& locator) const
 
 #endif
 
+bool
+IceProxy::Ice::Object::ice_isSecure() const
+{
+    return _reference->getSecure();
+}
+
+ObjectPrx
+IceProxy::Ice::Object::ice_secure(bool b) const
+{
+    if(b == _reference->getSecure())
+    {
+        return ObjectPrx(const_cast< ::IceProxy::Ice::Object*>(this));
+    }
+    else
+    {
+        ObjectPrx proxy(new ::IceProxy::Ice::Object());
+        proxy->setup(_reference->changeSecure(b));
+        return proxy;
+    }
+}
+
+ObjectPrx
+IceProxy::Ice::Object::ice_timeout(int t) const
+{
+    ReferencePtr ref = _reference->changeTimeout(t);
+    if(ref == _reference)
+    {
+	return ObjectPrx(const_cast< ::IceProxy::Ice::Object*>(this));
+    }
+    else
+    {
+	ObjectPrx proxy(new ::IceProxy::Ice::Object());
+	proxy->setup(ref);
+	return proxy;
+    }
+}
+
 ConnectionPtr
-IceProxy::Ice::Object::ice_connection()
+IceProxy::Ice::Object::ice_getConnection()
 {
     ::IceUtil::Mutex::Lock sync(*this);
 
@@ -624,20 +586,21 @@ IceProxy::Ice::Object::ice_connection()
         // object.
         //
 #ifdef ICEE_HAS_ROUTER
-        RoutableReferencePtr rr = RoutableReferencePtr::dynamicCast(_reference);
-        if(rr && rr->getRouterInfo())
+        RouterInfoPtr ri = _reference->getRouterInfo();
+        if(ri)
         {
-            rr->getRouterInfo()->addProxy(this);
+            ri->addProxy(this);
         }
 #endif
     }
     return _connection;
 }
 
-ReferencePtr
-IceProxy::Ice::Object::__reference() const
+ConnectionPtr
+IceProxy::Ice::Object::ice_getCachedConnection() const
 {
-    return _reference;
+    ::IceUtil::Mutex::Lock sync(*this);
+    return _connection;
 }
 
 void
@@ -666,14 +629,17 @@ IceProxy::Ice::Object::__copyFrom(const ObjectPrx& from)
 }
 
 void
-IceProxy::Ice::Object::__handleException(const LocalException& ex, int& cnt)
+IceProxy::Ice::Object::__handleException(const ConnectionPtr& connection, const LocalException& ex, int& cnt)
 {
     //
     // Only _connection needs to be mutex protected here.
     //
     {
         ::IceUtil::Mutex::Lock sync(*this);
-        _connection = 0;
+        if(connection.get() == _connection.get())
+        {
+            _connection = 0;
+        }
     }
 
     ProxyFactoryPtr proxyFactory = _reference->getInstance()->proxyFactory();
@@ -691,11 +657,14 @@ IceProxy::Ice::Object::__handleException(const LocalException& ex, int& cnt)
 }
 
 void
-IceProxy::Ice::Object::__handleExceptionWrapper(const LocalExceptionWrapper& ex)
+IceProxy::Ice::Object::__handleExceptionWrapper(const ConnectionPtr& connection, const LocalExceptionWrapper& ex)
 {
     {
         IceUtil::Mutex::Lock sync(*this);
-        _connection = 0;
+        if(connection.get() == _connection.get())
+        {
+            _connection = 0;
+        }
     }
 
     if(!ex.retry())
@@ -705,16 +674,21 @@ IceProxy::Ice::Object::__handleExceptionWrapper(const LocalExceptionWrapper& ex)
 }
 
 void
-IceProxy::Ice::Object::__handleExceptionWrapperRelaxed(const LocalExceptionWrapper& ex, int& cnt)
+IceProxy::Ice::Object::__handleExceptionWrapperRelaxed(const ConnectionPtr& connection,
+                                                       const LocalExceptionWrapper& ex, 
+                                                       int& cnt)
 {
     if(!ex.retry())
     {
-        __handleException(*ex.get(), cnt);
+        __handleException(connection, *ex.get(), cnt);
     }
     else
     {
         IceUtil::Mutex::Lock sync(*this);
-        _connection = 0;
+        if(connection.get() == _connection.get())
+        {
+            _connection = 0;
+        }
     }
 }
 
@@ -734,24 +708,25 @@ IceProxy::Ice::Object::__checkTwowayOnly(const char* name) const
     }
 }
 
-const Context&
-IceProxy::Ice::Object::__defaultContext() const
+ReferenceMode
+IceProxy::Ice::Object::getMode() const
 {
-    return _reference->getContext();
+    return _reference->getMode();
 }
 
-void
-IceProxy::Ice::Object::setup(const ReferencePtr& ref)
+ObjectPrx
+IceProxy::Ice::Object::changeMode(ReferenceMode newMode) const
 {
-    //
-    // No need to synchronize "*this", as this operation is only
-    // called upon initialization.
-    //
-
-    assert(!_reference);
-    assert(!_connection);
-
-    _reference = ref;
+    if(_reference->getMode() == newMode)
+    {
+	return ObjectPrx(const_cast< ::IceProxy::Ice::Object*>(this));
+    }
+    else
+    {
+	ObjectPrx proxy(new ::IceProxy::Ice::Object());
+	proxy->setup(_reference->changeMode(newMode));
+	return proxy;
+    }
 }
 
 bool
