@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,7 +11,6 @@
 #include <IceE/ReferenceFactory.h>
 #include <IceE/LocalException.h>
 #include <IceE/Instance.h>
-#include <IceE/IdentityUtil.h>
 #include <IceE/Endpoint.h>
 #include <IceE/BasicStream.h>
 
@@ -36,16 +35,29 @@ using namespace std;
 using namespace Ice;
 using namespace IceInternal;
 
-void IceInternal::incRef(IceInternal::Reference* p) { p->__incRef(); }
-void IceInternal::decRef(IceInternal::Reference* p) { p->__decRef(); }
+IceUtil::Shared* IceInternal::upCast(IceInternal::Reference* p) { return p; }
 
-ReferencePtr
-IceInternal::Reference::defaultContext() const
+class ConnectionIsDatagram : public unary_function<ConnectionPtr, bool>
 {
-    ReferencePtr r = _instance->referenceFactory()->copy(this);
-    r->_context = _instance->getDefaultContext();
-    return r;
-}
+public:
+
+    bool
+    operator()(ConnectionPtr p) const
+    {
+        return p->endpoint()->datagram();
+    }
+};
+
+class ConnectionIsSecure : public unary_function<ConnectionPtr, bool>
+{
+public:
+
+    bool
+    operator()(ConnectionPtr p) const
+    {
+        return p->endpoint()->secure();
+    }
+};
 
 CommunicatorPtr
 IceInternal::Reference::getCommunicator() const
@@ -62,7 +74,7 @@ IceInternal::Reference::changeContext(const Context& newContext) const
 }
 
 ReferencePtr
-IceInternal::Reference::changeMode(Mode newMode) const
+IceInternal::Reference::changeMode(ReferenceMode newMode) const
 {
     if(newMode == _mode)
     {
@@ -73,6 +85,18 @@ IceInternal::Reference::changeMode(Mode newMode) const
     return r;
 }
 
+ReferencePtr
+IceInternal::Reference::changeSecure(bool newSecure) const
+{
+    if(newSecure == _secure)
+    {
+	return ReferencePtr(const_cast<Reference*>(this));
+    }
+    ReferencePtr r = _instance->referenceFactory()->copy(this);
+    r->_secure = newSecure;
+    return r;
+}
+
 ReferencePtr
 IceInternal::Reference::changeIdentity(const Identity& newIdentity) const
 {
@@ -97,6 +121,19 @@ IceInternal::Reference::changeFacet(const string& newFacet) const
     return r;
 }
 
+ReferencePtr
+IceInternal::Reference::changeTimeout(int newTimeout) const
+{
+    if(_overrideTimeout && newTimeout == _timeout)
+    {
+	return ReferencePtr(const_cast<Reference*>(this));
+    }
+    ReferencePtr r = getInstance()->referenceFactory()->copy(this);
+    r->_timeout = newTimeout;
+    r->_overrideTimeout = true;
+    return r;
+}
+
 Int
 Reference::hash() const
 {
@@ -184,8 +221,8 @@ IceInternal::Reference::toString() const
     // the reference parser uses as separators, then we enclose
     // the identity string in quotes.
     //
-    string id = identityToString(_identity);
-    if(id.find_first_of(" \t\n\r:@") != string::npos)
+    string id = _instance->identityToString(_identity);
+    if(id.find_first_of(" :@") != string::npos)
     {
         s += "\"";
 	s += id;
@@ -205,8 +242,18 @@ IceInternal::Reference::toString() const
 	// the reference parser uses as separators, then we enclose
 	// the facet string in quotes.
 	//
-	string fs = IceUtil::escapeString(_facet, "");
-	if(fs.find_first_of(" \t\n\r:@") != string::npos)
+        string fs = _facet;
+#ifdef ICEE_HAS_WSTRING
+        if(_instance->initializationData().stringConverter)
+        {
+            UTF8BufferI buffer;
+            Byte* last =
+                _instance->initializationData().stringConverter->toUTF8(fs.data(), fs.data() + fs.size(), buffer);
+            fs = string(reinterpret_cast<const char*>(buffer.getBuffer()), last - buffer.getBuffer());
+        }
+#endif
+	fs = IceUtil::escapeString(fs, "");
+	if(fs.find_first_of(" :@") != string::npos)
 	{
             s += "\"";
 	    s += fs;
@@ -220,31 +267,31 @@ IceInternal::Reference::toString() const
 
     switch(_mode)
     {
-	case ModeTwoway:
+	case ReferenceModeTwoway:
 	{
 	    s += " -t";
 	    break;
 	}
 
-	case ModeOneway:
+	case ReferenceModeOneway:
 	{
 	    s += " -o";
 	    break;
 	}
 
-	case ModeBatchOneway:
+	case ReferenceModeBatchOneway:
 	{
 	    s += " -O";
 	    break;
 	}
 
-	case ModeDatagram:
+	case ReferenceModeDatagram:
 	{
 	    s += " -d";
 	    break;
 	}
 
-	case ModeBatchDatagram:
+	case ReferenceModeBatchDatagram:
 	{
 	    s += " -D";
 	    break;
@@ -267,6 +314,11 @@ IceInternal::Reference::operator==(const Reference& r) const
     //
     // Note: if(this == &r) test is performed by each non-abstract derived class.
     //
+
+    if(getType() != r.getType())
+    {
+	return false;
+    }
     
     if(_mode != r._mode)
     {
@@ -293,13 +345,12 @@ IceInternal::Reference::operator==(const Reference& r) const
 	return false;
     }
 
-    return true;
-}
+    if(_overrideTimeout != r._overrideTimeout || _overrideTimeout && _timeout != r._timeout)
+    {
+	return false;
+    }
 
-bool
-IceInternal::Reference::operator!=(const Reference& r) const
-{
-    return !operator==(r);
+    return true;
 }
 
 bool
@@ -354,19 +405,50 @@ IceInternal::Reference::operator<(const Reference& r) const
 	return false;
     }
 
+    if(!_overrideTimeout && r._overrideTimeout)
+    {
+	return true;
+    }
+    else if(r._overrideTimeout < _overrideTimeout)
+    {
+	return false;
+    }
+    else if(_overrideTimeout)
+    {
+	if(_timeout < r._timeout)
+	{
+	    return true;
+	}
+	else if(r._timeout < _timeout)
+	{
+	    return false;
+	}
+    }
+
+    if(getType() < r.getType())
+    {
+	return true;
+    }
+    else if(r.getType() < getType())
+    {
+	return false;
+    }
+
     return false;
 }
 
 IceInternal::Reference::Reference(const InstancePtr& inst, const CommunicatorPtr& com, const Identity& ident,
-				  const Context& ctx, const string& fs, Mode md, bool sec) :
+				  const Context& context, const string& fs, ReferenceMode md, bool sec) :
     _hashInitialized(false),
     _instance(inst),
     _communicator(com),
     _mode(md),
     _secure(sec),
     _identity(ident),
-    _context(ctx),
-    _facet(fs)
+    _context(context),
+    _facet(fs),
+    _overrideTimeout(false),
+    _timeout(-1)
 {
 }
 
@@ -378,25 +460,41 @@ IceInternal::Reference::Reference(const Reference& r) :
     _secure(r._secure),
     _identity(r._identity),
     _context(r._context),
-    _facet(r._facet)
+    _facet(r._facet),
+    _overrideTimeout(r._overrideTimeout),
+    _timeout(r._timeout)
 {
 }
 
-void IceInternal::incRef(IceInternal::FixedReference* p) { p->__incRef(); }
-void IceInternal::decRef(IceInternal::FixedReference* p) { p->__decRef(); }
+void
+IceInternal::Reference::applyOverrides(vector<EndpointPtr>& endpts) const
+{
+    //
+    // Apply the endpoint overrides to each endpoint.
+    //
+    for(vector<EndpointPtr>::iterator p = endpts.begin(); p != endpts.end(); ++p)
+    {
+	if(_overrideTimeout)
+	{
+		*p = (*p)->timeout(_timeout);
+	}
+    }
+}
+
+IceUtil::Shared* IceInternal::upCast(IceInternal::FixedReference* p) { return p; }
 
 IceInternal::FixedReference::FixedReference(const InstancePtr& inst, const CommunicatorPtr& com, const Identity& ident,
-					    const Context& ctx, const string& fs, Mode md,
+					    const Context& context, const string& fs, ReferenceMode md, 
 					    const vector<ConnectionPtr>& fixedConns) :
-    Reference(inst, com, ident, ctx, fs, md, false),
+    Reference(inst, com, ident, context, fs, md, false),
     _fixedConnections(fixedConns)
 {
 }
 
-const vector<ConnectionPtr>&
-IceInternal::FixedReference::getFixedConnections() const
+Reference::Type
+IceInternal::FixedReference::getType() const
 {
-    return _fixedConnections;
+    return TypeFixed;
 }
 
 vector<EndpointPtr>
@@ -410,17 +508,32 @@ IceInternal::FixedReference::getEndpoints() const
 ReferencePtr
 IceInternal::FixedReference::changeRouter(const RouterPrx&) const
 {
-    return FixedReferencePtr(const_cast<FixedReference*>(this));
+    throw FixedProxyException(__FILE__, __LINE__);
+    return 0; // Keep the compiler happy.
 }
 
 #endif
 
 #ifdef ICEE_HAS_LOCATOR
 
+string
+IceInternal::FixedReference::getAdapterId() const
+{
+    return string();
+}
+
+ReferencePtr
+IceInternal::FixedReference::changeAdapterId(const std::string&) const
+{
+    throw FixedProxyException(__FILE__, __LINE__);
+    return 0; // Keep the compiler happy.
+}
+
 ReferencePtr
 IceInternal::FixedReference::changeLocator(const LocatorPrx&) const
 {
-    return FixedReferencePtr(const_cast<FixedReference*>(this));
+    throw FixedProxyException(__FILE__, __LINE__);
+    return 0; // Keep the compiler happy.
 }
 
 #endif
@@ -428,70 +541,109 @@ IceInternal::FixedReference::changeLocator(const LocatorPrx&) const
 ReferencePtr
 IceInternal::FixedReference::changeTimeout(int) const
 {
-    return FixedReferencePtr(const_cast<FixedReference*>(this));
+    throw FixedProxyException(__FILE__, __LINE__);
+    return 0; // Keep the compiler happy.
 }
 
 void
 IceInternal::FixedReference::streamWrite(BasicStream* s) const
 {
-    throw MarshalException(__FILE__, __LINE__, "Cannot marshal a fixed proxy");
+    throw FixedProxyException(__FILE__, __LINE__);
 }
 
 string
 IceInternal::FixedReference::toString() const
 {
-    throw MarshalException(__FILE__, __LINE__, "Cannot marshal a fixed proxy");
+    throw FixedProxyException(__FILE__, __LINE__);
+    return string(); // To keep the compiler from complaining.
 }
 
 ConnectionPtr
 IceInternal::FixedReference::getConnection() const
 {
+    vector<ConnectionPtr> connections = _fixedConnections;
+    switch(getMode())
+    {
+        case ReferenceModeTwoway:
+        case ReferenceModeOneway:
+#ifdef ICEE_HAS_BATCH
+        case ReferenceModeBatchOneway:
+#endif
+        {
+            //
+            // Filter out datagram connections.
+            //
+            connections.erase(remove_if(connections.begin(), connections.end(), ConnectionIsDatagram()),
+                              connections.end());
+            break;
+        }
+        
+        case ReferenceModeDatagram:
+#ifdef ICEE_HAS_BATCH
+        case ReferenceModeBatchDatagram:
+#endif
+        {
+            //
+            // Filter out non-datagram connections.
+            //
+            connections.erase(remove_if(connections.begin(), connections.end(), not1(ConnectionIsDatagram())),
+                              connections.end());
+            break;
+        }
+
+#ifndef ICEE_HAS_BATCH
+        case ReferenceModeBatchDatagram:
+        case ReferenceModeBatchOneway:
+	{
+	    throw FeatureNotSupportedException(__FILE__, __LINE__, "batch proxy mode");
+	}
+#endif
+    }
+    
     //
     // Randomize the order of connections.
     //
-    // If a reference is secure or the mode is datagram or batch
-    // datagram then we throw a NoEndpointException since IceE lacks
-    // this support.
-    //
-    if(getSecure() || getMode() == ModeDatagram || getMode() == ModeBatchDatagram || _fixedConnections.empty()
-#ifndef ICEE_HAS_BATCH
-       || getMode() == ModeBatchOneway
-#endif
-      )
-    {
-	if(_fixedConnections.empty())
-	{
-	    NoEndpointException ex(__FILE__, __LINE__);
-	    ex.proxy = ""; // No stringified representation for fixed proxies.
-	    throw ex;
-	}
+    random_shuffle(connections.begin(), connections.end());
 
-	FeatureNotSupportedException ex(__FILE__, __LINE__);
-	if(getSecure())
+    //
+    // If a secure connection is requested or secure overrides is set,
+    // remove all non-secure connections. Otherwise make non-secure
+    // connections preferred over secure connections by partitioning
+    // the connection vector, so that non-secure connections come
+    // first.
+    //
+    // NOTE: we don't use the stable_partition algorithm from STL to
+    // keep the code size down.
+    //
+    vector<ConnectionPtr>::iterator p = connections.begin();
+    vector<ConnectionPtr> secureConnections;
+    while(p != connections.end())
+    {
+	if((*p)->endpoint()->secure())
 	{
-	    ex.unsupportedFeature = "ssl";
+	    secureConnections.push_back(*p);
+	    p = connections.erase(p);
 	}
-	else if(getMode() == ModeDatagram)
+	else
 	{
-	    ex.unsupportedFeature = "datagram";
+	    ++p;
 	}
-	else if(getMode() == ModeBatchDatagram)
-	{
-	    ex.unsupportedFeature = "batch datagram";
-	}
-#ifndef ICEE_HAS_BATCH
-	else if(getMode() == ModeBatchOneway)
-	{
-	    ex.unsupportedFeature = "batch";
-	}
-#endif
-	throw ex;
+    }
+    if(getSecure())
+    {
+	connections.swap(secureConnections);
+    }
+    else
+    {
+	connections.insert(connections.end(), secureConnections.begin(), secureConnections.end());
+    }
+    
+    if(connections.empty())
+    {
+	throw NoEndpointException(__FILE__, __LINE__); // No stringified representation for fixed proxies.
     }
 
-    vector<ConnectionPtr> randomCons = _fixedConnections;
-    random_shuffle(randomCons.begin(), randomCons.end());
-
-    ConnectionPtr connection = randomCons[0];
+    ConnectionPtr connection = connections[0];
     assert(connection);
     connection->throwException(); // Throw in case our connection is already destroyed.
 
@@ -513,12 +665,6 @@ IceInternal::FixedReference::operator==(const Reference& r) const
     return _fixedConnections == rhs->_fixedConnections;
 }
 
-bool
-IceInternal::FixedReference::operator!=(const Reference& r) const
-{
-    return !operator==(r);
-}
-
 bool
 IceInternal::FixedReference::operator<(const Reference& r) const
 {
@@ -533,10 +679,8 @@ IceInternal::FixedReference::operator<(const Reference& r) const
     if(Reference::operator==(r))
     {
         const FixedReference* rhs = dynamic_cast<const FixedReference*>(&r);
-        if(rhs)
-        {
-            return _fixedConnections < rhs->_fixedConnections;
-        }
+        assert(rhs);
+	return _fixedConnections < rhs->_fixedConnections;
     }
     return false;
 }
@@ -554,8 +698,7 @@ IceInternal::FixedReference::FixedReference(const FixedReference& r) :
 }
 
 #ifdef ICEE_HAS_ROUTER
-void IceInternal::incRef(IceInternal::RoutableReference* p) { p->__incRef(); }
-void IceInternal::decRef(IceInternal::RoutableReference* p) { p->__decRef(); }
+IceUtil::Shared* IceInternal::upCast(IceInternal::RoutableReference* p) { return p; }
 
 vector<EndpointPtr>
 IceInternal::RoutableReference::getRoutedEndpoints() const
@@ -566,8 +709,7 @@ IceInternal::RoutableReference::getRoutedEndpoints() const
 	// If we route, we send everything to the router's client
 	// proxy endpoints.
 	//
-	ObjectPrx clientProxy = _routerInfo->getClientProxy();
-	return clientProxy->__reference()->getEndpoints();
+	return _routerInfo->getClientEndpoints();
     }
     return vector<EndpointPtr>();
 }
@@ -606,12 +748,6 @@ IceInternal::RoutableReference::operator==(const Reference& r) const
     return _routerInfo == rhs->_routerInfo;
 }
 
-bool
-IceInternal::RoutableReference::operator!=(const Reference& r) const
-{
-    return !operator==(r);
-}
-
 bool
 IceInternal::RoutableReference::operator<(const Reference& r) const
 {
@@ -626,18 +762,16 @@ IceInternal::RoutableReference::operator<(const Reference& r) const
     if(Reference::operator==(r))
     {
         const RoutableReference* rhs = dynamic_cast<const RoutableReference*>(&r);
-        if(rhs)
-        {
-            return _routerInfo < rhs->_routerInfo;
-        }
+        assert(rhs);
+	return _routerInfo < rhs->_routerInfo;
     }
     return false;
 }
 
 IceInternal::RoutableReference::RoutableReference(const InstancePtr& inst, const CommunicatorPtr& com,
-						  const Identity& ident, const Context& ctx, const string& fs,
-						  Mode md, bool sec, const RouterInfoPtr& rtrInfo) :
-    Reference(inst, com, ident, ctx, fs, md, sec), _routerInfo(rtrInfo)
+						  const Identity& ident, const Context& context, const string& fs,
+						  ReferenceMode md, bool sec, const RouterInfoPtr& rtrInfo) :
+    Reference(inst, com, ident, context, fs, md, sec), _routerInfo(rtrInfo)
 {
 }
 
@@ -647,68 +781,73 @@ IceInternal::RoutableReference::RoutableReference(const RoutableReference& r) :
 }
 #endif
 
-void IceInternal::incRef(IceInternal::DirectReference* p) { p->__incRef(); }
-void IceInternal::decRef(IceInternal::DirectReference* p) { p->__decRef(); }
+IceUtil::Shared* IceInternal::upCast(IceInternal::DirectReference* p) { return p; }
 
 
 #ifdef ICEE_HAS_ROUTER
 IceInternal::DirectReference::DirectReference(const InstancePtr& inst, const CommunicatorPtr& com,
-					      const Identity& ident, const Context& ctx, const string& fs, Mode md,
+					      const Identity& ident, const Context& context, const string& fs, ReferenceMode md,
 					      bool sec, const vector<EndpointPtr>& endpts,
 					      const RouterInfoPtr& rtrInfo) :
 
-    RoutableReference(inst, com, ident, ctx, fs, md, sec, rtrInfo),
+    RoutableReference(inst, com, ident, context, fs, md, sec, rtrInfo),
     _endpoints(endpts)
 {
 }
 #else
 IceInternal::DirectReference::DirectReference(const InstancePtr& inst, const CommunicatorPtr& com,
-					      const Identity& ident, const Context& ctx, const string& fs, Mode md,
+					      const Identity& ident, const Context& context, const string& fs, ReferenceMode md,
 					      bool sec, const vector<EndpointPtr>& endpts) :
-    Reference(inst, com, ident, ctx, fs, md, sec),
+    Reference(inst, com, ident, context, fs, md, sec),
     _endpoints(endpts)
 {
 }
 #endif
 
+Reference::Type
+IceInternal::DirectReference::getType() const
+{
+    return TypeDirect;
+}
+
 vector<EndpointPtr>
 IceInternal::DirectReference::getEndpoints() const
 {
     return _endpoints;
 }
 
-DirectReferencePtr
-IceInternal::DirectReference::changeEndpoints(const vector<EndpointPtr>& newEndpoints) const
+#ifdef ICEE_HAS_LOCATOR
+
+string
+IceInternal::DirectReference::getAdapterId() const
 {
-    if(newEndpoints == _endpoints)
-    {
-	return DirectReferencePtr(const_cast<DirectReference*>(this));
-    }
-    DirectReferencePtr r = DirectReferencePtr::dynamicCast(getInstance()->referenceFactory()->copy(this));
-    r->_endpoints = newEndpoints;
-    return r;
+    return string();
 }
 
-#ifdef ICEE_HAS_LOCATOR
+ReferencePtr
+DirectReference::changeAdapterId(const string& newAdapterId) const
+{
+    if(!newAdapterId.empty())
+    {
+        LocatorInfoPtr locatorInfo = 
+            getInstance()->locatorManager()->get(getInstance()->referenceFactory()->getDefaultLocator());
+        return getInstance()->referenceFactory()->create(getIdentity(), *getContext(), getFacet(), getMode(),
+                                                         getSecure(), newAdapterId,
+#ifdef ICEE_HAS_ROUTER
+							 getRouterInfo(),
+#endif
+                                                         locatorInfo);
+    }
+    else
+    {
+        return DirectReferencePtr(const_cast<DirectReference*>(this));
+    }
+}
 
 ReferencePtr
 IceInternal::DirectReference::changeLocator(const LocatorPrx& newLocator) const
 {
-    if(newLocator)
-    {
-	LocatorInfoPtr newLocatorInfo = getInstance()->locatorManager()->get(newLocator);
-#ifdef ICEE_HAS_ROUTER
-	return getInstance()->referenceFactory()->create(getIdentity(), getContext(), getFacet(), getMode(),
-							 getSecure(), "", 0, newLocatorInfo);
-#else
-	return getInstance()->referenceFactory()->create(getIdentity(), getContext(), getFacet(), getMode(),
-							 getSecure(), "", newLocatorInfo);
-#endif
-    }
-    else
-    {
-	return DirectReferencePtr(const_cast<DirectReference*>(this));
-    }
+    return DirectReferencePtr(const_cast<DirectReference*>(this));
 }
 
 #endif
@@ -716,14 +855,16 @@ IceInternal::DirectReference::changeLocator(const LocatorPrx& newLocator) const
 ReferencePtr
 IceInternal::DirectReference::changeTimeout(int newTimeout) const
 {
-    DirectReferencePtr r = DirectReferencePtr::dynamicCast(getInstance()->referenceFactory()->copy(this));
-    vector<EndpointPtr> newEndpoints;
-    vector<EndpointPtr>::const_iterator p;
-    for(p = _endpoints.begin(); p != _endpoints.end(); ++p)
+    DirectReferencePtr r = DirectReferencePtr::dynamicCast(Parent::changeTimeout(newTimeout));
+    if(r.get() != this) // Also override the timeout on the endpoints if it was updated.
     {
-	newEndpoints.push_back((*p)->timeout(newTimeout));
+	vector<EndpointPtr> newEndpoints;
+	for(vector<EndpointPtr>::const_iterator p = _endpoints.begin(); p != _endpoints.end(); ++p)
+	{
+	    newEndpoints.push_back((*p)->timeout(newTimeout));
+	}
+	r->_endpoints = newEndpoints;
     }
-    r->_endpoints = newEndpoints;
     return r;
 }
 
@@ -770,9 +911,11 @@ IceInternal::DirectReference::getConnection() const
 {
 #ifdef ICEE_HAS_ROUTER
     vector<EndpointPtr> endpts = Parent::getRoutedEndpoints();
+    applyOverrides(endpts);
+
     if(endpts.empty())
     {
-	endpts = _endpoints;
+	endpts = _endpoints; // Endpoint overrides are already applied on these endpoints.
     }
 #else
     vector<EndpointPtr> endpts = _endpoints;
@@ -780,9 +923,7 @@ IceInternal::DirectReference::getConnection() const
     vector<EndpointPtr> filteredEndpoints = filterEndpoints(endpts, getMode(), getSecure());
     if(filteredEndpoints.empty())
     {
-        NoEndpointException ex(__FILE__, __LINE__);
-	ex.proxy = toString();
-	throw ex;
+        throw NoEndpointException(__FILE__, __LINE__, toString());
     }
 
     OutgoingConnectionFactoryPtr factory = getInstance()->outgoingConnectionFactory();
@@ -820,12 +961,6 @@ IceInternal::DirectReference::operator==(const Reference& r) const
     return _endpoints == rhs->_endpoints;
 }
 
-bool
-IceInternal::DirectReference::operator!=(const Reference& r) const
-{
-    return !operator==(r);
-}
-
 bool
 IceInternal::DirectReference::operator<(const Reference& r) const
 {
@@ -840,10 +975,8 @@ IceInternal::DirectReference::operator<(const Reference& r) const
     if(Parent::operator==(r))
     {
         const DirectReference* rhs = dynamic_cast<const DirectReference*>(&r);
-        if(rhs)
-        {
-            return _endpoints < rhs->_endpoints;
-        }
+	assert(rhs);
+	return _endpoints < rhs->_endpoints;
     }
     return false;
 }
@@ -861,75 +994,70 @@ IceInternal::DirectReference::DirectReference(const DirectReference& r) :
 
 #ifdef ICEE_HAS_LOCATOR
 
-void IceInternal::incRef(IceInternal::IndirectReference* p) { p->__incRef(); }
-void IceInternal::decRef(IceInternal::IndirectReference* p) { p->__decRef(); }
+IceUtil::Shared* IceInternal::upCast(IceInternal::IndirectReference* p) { return p; }
 
 #ifdef ICEE_HAS_ROUTER
 IceInternal::IndirectReference::IndirectReference(const InstancePtr& inst, const CommunicatorPtr& com,
-						  const Identity& ident, const Context& ctx, const string& fs,
-						  Mode md, bool sec, const string& adptid,
+						  const Identity& ident, const Context& context, const string& fs,
+						  ReferenceMode md, bool sec, const string& adptid,
 						  const RouterInfoPtr& rtrInfo, const LocatorInfoPtr& locInfo) :
-    RoutableReference(inst, com, ident, ctx, fs, md, sec, rtrInfo),
+    RoutableReference(inst, com, ident, context, fs, md, sec, rtrInfo),
     _adapterId(adptid),
     _locatorInfo(locInfo)
 {
 }
 #else
 IceInternal::IndirectReference::IndirectReference(const InstancePtr& inst, const CommunicatorPtr& com, 
-						  const Identity& ident, const Context& ctx, const string& fs,
-						  Mode md, bool sec, const string& adptid,
+						  const Identity& ident, const Context& context, const string& fs,
+						  ReferenceMode md, bool sec, const string& adptid,
 						  const LocatorInfoPtr& locInfo) :
-    Reference(inst, com, ident, ctx, fs, md, sec),
+    Reference(inst, com, ident, context, fs, md, sec),
     _adapterId(adptid),
     _locatorInfo(locInfo)
 {
 }
 #endif
 
+Reference::Type
+IceInternal::IndirectReference::getType() const
+{
+    return TypeIndirect;
+}
+
 vector<EndpointPtr>
 IceInternal::IndirectReference::getEndpoints() const
 {
     return vector<EndpointPtr>();
 }
 
-ReferencePtr
-IceInternal::IndirectReference::changeLocator(const LocatorPrx& newLocator) const
+string
+IceInternal::IndirectReference::getAdapterId() const
 {
-    //
-    // Return a direct reference if a null locator is given.
-    //
-    if(!newLocator)
-    {
-#ifdef ICEE_HAS_ROUTER
-	return getInstance()->referenceFactory()->create(getIdentity(), getContext(), getFacet(), getMode(),
-							 getSecure(), vector<EndpointPtr>(), getRouterInfo());
-#else
-	return getInstance()->referenceFactory()->create(getIdentity(), getContext(), getFacet(), getMode(),
-							 getSecure(), vector<EndpointPtr>());
-#endif
-    }
-    else
-    {
-	LocatorInfoPtr newLocatorInfo = getInstance()->locatorManager()->get(newLocator);
-	if(newLocatorInfo == _locatorInfo)
-	{
-	    return IndirectReferencePtr(const_cast<IndirectReference*>(this));
-	}
-	IndirectReferencePtr r = IndirectReferencePtr::dynamicCast(getInstance()->referenceFactory()->copy(this));
-	r->_locatorInfo = newLocatorInfo;
-	return r;
-    }
+    return _adapterId;
 }
 
 ReferencePtr
-IceInternal::IndirectReference::changeTimeout(int newTimeout) const
+IceInternal::IndirectReference::changeAdapterId(const string& newAdapterId) const
 {
-    IndirectReferencePtr r = IndirectReferencePtr::dynamicCast(getInstance()->referenceFactory()->copy(this));
-    if(_locatorInfo)
+    if(newAdapterId == _adapterId)
     {
-	LocatorPrx newLocator = LocatorPrx::uncheckedCast(_locatorInfo->getLocator()->ice_timeout(newTimeout));
-	r->_locatorInfo = getInstance()->locatorManager()->get(newLocator);
+        return IndirectReferencePtr(const_cast<IndirectReference*>(this));
     }
+    IndirectReferencePtr r = IndirectReferencePtr::dynamicCast(getInstance()->referenceFactory()->copy(this));
+    r->_adapterId = newAdapterId;
+    return r;
+}
+
+ReferencePtr
+IceInternal::IndirectReference::changeLocator(const LocatorPrx& newLocator) const
+{
+    LocatorInfoPtr newLocatorInfo = getInstance()->locatorManager()->get(newLocator);
+    if(newLocatorInfo == _locatorInfo)
+    {
+	return IndirectReferencePtr(const_cast<IndirectReference*>(this));
+    }
+    IndirectReferencePtr r = IndirectReferencePtr::dynamicCast(getInstance()->referenceFactory()->copy(this));
+    r->_locatorInfo = newLocatorInfo;
     return r;
 }
 
@@ -958,8 +1086,17 @@ IceInternal::IndirectReference::toString() const
     // reference parser uses as separators, then we enclose the
     // adapter id string in quotes.
     //
-    string a = IceUtil::escapeString(_adapterId, "");
-    if(a.find_first_of(" \t\n\r") != string::npos)
+    string a = _adapterId;
+#ifdef ICEE_HAS_WSTRING
+    if(getInstance()->initializationData().stringConverter)
+    {
+        UTF8BufferI buffer;
+        Byte* last = getInstance()->initializationData().stringConverter->toUTF8(a.data(), a.data() + a.size(), buffer);
+        a = string(reinterpret_cast<const char*>(buffer.getBuffer()), last - buffer.getBuffer());
+    }
+#endif
+    a = IceUtil::escapeString(a, "");
+    if(a.find_first_of(" ") != string::npos)
     {
 	result.append("\"");
 	result.append(a);
@@ -990,12 +1127,13 @@ IceInternal::IndirectReference::getConnection() const
 	    const IndirectReferencePtr self = const_cast<IndirectReference*>(this);
 	    endpts = _locatorInfo->getEndpoints(self, cached);
 	}
+
+	applyOverrides(endpts);
+
 	vector<EndpointPtr> filteredEndpoints = filterEndpoints(endpts, getMode(), getSecure());
 	if(filteredEndpoints.empty())
 	{
-	    NoEndpointException ex(__FILE__, __LINE__);
-	    ex.proxy = toString();
-	    throw ex;
+	    throw NoEndpointException(__FILE__, __LINE__, toString());
 	}
 
 	try
@@ -1017,10 +1155,9 @@ IceInternal::IndirectReference::getConnection() const
 		if(cached)
 		{
 		    TraceLevelsPtr traceLevels = getInstance()->traceLevels();
-		    LoggerPtr logger = getInstance()->logger();
 		    if(traceLevels->retry >= 2)
 		    {
-		        Trace out(logger, traceLevels->retryCat);
+		        Trace out(getInstance()->initializationData().logger, traceLevels->retryCat);
 			out << "connection to cached endpoints failed\n"
 			    << "removing endpoints from cache and trying one more time\n" << ex.toString();
 		    }
@@ -1087,12 +1224,6 @@ IceInternal::IndirectReference::operator==(const Reference& r) const
     return _adapterId == rhs->_adapterId && _locatorInfo == rhs->_locatorInfo;
 }
 
-bool
-IceInternal::IndirectReference::operator!=(const Reference& r) const
-{
-    return !operator==(r);
-}
-
 bool
 IceInternal::IndirectReference::operator<(const Reference& r) const
 {
@@ -1107,18 +1238,16 @@ IceInternal::IndirectReference::operator<(const Reference& r) const
     if(Parent::operator==(r))
     {
         const IndirectReference* rhs = dynamic_cast<const IndirectReference*>(&r);
-        if(rhs)
-        {
-            if(_adapterId < rhs->_adapterId)
-            {
-                return true;
-            }
-            else if(rhs->_adapterId < _adapterId)
-            {
-                return false;
-            }
-            return _locatorInfo < rhs->_locatorInfo;
-        }
+	assert(rhs);
+	if(_adapterId < rhs->_adapterId)
+	{
+	    return true;
+	}
+	else if(rhs->_adapterId < _adapterId)
+	{
+	    return false;
+	}
+	return _locatorInfo < rhs->_locatorInfo;
     }
     return false;
 }
@@ -1130,62 +1259,104 @@ IceInternal::IndirectReference::clone() const
 }
 
 IceInternal::IndirectReference::IndirectReference(const IndirectReference& r)
-    : Parent(r), _adapterId(r._adapterId), _locatorInfo(r._locatorInfo)
+    : Parent(r), 
+      _adapterId(r._adapterId), 
+      _locatorInfo(r._locatorInfo)
 {
 }
 
 #endif // ICEE_HAS_LOCATOR
 
 vector<EndpointPtr>
-IceInternal::filterEndpoints(const vector<EndpointPtr>& allEndpoints, Reference::Mode m, bool sec)
+IceInternal::filterEndpoints(const vector<EndpointPtr>& allEndpoints, ReferenceMode m, bool sec)
 {
-    vector<EndpointPtr> endpoints;
-
-    //
-    // If a secure endpoint, batch (if batch is not supported),
-    // datagram or batch datagram endpoint is requested since IceE
-    // lacks this support we throw an unsupported feature.
-    //
-    if(sec || m == Reference::ModeDatagram || m == Reference::ModeBatchDatagram
-#ifndef ICEE_HAS_BATCH
-       || m == Reference::ModeBatchOneway
-#endif
-	)
-    {
-	FeatureNotSupportedException ex(__FILE__, __LINE__);
-	if(sec)
-	{
-	    ex.unsupportedFeature = "ssl";
-	}
-	else if(m == Reference::ModeDatagram)
-	{
-	    ex.unsupportedFeature = "datagram";
-	}
-	else if(m == Reference::ModeBatchDatagram)
-	{
-	    ex.unsupportedFeature = "batch datagram";
-	}
-#ifndef ICEE_HAS_BATCH
-	else if(m == Reference::ModeBatchOneway)
-	{
-	    ex.unsupportedFeature = "batch";
-	}
-#endif
-	throw ex;
-    }
-
-    endpoints = allEndpoints;
+    vector<EndpointPtr> endpoints = allEndpoints;
 
     //
     // Filter out unknown endpoints.
     //
     endpoints.erase(remove_if(endpoints.begin(), endpoints.end(), Ice::constMemFun(&Endpoint::unknown)),
                     endpoints.end());
-    
+
+    //
+    // Filter out endpoints according to the mode of the reference.
+    //
+    switch(m)
+    {
+        case ReferenceModeTwoway:
+        case ReferenceModeOneway:
+#ifdef ICEE_HAS_BATCH
+        case ReferenceModeBatchOneway:
+#endif
+        {
+            //
+            // Filter out datagram endpoints.
+            //
+            endpoints.erase(remove_if(endpoints.begin(), endpoints.end(), Ice::constMemFun(&Endpoint::datagram)),
+                            endpoints.end());
+            break;
+        }
+        
+        case ReferenceModeDatagram:
+#ifdef ICEE_HAS_BATCH
+        case ReferenceModeBatchDatagram:
+#endif
+        {
+            //
+            // Filter out non-datagram endpoints.
+            //
+	    endpoints.erase(remove_if(endpoints.begin(), endpoints.end(),
+				      not1(Ice::constMemFun(&Endpoint::datagram))),
+			    endpoints.end());
+            break;
+        }
+
+#ifndef ICEE_HAS_BATCH
+        case ReferenceModeBatchDatagram:
+        case ReferenceModeBatchOneway:
+	{
+	    throw FeatureNotSupportedException(__FILE__, __LINE__, "batch proxy mode");
+	}
+#endif
+    }
+
     //
     // Randomize the order of endpoints.
     //
     random_shuffle(endpoints.begin(), endpoints.end());
     
+    //
+    // If a secure connection is requested or secure overrides is set,
+    // remove all non-secure endpoints. Otherwise make non-secure
+    // endpoints preferred over secure endpoints by partitioning
+    // the endpoint vector, so that non-secure endpoints come
+    // first.
+    //
+    // NOTE: we don't use the stable_partition algorithm from STL to
+    // keep the code size down.
+    //
+    vector<EndpointPtr>::iterator p = endpoints.begin();
+    vector<EndpointPtr> secureEndpoints;
+    while(p != endpoints.end())
+    {
+	if((*p)->secure())
+	{
+	    secureEndpoints.push_back(*p);
+	    p = endpoints.erase(p);
+	}
+	else
+	{
+	    ++p;
+	}
+    }
+    if(sec)
+    {
+	endpoints.swap(secureEndpoints);
+    }
+    else
+    {
+	endpoints.insert(endpoints.end(), secureEndpoints.begin(), secureEndpoints.end());
+    }
+
     return endpoints;
 }
