@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -12,10 +12,10 @@
 
 #include <IceE/ServantManagerF.h>
 #include <IceE/InstanceF.h>
+#include <IceE/ObjectAdapter.h>
 #include <IceE/Shared.h>
 #include <IceE/Mutex.h>
 #include <IceE/Identity.h>
-#include <IceE/FacetMap.h>
 
 namespace IceInternal
 {
