@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -26,6 +26,8 @@ public:
     virtual Ice::Short type() const;
     virtual Ice::Int timeout() const;
     virtual EndpointPtr timeout(Ice::Int) const;
+    virtual bool secure() const;
+    virtual bool datagram() const;
     virtual bool unknown() const;
     virtual ConnectorPtr connector() const;
 #ifndef ICEE_PURE_CLIENT
