@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -31,9 +31,3 @@ TestI::deactivate(const Current& current)
     current.adapter->deactivate();
     IceUtil::ThreadControl::sleep(IceUtil::Time::seconds(1));
 }
-
-string
-CookieI::message() const
-{
-    return "blahblah";
-}
