@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -20,11 +20,4 @@ public:
     virtual void deactivate(const Ice::Current&);
 };
 
-class CookieI : public Test::Cookie
-{
-public:
-
-    virtual std::string message() const;
-};
-
 #endif
