@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -11,6 +11,7 @@
 #include <TestCommon.h>
 #include <TestApplication.h>
 #include <Test.h>
+#include <StringConverterI.h>
 
 using namespace std;
 
@@ -26,10 +27,16 @@ public:
     virtual int
     run(int argc, char* argv[])
     {
-	Ice::PropertiesPtr properties = Ice::createProperties();
+        Ice::InitializationData initData;
+	initData.properties = Ice::createProperties();
+#ifdef ICEE_HAS_WSTRING
+        initData.stringConverter = new Test::StringConverterI();
+        initData.wstringConverter = new Test::WstringConverterI();
+#endif
 
-	loadConfig(properties);
-	setCommunicator(Ice::initializeWithProperties(argc, argv, properties));
+	loadConfig(initData.properties);
+	initData.logger = getLogger();
+	setCommunicator(Ice::initialize(argc, argv, initData));
 
         Test::TestIntfPrx allTests(const Ice::CommunicatorPtr&);
         Test::TestIntfPrx test = allTests(communicator());
