@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -26,9 +26,11 @@ public:
     virtual int
     run(int argc, char* argv[])
     {
-        Ice::PropertiesPtr properties = Ice::createProperties();
-        loadConfig(properties);
-        setCommunicator(Ice::initializeWithProperties(argc, argv, properties));
+        Ice::InitializationData initData;
+        initData.properties = Ice::createProperties();
+        loadConfig(initData.properties);
+	initData.logger = getLogger();
+        setCommunicator(Ice::initialize(argc, argv, initData));
 
         GPrx allTests(const Ice::CommunicatorPtr&);
         GPrx g = allTests(communicator());
