@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -54,7 +54,7 @@ batchOneways(const Test::MyClassPrx& p)
     
     int i;
 
-    for(i = 0 ; i < 9 ; ++i)
+    for(i = 0 ; i < 30 ; ++i)
     {
 	try
 	{
@@ -65,35 +65,7 @@ batchOneways(const Test::MyClassPrx& p)
 	{
 	    test(false);
 	}
-	
-	batch->ice_connection()->flushBatchRequests();
     }
     
-    for(i = 0 ; i < 10 ; ++i)
-    {
-	try
-	{
-	    batch->opByteSOneway(bs1);
-	    test(i < 9);
-	}
-	catch(const Ice::MemoryLimitException&)
-	{
-	    test(i == 9);
-	}
-    }
-    
-    for(i = 0 ; i < 9 ; ++i)
-    {
-	try
-	{
-	    batch->opByteSOneway(bs1);
-	    test(true);
-	}
-	catch(const Ice::MemoryLimitException&)
-	{
-	    test(false);
-	}
-	
-	batch->ice_connection()->flushBatchRequests();
-    }
+    batch->ice_getConnection()->flushBatchRequests();
 }
