@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice-E is licensed to you under the terms described in the
 // ICEE_LICENSE file included in this distribution.
@@ -10,7 +10,6 @@
 #include <TestSuite.h>
 #include <MutexTest.h>
 #include <RecMutexTest.h>
-#include <RWRecMutexTest.h>
 #include <StaticMutexTest.h>
 #include <CreateTest.h>
 #include <AliveTest.h>
@@ -28,7 +27,6 @@ initializeTestSuite()
     allTests.push_back(new CreateTest);
     allTests.push_back(new AliveTest);
     allTests.push_back(new RecMutexTest);
-    allTests.push_back(new RWRecMutexTest);
     allTests.push_back(new StaticMutexTest);
     allTests.push_back(new MonitorMutexTest);
     allTests.push_back(new MonitorRecMutexTest);
